searchData={"items":[{"type":"module","title":"Gemini.APIs.Models","doc":"Complete Models API implementation following the unified architecture.\n\nProvides comprehensive access to Gemini model information including:\n- Listing available models with pagination\n- Getting detailed model information\n- Querying model capabilities and features\n- Statistical analysis and filtering","ref":"Gemini.APIs.Models.html"},{"type":"module","title":"Examples - Gemini.APIs.Models","doc":"# List all available models\n    {:ok, response} = Models.list()\n    models = response.models\n\n    # Get specific model information\n    {:ok, model} = Models.get(\"gemini-2.0-flash\")\n\n    # Check if model exists\n    {:ok, true} = Models.exists?(\"gemini-pro\")\n\n    # Filter models by capabilities\n    {:ok, streaming_models} = Models.supporting_method(\"streamGenerateContent\")","ref":"Gemini.APIs.Models.html#module-examples"},{"type":"function","title":"Gemini.APIs.Models.exists?/1","doc":"Check if a specific model exists and is available.","ref":"Gemini.APIs.Models.html#exists?/1"},{"type":"function","title":"Parameters - Gemini.APIs.Models.exists?/1","doc":"- `model_name` - The model identifier to check","ref":"Gemini.APIs.Models.html#exists?/1-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Models.exists?/1","doc":"- `{:ok, true}` - Model exists and is available\n- `{:ok, false}` - Model does not exist\n- `{:error, Error.t()}` - Network or other API error","ref":"Gemini.APIs.Models.html#exists?/1-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.exists?/1","doc":"{:ok, true} = Models.exists?(\"gemini-2.0-flash\")\n    {:ok, false} = Models.exists?(\"nonexistent-model\")\n\n    # Use in conditional logic\n    case Models.exists?(\"gemini-pro\") do\n      {:ok, true} -> generate_with_model(\"gemini-pro\")\n      {:ok, false} -> use_fallback_model()\n      {:error, _} -> handle_api_error()\n    end","ref":"Gemini.APIs.Models.html#exists?/1-examples"},{"type":"function","title":"Gemini.APIs.Models.filter/1","doc":"Get models filtered by capabilities or parameters.","ref":"Gemini.APIs.Models.html#filter/1"},{"type":"function","title":"Parameters - Gemini.APIs.Models.filter/1","doc":"- `filter_opts` - Keyword list of filter criteria:\n  - `:min_input_tokens` - Minimum input token limit\n  - `:min_output_tokens` - Minimum output token limit\n  - `:supports_methods` - List of required methods\n  - `:has_temperature` - Boolean, requires temperature parameter\n  - `:has_top_k` - Boolean, requires top_k parameter\n  - `:has_top_p` - Boolean, requires top_p parameter\n  - `:production_ready` - Boolean, filter for production-ready models\n  - `:model_family` - String, filter by model family (e.g., \"gemini\")","ref":"Gemini.APIs.Models.html#filter/1-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Models.filter/1","doc":"- `{:ok, [Model.t()]}` - Filtered list of models\n- `{:error, Error.t()}` - API error","ref":"Gemini.APIs.Models.html#filter/1-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.filter/1","doc":"# High-capacity models\n    {:ok, large_models} = Models.filter(min_input_tokens: 100_000)\n\n    # Models with advanced parameters\n    {:ok, tunable_models} = Models.filter(has_temperature: true, has_top_k: true)\n\n    # Multi-method support\n    {:ok, versatile_models} = Models.filter(\n      supports_methods: [\"generateContent\", \"streamGenerateContent\"]\n    )\n\n    # Production-ready models only\n    {:ok, production_models} = Models.filter(production_ready: true)","ref":"Gemini.APIs.Models.html#filter/1-examples"},{"type":"function","title":"Gemini.APIs.Models.get/1","doc":"Get detailed information about a specific model.","ref":"Gemini.APIs.Models.html#get/1"},{"type":"function","title":"Parameters - Gemini.APIs.Models.get/1","doc":"- `model_name` - The model identifier, with or without \"models/\" prefix\n  Examples: \"gemini-2.0-flash\", \"models/gemini-1.5-pro\"","ref":"Gemini.APIs.Models.html#get/1-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Models.get/1","doc":"- `{:ok, Model.t()}` - Success with model details\n- `{:error, Error.t()}` - Model not found, validation error, or API error","ref":"Gemini.APIs.Models.html#get/1-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.get/1","doc":"# Get model by base ID\n    {:ok, model} = Models.get(\"gemini-2.0-flash\")\n\n    # Get model by full resource name\n    {:ok, model} = Models.get(\"models/gemini-1.5-pro\")\n\n    # Handle not found cases properly in your application code","ref":"Gemini.APIs.Models.html#get/1-examples"},{"type":"function","title":"API Reference - Gemini.APIs.Models.get/1","doc":"Corresponds to: `GET https://generativelanguage.googleapis.com/v1beta/{name=models/*}`","ref":"Gemini.APIs.Models.html#get/1-api-reference"},{"type":"function","title":"Gemini.APIs.Models.get_stats/0","doc":"Get comprehensive model statistics and summary.","ref":"Gemini.APIs.Models.html#get_stats/0"},{"type":"function","title":"Returns - Gemini.APIs.Models.get_stats/0","doc":"- `{:ok, map()}` - Statistics about available models\n- `{:error, Error.t()}` - API error","ref":"Gemini.APIs.Models.html#get_stats/0-returns"},{"type":"function","title":"Example Response - Gemini.APIs.Models.get_stats/0","doc":"{:ok, stats} = Models.get_stats()\n    # =>\n    # %{\n    #   total_models: 5,\n    #   by_version: %{\"1.5\" => 3, \"2.0\" => 2},\n    #   by_method: %{\n    #     \"generateContent\" => 5,\n    #     \"streamGenerateContent\" => 4,\n    #     \"countTokens\" => 5\n    #   },\n    #   token_limits: %{\n    #     max_input: 2_000_000,\n    #     max_output: 8192,\n    #     avg_input: 800_000,\n    #     avg_output: 4096\n    #   },\n    #   capabilities: %{\n    #     with_temperature: 5,\n    #     with_top_k: 3,\n    #     with_top_p: 5,\n    #     production_ready: 4\n    #   },\n    #   by_family: %{\"gemini\" => 4, \"text\" => 1}\n    # }","ref":"Gemini.APIs.Models.html#get_stats/0-example-response"},{"type":"function","title":"Gemini.APIs.Models.list/1","doc":"List available Gemini models with optional pagination.","ref":"Gemini.APIs.Models.html#list/1"},{"type":"function","title":"Parameters - Gemini.APIs.Models.list/1","doc":"- `opts` - Keyword list of options:\n  - `:page_size` - Maximum number of models per page (1-1000, default: 50)\n  - `:page_token` - Token for retrieving the next page of results","ref":"Gemini.APIs.Models.html#list/1-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Models.list/1","doc":"- `{:ok, ListModelsResponse.t()}` - Success with models and pagination info\n- `{:error, Error.t()}` - Validation error, API error, or network failure","ref":"Gemini.APIs.Models.html#list/1-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.list/1","doc":"# List first 50 models (default)\n    {:ok, response} = Models.list()\n    models = response.models\n    next_token = response.next_page_token\n\n    # Custom page size\n    {:ok, response} = Models.list(page_size: 10)\n\n    # Pagination\n    {:ok, page1} = Models.list(page_size: 10)\n    {:ok, page2} = Models.list(page_size: 10, page_token: page1.next_page_token)","ref":"Gemini.APIs.Models.html#list/1-examples"},{"type":"function","title":"API Reference - Gemini.APIs.Models.list/1","doc":"Corresponds to: `GET https://generativelanguage.googleapis.com/v1beta/models`","ref":"Gemini.APIs.Models.html#list/1-api-reference"},{"type":"function","title":"Gemini.APIs.Models.list_names/0","doc":"List all available model names as simple strings.\n\nThis is a convenience function that extracts just the base model IDs\nfrom the full models list response.","ref":"Gemini.APIs.Models.html#list_names/0"},{"type":"function","title":"Returns - Gemini.APIs.Models.list_names/0","doc":"- `{:ok, [String.t()]}` - List of base model IDs\n- `{:error, Error.t()}` - API error","ref":"Gemini.APIs.Models.html#list_names/0-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.list_names/0","doc":"{:ok, names} = Models.list_names()\n    # => [\"gemini-2.0-flash\", \"gemini-1.5-pro\", \"gemini-1.5-flash\"]\n\n    # Use with enum functions\n    {:ok, names} = Models.list_names()\n    flash_models = Enum.filter(names, &String.contains?(&1, \"flash\"))","ref":"Gemini.APIs.Models.html#list_names/0-examples"},{"type":"function","title":"Gemini.APIs.Models.supporting_method/1","doc":"Get models that support a specific generation method.","ref":"Gemini.APIs.Models.html#supporting_method/1"},{"type":"function","title":"Parameters - Gemini.APIs.Models.supporting_method/1","doc":"- `method` - The generation method to filter by\n  Examples: \"generateContent\", \"streamGenerateContent\", \"countTokens\"","ref":"Gemini.APIs.Models.html#supporting_method/1-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Models.supporting_method/1","doc":"- `{:ok, [Model.t()]}` - List of models supporting the method\n- `{:error, Error.t()}` - API error","ref":"Gemini.APIs.Models.html#supporting_method/1-returns"},{"type":"function","title":"Examples - Gemini.APIs.Models.supporting_method/1","doc":"# Find streaming-capable models\n    {:ok, streaming_models} = Models.supporting_method(\"streamGenerateContent\")\n\n    # Find models that support content generation\n    {:ok, generation_models} = Models.supporting_method(\"generateContent\")\n\n    # Check capabilities\n    {:ok, models} = Models.supporting_method(\"countTokens\")\n    token_counting_available = length(models) > 0","ref":"Gemini.APIs.Models.html#supporting_method/1-examples"},{"type":"module","title":"Gemini.APIs.Tokens","doc":"Token counting functionality for Gemini API.\n\nProvides comprehensive token counting capabilities including:\n- Counting tokens for simple text content\n- Counting tokens for multimodal content (text + images)\n- Counting tokens for complete GenerateContentRequest structures\n- Batch token counting operations\n\nToken counting helps with:\n- Request planning and validation\n- Cost estimation and budgeting\n- Content chunking and splitting\n- Model selection based on token limits","ref":"Gemini.APIs.Tokens.html"},{"type":"module","title":"Examples - Gemini.APIs.Tokens","doc":"# Simple text counting\n    {:ok, response} = Tokens.count(\"Hello, world!\")\n    total_tokens = response.total_tokens\n\n    # Multimodal content counting\n    contents = [\n      Content.text(\"What's in this image?\"),\n      Content.image(\"path/to/image.jpg\")\n    ]\n    {:ok, response} = Tokens.count(contents)\n\n    # Count tokens for a complete generation request\n    {:ok, gen_request} = GenerateContentRequest.new(\"Hello\", generation_config: config)\n    {:ok, response} = Tokens.count_for_request(gen_request)\n\n    # Batch counting for multiple inputs\n    inputs = [\"Text 1\", \"Text 2\", \"Text 3\"]\n    {:ok, results} = Tokens.count_batch(inputs)","ref":"Gemini.APIs.Tokens.html#module-examples"},{"type":"function","title":"Gemini.APIs.Tokens.check_fit/3","doc":"Check if content fits within a model's token limit.","ref":"Gemini.APIs.Tokens.html#check_fit/3"},{"type":"function","title":"Parameters - Gemini.APIs.Tokens.check_fit/3","doc":"- `content` - Content to check\n- `model_name` - Model to check against (optional, uses default if not provided)\n- `opts` - Options including:\n  - `:buffer` - Safety buffer to subtract from limit (default: 100)\n  - `:include_output` - Reserve space for output tokens (default: 1000)","ref":"Gemini.APIs.Tokens.html#check_fit/3-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Tokens.check_fit/3","doc":"- `{:ok, %{fits: boolean(), tokens: integer(), limit: integer()}}` - Fit analysis\n- `{:error, Error.t()}` - Error details","ref":"Gemini.APIs.Tokens.html#check_fit/3-returns"},{"type":"function","title":"Examples - Gemini.APIs.Tokens.check_fit/3","doc":"{:ok, analysis} = Tokens.check_fit(\"Hello world\", \"gemini-2.0-flash\")\n    # => {:ok, %{fits: true, tokens: 3, limit: 1000000, remaining: 999997}}\n\n    # With output buffer\n    {:ok, analysis} = Tokens.check_fit(long_text, include_output: 2000)","ref":"Gemini.APIs.Tokens.html#check_fit/3-examples"},{"type":"function","title":"Gemini.APIs.Tokens.count/2","doc":"Count tokens in the given content.","ref":"Gemini.APIs.Tokens.html#count/2"},{"type":"function","title":"Parameters - Gemini.APIs.Tokens.count/2","doc":"- `contents` - Content to count (string, list of Content structs, or GenerateContentRequest)\n- `opts` - Options:\n  - `:model` - Model name (default: from config)\n  - `:generate_content_request` - Use full GenerateContentRequest for counting","ref":"Gemini.APIs.Tokens.html#count/2-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Tokens.count/2","doc":"- `{:ok, CountTokensResponse.t()}` - Success with token count\n- `{:error, Error.t()}` - Validation error, API error, or network failure","ref":"Gemini.APIs.Tokens.html#count/2-returns"},{"type":"function","title":"Examples - Gemini.APIs.Tokens.count/2","doc":"# Simple text\n    {:ok, response} = Tokens.count(\"Hello, world!\")\n    # => %CountTokensResponse{total_tokens: 3}\n\n    # Multiple contents\n    contents = [\n      Content.text(\"Hello\"),\n      Content.text(\"World\")\n    ]\n    {:ok, response} = Tokens.count(contents)\n\n    # With specific model\n    {:ok, response} = Tokens.count(\"Hello\", model: \"gemini-1.5-pro\")\n\n    # Using GenerateContentRequest\n    {:ok, gen_request} = GenerateContentRequest.new(\"Hello\", generation_config: config)\n    {:ok, response} = Tokens.count(gen_request)","ref":"Gemini.APIs.Tokens.html#count/2-examples"},{"type":"function","title":"Gemini.APIs.Tokens.count_batch/2","doc":"Count tokens for multiple inputs in batch.\n\nEfficiently counts tokens for multiple separate inputs, returning\nresults in the same order as the inputs.","ref":"Gemini.APIs.Tokens.html#count_batch/2"},{"type":"function","title":"Parameters - Gemini.APIs.Tokens.count_batch/2","doc":"- `inputs` - List of content inputs (strings or Content lists)\n- `opts` - Options applied to all requests","ref":"Gemini.APIs.Tokens.html#count_batch/2-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Tokens.count_batch/2","doc":"- `{:ok, [CountTokensResponse.t()]}` - List of token counts\n- `{:error, Error.t()}` - Error details","ref":"Gemini.APIs.Tokens.html#count_batch/2-returns"},{"type":"function","title":"Examples - Gemini.APIs.Tokens.count_batch/2","doc":"inputs = [\n      \"Hello world\",\n      \"How are you?\",\n      \"Tell me a story about dragons\"\n    ]\n    \n    {:ok, results} = Tokens.count_batch(inputs)\n    \n    # Process results in your application code\n\n    # With options\n    {:ok, results} = Tokens.count_batch(inputs, model: \"gemini-1.5-pro\")","ref":"Gemini.APIs.Tokens.html#count_batch/2-examples"},{"type":"function","title":"Gemini.APIs.Tokens.count_for_request/2","doc":"Count tokens for a GenerateContentRequest.\n\nThis is useful when you want to know the token count for a complete\ngeneration request including all parameters, system instructions, etc.","ref":"Gemini.APIs.Tokens.html#count_for_request/2"},{"type":"function","title":"Parameters - Gemini.APIs.Tokens.count_for_request/2","doc":"- `generate_request` - A GenerateContentRequest struct\n- `opts` - Options (model, etc.)","ref":"Gemini.APIs.Tokens.html#count_for_request/2-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Tokens.count_for_request/2","doc":"- `{:ok, CountTokensResponse.t()}` - Token count for the complete request\n- `{:error, Error.t()}` - Error details","ref":"Gemini.APIs.Tokens.html#count_for_request/2-returns"},{"type":"function","title":"Examples - Gemini.APIs.Tokens.count_for_request/2","doc":"# Create a generation request\n    {:ok, gen_request} = GenerateContentRequest.new(\n      \"Explain quantum physics\",\n      generation_config: GenerationConfig.creative(),\n      system_instruction: \"You are a physics professor\"\n    )\n\n    # Count tokens and use in your application code","ref":"Gemini.APIs.Tokens.html#count_for_request/2-examples"},{"type":"function","title":"Gemini.APIs.Tokens.estimate/2","doc":"Estimate tokens for content without making an API call.\n\nProvides a rough estimate of token count using heuristics.\nUseful for quick validation and pre-filtering before actual counting.","ref":"Gemini.APIs.Tokens.html#estimate/2"},{"type":"function","title":"Parameters - Gemini.APIs.Tokens.estimate/2","doc":"- `content` - Content to estimate (string or Content list)\n- `opts` - Options (currently unused, for future expansion)","ref":"Gemini.APIs.Tokens.html#estimate/2-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Tokens.estimate/2","doc":"- `{:ok, integer()}` - Estimated token count\n- `{:error, Error.t()}` - Error if content is invalid","ref":"Gemini.APIs.Tokens.html#estimate/2-returns"},{"type":"function","title":"Examples - Gemini.APIs.Tokens.estimate/2","doc":"{:ok, estimate} = Tokens.estimate(\"Hello, world!\")\n    # => {:ok, 3}\n\n    # For longer text\n    text = \"This is a longer piece of text that we want to estimate...\"\n    {:ok, estimate} = Tokens.estimate(text)","ref":"Gemini.APIs.Tokens.html#estimate/2-examples"},{"type":"function","title":"Note - Gemini.APIs.Tokens.estimate/2","doc":"This is a heuristic estimate and may not match the actual token count\nfrom the API. Use `count/2` for accurate token counting.","ref":"Gemini.APIs.Tokens.html#estimate/2-note"},{"type":"module","title":"Gemini.Auth.JWT","doc":"JWT token generation and management for Google Cloud service accounts.\n\nThis module handles JWT creation and signing for Vertex AI authentication\nbased on the documentation in v1.md. It supports both service account key files\nand the Google Cloud IAM signJwt API.","ref":"Gemini.Auth.JWT.html"},{"type":"function","title":"Gemini.Auth.JWT.create_payload/3","doc":"Create a JWT payload for Vertex AI authentication.","ref":"Gemini.Auth.JWT.html#create_payload/3"},{"type":"function","title":"Parameters - Gemini.Auth.JWT.create_payload/3","doc":"- `service_account_email`: The email of the service account (issuer)\n- `audience`: The audience for the JWT (must match deployment config)\n- `opts`: Optional parameters\n  - `:lifetime` - Token lifetime in seconds (default: 3600)\n  - `:issued_at` - Custom issued at time (default: current time)","ref":"Gemini.Auth.JWT.html#create_payload/3-parameters"},{"type":"function","title":"Examples - Gemini.Auth.JWT.create_payload/3","doc":"iex> payload = Gemini.Auth.JWT.create_payload(\n    ...>   \"my-service@project.iam.gserviceaccount.com\",\n    ...>   \"my-app-audience\"\n    ...> )\n    iex> payload.iss\n    \"my-service@project.iam.gserviceaccount.com\"","ref":"Gemini.Auth.JWT.html#create_payload/3-examples"},{"type":"function","title":"Gemini.Auth.JWT.create_signed_token/3","doc":"Create a signed JWT token using the most appropriate method.\n\nThis function automatically chooses between local signing (if private key is available)\nor IAM API signing (if access token is provided).","ref":"Gemini.Auth.JWT.html#create_signed_token/3"},{"type":"function","title":"Examples - Gemini.Auth.JWT.create_signed_token/3","doc":"# Using service account key file\n    iex> {:ok, token} = Gemini.Auth.JWT.create_signed_token(\n    ...>   \"my-service@project.iam.gserviceaccount.com\",\n    ...>   \"my-app-audience\",\n    ...>   service_account_key: \"/path/to/key.json\"\n    ...> )\n\n    # Using IAM API\n    iex> {:ok, token} = Gemini.Auth.JWT.create_signed_token(\n    ...>   \"my-service@project.iam.gserviceaccount.com\",\n    ...>   \"my-app-audience\",\n    ...>   access_token: \"ya29.access-token\"\n    ...> )","ref":"Gemini.Auth.JWT.html#create_signed_token/3-examples"},{"type":"function","title":"Gemini.Auth.JWT.get_service_account_email/1","doc":"Extract service account email from a service account key.","ref":"Gemini.Auth.JWT.html#get_service_account_email/1"},{"type":"function","title":"Examples - Gemini.Auth.JWT.get_service_account_email/1","doc":"iex> key = %{client_email: \"my-service@project.iam.gserviceaccount.com\"}\n    iex> Gemini.Auth.JWT.get_service_account_email(key)\n    \"my-service@project.iam.gserviceaccount.com\"","ref":"Gemini.Auth.JWT.html#get_service_account_email/1-examples"},{"type":"type","title":"Gemini.Auth.JWT.jwt_payload/0","doc":"","ref":"Gemini.Auth.JWT.html#t:jwt_payload/0"},{"type":"function","title":"Gemini.Auth.JWT.load_service_account_key/1","doc":"Load and parse a service account key file.","ref":"Gemini.Auth.JWT.html#load_service_account_key/1"},{"type":"function","title":"Examples - Gemini.Auth.JWT.load_service_account_key/1","doc":"iex> {:ok, key} = Gemini.Auth.JWT.load_service_account_key(\"/path/to/key.json\")\n    iex> key.client_email\n    \"my-service@project.iam.gserviceaccount.com\"","ref":"Gemini.Auth.JWT.html#load_service_account_key/1-examples"},{"type":"type","title":"Gemini.Auth.JWT.service_account_key/0","doc":"","ref":"Gemini.Auth.JWT.html#t:service_account_key/0"},{"type":"function","title":"Gemini.Auth.JWT.sign_with_iam_api/3","doc":"Sign a JWT payload using Google Cloud IAM signJwt API.\n\nThis method uses the Google Cloud IAM service to sign the JWT, which requires\nthe caller to have the `roles/iam.serviceAccountTokenCreator` role.","ref":"Gemini.Auth.JWT.html#sign_with_iam_api/3"},{"type":"function","title":"Parameters - Gemini.Auth.JWT.sign_with_iam_api/3","doc":"- `payload`: The JWT payload to sign\n- `service_account_email`: The service account email\n- `access_token`: A valid access token for authentication","ref":"Gemini.Auth.JWT.html#sign_with_iam_api/3-parameters"},{"type":"function","title":"Examples - Gemini.Auth.JWT.sign_with_iam_api/3","doc":"iex> payload = %{iss: \"...\", aud: \"...\", sub: \"...\", iat: 123, exp: 456}\n    iex> {:ok, token} = Gemini.Auth.JWT.sign_with_iam_api(\n    ...>   payload,\n    ...>   \"my-service@project.iam.gserviceaccount.com\",\n    ...>   \"ya29.access-token\"\n    ...> )","ref":"Gemini.Auth.JWT.html#sign_with_iam_api/3-examples"},{"type":"function","title":"Gemini.Auth.JWT.sign_with_key/2","doc":"Sign a JWT payload using a service account private key.\n\nThis method signs the JWT locally using the private key from the service account JSON file.","ref":"Gemini.Auth.JWT.html#sign_with_key/2"},{"type":"function","title":"Examples - Gemini.Auth.JWT.sign_with_key/2","doc":"iex> key = %{private_key: \"-----BEGIN PRIVATE KEY-----...\", client_email: \"...\"}\n    iex> payload = %{iss: \"...\", aud: \"...\", sub: \"...\", iat: 123, exp: 456}\n    iex> {:ok, token} = Gemini.Auth.JWT.sign_with_key(payload, key)","ref":"Gemini.Auth.JWT.html#sign_with_key/2-examples"},{"type":"function","title":"Gemini.Auth.JWT.validate_payload/1","doc":"Validate a JWT payload has all required fields.","ref":"Gemini.Auth.JWT.html#validate_payload/1"},{"type":"function","title":"Examples - Gemini.Auth.JWT.validate_payload/1","doc":"iex> payload = %{iss: \"test\", aud: \"test\", sub: \"test\", iat: 123, exp: 456}\n    iex> Gemini.Auth.JWT.validate_payload(payload)\n    :ok","ref":"Gemini.Auth.JWT.html#validate_payload/1-examples"},{"type":"behaviour","title":"Gemini.Auth.Strategy","doc":"Behavior for authentication strategies.","ref":"Gemini.Auth.Strategy.html"},{"type":"callback","title":"Gemini.Auth.Strategy.base_url/1","doc":"","ref":"Gemini.Auth.Strategy.html#c:base_url/1"},{"type":"callback","title":"Gemini.Auth.Strategy.build_path/3","doc":"","ref":"Gemini.Auth.Strategy.html#c:build_path/3"},{"type":"callback","title":"Gemini.Auth.Strategy.headers/1","doc":"","ref":"Gemini.Auth.Strategy.html#c:headers/1"},{"type":"callback","title":"Gemini.Auth.Strategy.refresh_credentials/1","doc":"","ref":"Gemini.Auth.Strategy.html#c:refresh_credentials/1"},{"type":"module","title":"Gemini.Generate","doc":"API for generating content with Gemini models.","ref":"Gemini.Generate.html"},{"type":"function","title":"Gemini.Generate.build_generate_request/2","doc":"Build a generate content request structure.\n\nThis function is exposed publicly to allow the streaming manager and other\ncomponents to construct requests using the same validation and normalization\nlogic as the main content generation functions.","ref":"Gemini.Generate.html#build_generate_request/2"},{"type":"function","title":"Parameters - Gemini.Generate.build_generate_request/2","doc":"- `contents` - List of Content structs or strings\n  - `opts` - Options including generation_config, safety_settings, etc.","ref":"Gemini.Generate.html#build_generate_request/2-parameters"},{"type":"function","title":"Returns - Gemini.Generate.build_generate_request/2","doc":"- Map representing the request structure ready for JSON encoding","ref":"Gemini.Generate.html#build_generate_request/2-returns"},{"type":"function","title":"Gemini.Generate.chat/1","doc":"Start a chat session for multi-turn conversations.","ref":"Gemini.Generate.html#chat/1"},{"type":"function","title":"Parameters - Gemini.Generate.chat/1","doc":"- `opts` - Options including:\n    - `:model` - Model name (default: from config)\n    - `:history` - Chat history as list of Content structs\n    - `:generation_config` - GenerationConfig struct\n    - `:safety_settings` - List of SafetySetting structs\n    - `:system_instruction` - System instruction","ref":"Gemini.Generate.html#chat/1-parameters"},{"type":"function","title":"Examples - Gemini.Generate.chat/1","doc":"iex> {:ok, chat} = Gemini.Generate.chat()\n    iex> Gemini.Generate.send_message(chat, \"Hello!\")\n    {:ok, response, updated_chat}","ref":"Gemini.Generate.html#chat/1-examples"},{"type":"function","title":"Gemini.Generate.content/2","doc":"Generate content using a Gemini model.","ref":"Gemini.Generate.html#content/2"},{"type":"function","title":"Parameters - Gemini.Generate.content/2","doc":"- `contents` - List of Content structs or strings\n  - `opts` - Options including:\n    - `:model` - Model name (default: from config)\n    - `:generation_config` - GenerationConfig struct\n    - `:safety_settings` - List of SafetySetting structs\n    - `:system_instruction` - System instruction as Content or string\n    - `:tools` - List of tool definitions\n    - `:tool_config` - Tool configuration","ref":"Gemini.Generate.html#content/2-parameters"},{"type":"function","title":"Examples - Gemini.Generate.content/2","doc":"iex> Gemini.Generate.content(\"Hello, world!\")\n    {:ok, %GenerateContentResponse{candidates: [%Candidate{...}]}}\n\n    iex> contents = [Content.text(\"Explain quantum physics\")]\n    iex> config = GenerationConfig.creative()\n    iex> Gemini.Generate.content(contents, generation_config: config)\n    {:ok, %GenerateContentResponse{...}}","ref":"Gemini.Generate.html#content/2-examples"},{"type":"function","title":"Gemini.Generate.count_tokens/2","doc":"Count tokens in the given content.","ref":"Gemini.Generate.html#count_tokens/2"},{"type":"function","title":"Parameters - Gemini.Generate.count_tokens/2","doc":"- `contents` - List of Content structs or strings\n  - `opts` - Options including:\n    - `:model` - Model name (default: from config)","ref":"Gemini.Generate.html#count_tokens/2-parameters"},{"type":"function","title":"Examples - Gemini.Generate.count_tokens/2","doc":"iex> Gemini.Generate.count_tokens(\"Hello, world!\")\n    {:ok, %CountTokensResponse{total_tokens: 3}}","ref":"Gemini.Generate.html#count_tokens/2-examples"},{"type":"function","title":"Gemini.Generate.send_message/2","doc":"Send a message in a chat session.","ref":"Gemini.Generate.html#send_message/2"},{"type":"function","title":"Parameters - Gemini.Generate.send_message/2","doc":"- `chat` - Chat session from `chat/1`\n  - `message` - Message content as string or Content struct","ref":"Gemini.Generate.html#send_message/2-parameters"},{"type":"function","title":"Returns - Gemini.Generate.send_message/2","doc":"- `{:ok, response, updated_chat}` on success\n  - `{:error, error}` on failure","ref":"Gemini.Generate.html#send_message/2-returns"},{"type":"function","title":"Gemini.Generate.stream_content/2","doc":"Generate content with streaming support.\n\nReturns a stream of partial responses as they become available.","ref":"Gemini.Generate.html#stream_content/2"},{"type":"function","title":"Parameters - Gemini.Generate.stream_content/2","doc":"- `contents` - List of Content structs or strings\n  - `opts` - Same options as `content/2`","ref":"Gemini.Generate.html#stream_content/2-parameters"},{"type":"function","title":"Examples - Gemini.Generate.stream_content/2","doc":"iex> Gemini.Generate.stream_content(\"Write a story\")\n    {:ok, [%GenerateContentResponse{...}, ...]}","ref":"Gemini.Generate.html#stream_content/2-examples"},{"type":"function","title":"Gemini.Generate.text/2","doc":"Generate content and return only the text from the first candidate.\n\nThis is a convenience function for simple text generation.","ref":"Gemini.Generate.html#text/2"},{"type":"function","title":"Examples - Gemini.Generate.text/2","doc":"iex> Gemini.Generate.text(\"What is the capital of France?\")\n    {:ok, \"The capital of France is Paris.\"}","ref":"Gemini.Generate.html#text/2-examples"},{"type":"module","title":"Gemini.Models","doc":"API for managing and querying Gemini models.","ref":"Gemini.Models.html"},{"type":"function","title":"Gemini.Models.exists?/1","doc":"Check if a model exists and is available.","ref":"Gemini.Models.html#exists?/1"},{"type":"function","title":"Examples - Gemini.Models.exists?/1","doc":"iex> Gemini.Models.exists?(\"gemini-2.0-flash\")\n    {:ok, true}\n\n    iex> Gemini.Models.exists?(\"invalid-model\")\n    {:ok, false}","ref":"Gemini.Models.html#exists?/1-examples"},{"type":"function","title":"Gemini.Models.get/1","doc":"Get information about a specific model.","ref":"Gemini.Models.html#get/1"},{"type":"function","title":"Parameters - Gemini.Models.get/1","doc":"- `model_name` - Name of the model (e.g., \"gemini-2.0-flash\")","ref":"Gemini.Models.html#get/1-parameters"},{"type":"function","title":"Examples - Gemini.Models.get/1","doc":"iex> Gemini.Models.get(\"gemini-2.0-flash\")\n    {:ok, %Model{name: \"models/gemini-2.0-flash\", ...}}\n\n    iex> Gemini.Models.get(\"invalid-model\")\n    {:error, %Gemini.Error{type: :api_error, ...}}","ref":"Gemini.Models.html#get/1-examples"},{"type":"function","title":"Gemini.Models.list/1","doc":"List available Gemini models.","ref":"Gemini.Models.html#list/1"},{"type":"function","title":"Options - Gemini.Models.list/1","doc":"- `:page_size` - Maximum number of models to return (default: 50)\n  - `:page_token` - Token for pagination","ref":"Gemini.Models.html#list/1-options"},{"type":"function","title":"Examples - Gemini.Models.list/1","doc":"iex> Gemini.Models.list()\n    {:ok, %ListModelsResponse{models: [%Model{...}], next_page_token: nil}}\n\n    iex> Gemini.Models.list(page_size: 10)\n    {:ok, %ListModelsResponse{...}}","ref":"Gemini.Models.html#list/1-examples"},{"type":"function","title":"Gemini.Models.list_names/0","doc":"List all available model names.","ref":"Gemini.Models.html#list_names/0"},{"type":"function","title":"Examples - Gemini.Models.list_names/0","doc":"iex> Gemini.Models.list_names()\n    {:ok, [\"gemini-2.0-flash\", \"gemini-1.5-pro\", \"gemini-1.5-flash\"]}","ref":"Gemini.Models.html#list_names/0-examples"},{"type":"function","title":"Gemini.Models.supporting_method/1","doc":"Get models that support a specific generation method.","ref":"Gemini.Models.html#supporting_method/1"},{"type":"function","title":"Parameters - Gemini.Models.supporting_method/1","doc":"- `method` - Generation method (e.g., \"generateContent\", \"streamGenerateContent\")","ref":"Gemini.Models.html#supporting_method/1-parameters"},{"type":"function","title":"Examples - Gemini.Models.supporting_method/1","doc":"iex> Gemini.Models.supporting_method(\"generateContent\")\n    {:ok, [%Model{...}, ...]}","ref":"Gemini.Models.html#supporting_method/1-examples"},{"type":"module","title":"Gemini.Streaming.Manager","doc":"GenServer for managing streaming connections and state.\n\nThis GenServer handles:\n- Managing multiple concurrent streaming sessions\n- Buffering and parsing Server-Sent Events\n- Maintaining connection state and metadata\n- Automatic reconnection and error recovery","ref":"Gemini.Streaming.Manager.html"},{"type":"function","title":"Gemini.Streaming.Manager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gemini.Streaming.Manager.html#child_spec/1"},{"type":"function","title":"Gemini.Streaming.Manager.get_stream_info/1","doc":"Get information about a specific stream.","ref":"Gemini.Streaming.Manager.html#get_stream_info/1"},{"type":"function","title":"Gemini.Streaming.Manager.get_stream_status/1","doc":"Get the current status of a stream.","ref":"Gemini.Streaming.Manager.html#get_stream_status/1"},{"type":"function","title":"Gemini.Streaming.Manager.list_streams/0","doc":"List all active streams.","ref":"Gemini.Streaming.Manager.html#list_streams/0"},{"type":"function","title":"Gemini.Streaming.Manager.start_link/1","doc":"","ref":"Gemini.Streaming.Manager.html#start_link/1"},{"type":"function","title":"Gemini.Streaming.Manager.start_stream/3","doc":"Start a new streaming session with contents and options.\n(Alternative signature for compatibility)","ref":"Gemini.Streaming.Manager.html#start_stream/3"},{"type":"function","title":"Gemini.Streaming.Manager.start_stream/6","doc":"Start a new streaming session.\n\nReturns a stream ID that can be used to subscribe to events.","ref":"Gemini.Streaming.Manager.html#start_stream/6"},{"type":"type","title":"Gemini.Streaming.Manager.state/0","doc":"","ref":"Gemini.Streaming.Manager.html#t:state/0"},{"type":"function","title":"Gemini.Streaming.Manager.stop_stream/1","doc":"Stop a streaming session.","ref":"Gemini.Streaming.Manager.html#stop_stream/1"},{"type":"type","title":"Gemini.Streaming.Manager.stream_id/0","doc":"","ref":"Gemini.Streaming.Manager.html#t:stream_id/0"},{"type":"type","title":"Gemini.Streaming.Manager.stream_state/0","doc":"","ref":"Gemini.Streaming.Manager.html#t:stream_state/0"},{"type":"function","title":"Gemini.Streaming.Manager.subscribe/2","doc":"Subscribe to events from a streaming session.","ref":"Gemini.Streaming.Manager.html#subscribe/2"},{"type":"function","title":"Gemini.Streaming.Manager.subscribe_stream/2","doc":"Subscribe to events from a streaming session.\n(Alias for subscribe/2)","ref":"Gemini.Streaming.Manager.html#subscribe_stream/2"},{"type":"function","title":"Gemini.Streaming.Manager.unsubscribe/2","doc":"Unsubscribe from a streaming session.","ref":"Gemini.Streaming.Manager.html#unsubscribe/2"},{"type":"module","title":"Gemini.Streaming.ManagerV2","doc":"Improved GenServer for managing streaming connections and state.\n\nFeatures:\n- Proper HTTP streaming with persistent connections\n- Resource management and cleanup\n- Subscriber pattern with backpressure\n- Error handling and automatic retries\n- Stream lifecycle management","ref":"Gemini.Streaming.ManagerV2.html"},{"type":"function","title":"Gemini.Streaming.ManagerV2.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gemini.Streaming.ManagerV2.html#child_spec/1"},{"type":"function","title":"Gemini.Streaming.ManagerV2.get_stats/0","doc":"Get manager statistics.","ref":"Gemini.Streaming.ManagerV2.html#get_stats/0"},{"type":"function","title":"Gemini.Streaming.ManagerV2.get_stream_info/1","doc":"Get information about a stream.","ref":"Gemini.Streaming.ManagerV2.html#get_stream_info/1"},{"type":"function","title":"Gemini.Streaming.ManagerV2.list_streams/0","doc":"List all active streams.","ref":"Gemini.Streaming.ManagerV2.html#list_streams/0"},{"type":"type","title":"Gemini.Streaming.ManagerV2.manager_state/0","doc":"","ref":"Gemini.Streaming.ManagerV2.html#t:manager_state/0"},{"type":"function","title":"Gemini.Streaming.ManagerV2.start_link/1","doc":"Start the streaming manager.","ref":"Gemini.Streaming.ManagerV2.html#start_link/1"},{"type":"function","title":"Gemini.Streaming.ManagerV2.start_stream/3","doc":"Start a new streaming session.","ref":"Gemini.Streaming.ManagerV2.html#start_stream/3"},{"type":"function","title":"Parameters - Gemini.Streaming.ManagerV2.start_stream/3","doc":"- `contents` - Content to stream (string or list of Content structs)\n- `opts` - Generation options (model, generation_config, etc.)\n- `subscriber_pid` - Process to receive stream events (default: calling process)","ref":"Gemini.Streaming.ManagerV2.html#start_stream/3-parameters"},{"type":"function","title":"Returns - Gemini.Streaming.ManagerV2.start_stream/3","doc":"- `{:ok, stream_id}` - Stream started successfully\n- `{:error, reason}` - Failed to start stream","ref":"Gemini.Streaming.ManagerV2.html#start_stream/3-returns"},{"type":"function","title":"Events sent to subscriber: - Gemini.Streaming.ManagerV2.start_stream/3","doc":"- `{:stream_event, stream_id, event_data}` - New event received\n- `{:stream_complete, stream_id}` - Stream completed successfully\n- `{:stream_error, stream_id, error}` - Stream failed with error","ref":"Gemini.Streaming.ManagerV2.html#start_stream/3-events-sent-to-subscriber"},{"type":"function","title":"Gemini.Streaming.ManagerV2.stop_stream/1","doc":"Stop a streaming session.","ref":"Gemini.Streaming.ManagerV2.html#stop_stream/1"},{"type":"type","title":"Gemini.Streaming.ManagerV2.stream_id/0","doc":"","ref":"Gemini.Streaming.ManagerV2.html#t:stream_id/0"},{"type":"type","title":"Gemini.Streaming.ManagerV2.stream_state/0","doc":"","ref":"Gemini.Streaming.ManagerV2.html#t:stream_state/0"},{"type":"function","title":"Gemini.Streaming.ManagerV2.subscribe_stream/2","doc":"Subscribe to events from an existing stream.","ref":"Gemini.Streaming.ManagerV2.html#subscribe_stream/2"},{"type":"type","title":"Gemini.Streaming.ManagerV2.subscriber_ref/0","doc":"","ref":"Gemini.Streaming.ManagerV2.html#t:subscriber_ref/0"},{"type":"function","title":"Gemini.Streaming.ManagerV2.unsubscribe_stream/2","doc":"Unsubscribe from a stream.","ref":"Gemini.Streaming.ManagerV2.html#unsubscribe_stream/2"},{"type":"module","title":"Gemini.Types.Blob","doc":"Binary data with MIME type for Gemini API.","ref":"Gemini.Types.Blob.html"},{"type":"type","title":"Gemini.Types.Blob.blob_data/0","doc":"Base64 encoded binary data.","ref":"Gemini.Types.Blob.html#t:blob_data/0"},{"type":"function","title":"Gemini.Types.Blob.from_file/1","doc":"Create a blob from a file path.","ref":"Gemini.Types.Blob.html#from_file/1"},{"type":"type","title":"Gemini.Types.Blob.mime_type/0","doc":"MIME type of the data.","ref":"Gemini.Types.Blob.html#t:mime_type/0"},{"type":"function","title":"Gemini.Types.Blob.new/2","doc":"Create a new blob with base64 encoded data.","ref":"Gemini.Types.Blob.html#new/2"},{"type":"type","title":"Gemini.Types.Blob.t/0","doc":"","ref":"Gemini.Types.Blob.html#t:t/0"},{"type":"module","title":"Gemini.Types.GenerationConfig","doc":"Configuration for content generation parameters.","ref":"Gemini.Types.GenerationConfig.html"},{"type":"function","title":"Gemini.Types.GenerationConfig.balanced/1","doc":"Create a balanced generation config.","ref":"Gemini.Types.GenerationConfig.html#balanced/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.creative/1","doc":"Create a creative generation config (higher temperature).","ref":"Gemini.Types.GenerationConfig.html#creative/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.deterministic/1","doc":"Create a deterministic generation config.","ref":"Gemini.Types.GenerationConfig.html#deterministic/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.json_response/1","doc":"Set JSON response format.","ref":"Gemini.Types.GenerationConfig.html#json_response/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.max_tokens/2","doc":"Set maximum output tokens.","ref":"Gemini.Types.GenerationConfig.html#max_tokens/2"},{"type":"function","title":"Gemini.Types.GenerationConfig.new/1","doc":"Create a new generation config with default values.","ref":"Gemini.Types.GenerationConfig.html#new/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.precise/1","doc":"Create a precise generation config (lower temperature).","ref":"Gemini.Types.GenerationConfig.html#precise/1"},{"type":"function","title":"Gemini.Types.GenerationConfig.stop_sequences/2","doc":"Add stop sequences.","ref":"Gemini.Types.GenerationConfig.html#stop_sequences/2"},{"type":"type","title":"Gemini.Types.GenerationConfig.t/0","doc":"","ref":"Gemini.Types.GenerationConfig.html#t:t/0"},{"type":"function","title":"Gemini.Types.GenerationConfig.text_response/1","doc":"Set plain text response format.","ref":"Gemini.Types.GenerationConfig.html#text_response/1"},{"type":"module","title":"Gemini.Types.Part","doc":"Part type for content in Gemini API.","ref":"Gemini.Types.Part.html"},{"type":"function","title":"Gemini.Types.Part.blob/2","doc":"Create a blob part with raw data and MIME type.","ref":"Gemini.Types.Part.html#blob/2"},{"type":"function","title":"Gemini.Types.Part.file/1","doc":"Create a part from a file path.","ref":"Gemini.Types.Part.html#file/1"},{"type":"type","title":"Gemini.Types.Part.inline_data/0","doc":"Inline data (base64 encoded).","ref":"Gemini.Types.Part.html#t:inline_data/0"},{"type":"function","title":"Gemini.Types.Part.inline_data/2","doc":"Create an inline data part with base64 encoded data.","ref":"Gemini.Types.Part.html#inline_data/2"},{"type":"type","title":"Gemini.Types.Part.t/0","doc":"","ref":"Gemini.Types.Part.html#t:t/0"},{"type":"function","title":"Gemini.Types.Part.text/1","doc":"Create a text part.","ref":"Gemini.Types.Part.html#text/1"},{"type":"type","title":"Gemini.Types.Part.text_content/0","doc":"Text content.","ref":"Gemini.Types.Part.html#t:text_content/0"},{"type":"module","title":"Gemini.Types.Request.CountTokensRequest","doc":"Request structure for counting tokens.\n\nSupports counting tokens for both simple contents and\nfull GenerateContentRequest structures.","ref":"Gemini.Types.Request.CountTokensRequest.html"},{"type":"function","title":"Gemini.Types.Request.CountTokensRequest.new/2","doc":"Create a new CountTokensRequest.","ref":"Gemini.Types.Request.CountTokensRequest.html#new/2"},{"type":"function","title":"Parameters - Gemini.Types.Request.CountTokensRequest.new/2","doc":"- `input` - Either contents (string/list) or a GenerateContentRequest\n- `opts` - Additional options","ref":"Gemini.Types.Request.CountTokensRequest.html#new/2-parameters"},{"type":"function","title":"Examples - Gemini.Types.Request.CountTokensRequest.new/2","doc":"iex> CountTokensRequest.new(\"Hello world\")\n    {:ok, %CountTokensRequest{contents: [%Content{...}]}}\n\n    iex> CountTokensRequest.new(generate_request)\n    {:ok, %CountTokensRequest{generate_content_request: generate_request}}","ref":"Gemini.Types.Request.CountTokensRequest.html#new/2-examples"},{"type":"type","title":"Gemini.Types.Request.CountTokensRequest.t/0","doc":"","ref":"Gemini.Types.Request.CountTokensRequest.html#t:t/0"},{"type":"function","title":"Gemini.Types.Request.CountTokensRequest.to_json_map/1","doc":"Convert request to map suitable for JSON encoding.\n\nOnly includes the non-nil field (either contents or generate_content_request).","ref":"Gemini.Types.Request.CountTokensRequest.html#to_json_map/1"},{"type":"module","title":"Gemini.Types.Request.GenerateContentRequest","doc":"Request structure for content generation.\n\nSupports all generation parameters including safety settings,\nsystem instructions, tools, and generation configuration.","ref":"Gemini.Types.Request.GenerateContentRequest.html"},{"type":"function","title":"Gemini.Types.Request.GenerateContentRequest.new/2","doc":"Create a new GenerateContentRequest with validation.","ref":"Gemini.Types.Request.GenerateContentRequest.html#new/2"},{"type":"function","title":"Parameters - Gemini.Types.Request.GenerateContentRequest.new/2","doc":"- `contents` - List of Content structs or single string\n- `opts` - Keyword list of options:\n  - `:generation_config` - GenerationConfig struct\n  - `:safety_settings` - List of SafetySetting structs\n  - `:system_instruction` - System instruction as Content or string\n  - `:tools` - List of tool definitions\n  - `:tool_config` - Tool configuration","ref":"Gemini.Types.Request.GenerateContentRequest.html#new/2-parameters"},{"type":"function","title":"Examples - Gemini.Types.Request.GenerateContentRequest.new/2","doc":"iex> GenerateContentRequest.new(\"Hello world\")\n    {:ok, %GenerateContentRequest{contents: [%Content{...}]}}\n\n    iex> GenerateContentRequest.new([Content.text(\"Hello\")])\n    {:ok, %GenerateContentRequest{...}}","ref":"Gemini.Types.Request.GenerateContentRequest.html#new/2-examples"},{"type":"type","title":"Gemini.Types.Request.GenerateContentRequest.t/0","doc":"","ref":"Gemini.Types.Request.GenerateContentRequest.html#t:t/0"},{"type":"function","title":"Gemini.Types.Request.GenerateContentRequest.to_json_map/1","doc":"Convert request to map suitable for JSON encoding.\n\nRemoves nil fields to create clean JSON payload.","ref":"Gemini.Types.Request.GenerateContentRequest.html#to_json_map/1"},{"type":"module","title":"Gemini.Types.Request.GetModelRequest","doc":"Request structure for getting a specific model.","ref":"Gemini.Types.Request.GetModelRequest.html"},{"type":"function","title":"Gemini.Types.Request.GetModelRequest.new/1","doc":"Create a new GetModelRequest with name normalization.","ref":"Gemini.Types.Request.GetModelRequest.html#new/1"},{"type":"function","title":"Examples - Gemini.Types.Request.GetModelRequest.new/1","doc":"iex> GetModelRequest.new(\"gemini-2.0-flash\")\n    {:ok, %GetModelRequest{name: \"models/gemini-2.0-flash\"}}\n\n    iex> GetModelRequest.new(\"models/gemini-1.5-pro\")\n    {:ok, %GetModelRequest{name: \"models/gemini-1.5-pro\"}}\n\n    iex> GetModelRequest.new(\"\")\n    {:error, \"Model name cannot be empty\"}","ref":"Gemini.Types.Request.GetModelRequest.html#new/1-examples"},{"type":"type","title":"Gemini.Types.Request.GetModelRequest.t/0","doc":"","ref":"Gemini.Types.Request.GetModelRequest.html#t:t/0"},{"type":"module","title":"Gemini.Types.Request.ListModelsRequest","doc":"Request structure for listing models with pagination support.","ref":"Gemini.Types.Request.ListModelsRequest.html"},{"type":"function","title":"Gemini.Types.Request.ListModelsRequest.new/1","doc":"Create a new ListModelsRequest with validation.","ref":"Gemini.Types.Request.ListModelsRequest.html#new/1"},{"type":"function","title":"Parameters - Gemini.Types.Request.ListModelsRequest.new/1","doc":"- `opts` - Keyword list of options:\n  - `:page_size` - Number of models per page (1-1000)\n  - `:page_token` - Token for pagination","ref":"Gemini.Types.Request.ListModelsRequest.html#new/1-parameters"},{"type":"function","title":"Examples - Gemini.Types.Request.ListModelsRequest.new/1","doc":"iex> ListModelsRequest.new(page_size: 50)\n    {:ok, %ListModelsRequest{page_size: 50}}\n\n    iex> ListModelsRequest.new(page_size: 2000)\n    {:error, \"Page size must be between 1 and 1000\"}","ref":"Gemini.Types.Request.ListModelsRequest.html#new/1-examples"},{"type":"type","title":"Gemini.Types.Request.ListModelsRequest.t/0","doc":"","ref":"Gemini.Types.Request.ListModelsRequest.html#t:t/0"},{"type":"function","title":"Gemini.Types.Request.ListModelsRequest.to_query_params/1","doc":"Build query parameters string from request.","ref":"Gemini.Types.Request.ListModelsRequest.html#to_query_params/1"},{"type":"module","title":"Gemini.Types.Response.Candidate","doc":"Content candidate in response.","ref":"Gemini.Types.Response.Candidate.html"},{"type":"type","title":"Gemini.Types.Response.Candidate.t/0","doc":"","ref":"Gemini.Types.Response.Candidate.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.CitationMetadata","doc":"Citation metadata for generated content.","ref":"Gemini.Types.Response.CitationMetadata.html"},{"type":"type","title":"Gemini.Types.Response.CitationMetadata.t/0","doc":"","ref":"Gemini.Types.Response.CitationMetadata.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.CitationSource","doc":"Citation source information.","ref":"Gemini.Types.Response.CitationSource.html"},{"type":"type","title":"Gemini.Types.Response.CitationSource.t/0","doc":"","ref":"Gemini.Types.Response.CitationSource.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.CountTokensResponse","doc":"Response from counting tokens.","ref":"Gemini.Types.Response.CountTokensResponse.html"},{"type":"type","title":"Gemini.Types.Response.CountTokensResponse.t/0","doc":"","ref":"Gemini.Types.Response.CountTokensResponse.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.GenerateContentResponse","doc":"Response from content generation.","ref":"Gemini.Types.Response.GenerateContentResponse.html"},{"type":"function","title":"Gemini.Types.Response.GenerateContentResponse.extract_text/1","doc":"Extract text content from the response.","ref":"Gemini.Types.Response.GenerateContentResponse.html#extract_text/1"},{"type":"function","title":"Gemini.Types.Response.GenerateContentResponse.finish_reason/1","doc":"Get the finish reason from the first candidate.","ref":"Gemini.Types.Response.GenerateContentResponse.html#finish_reason/1"},{"type":"type","title":"Gemini.Types.Response.GenerateContentResponse.t/0","doc":"","ref":"Gemini.Types.Response.GenerateContentResponse.html#t:t/0"},{"type":"function","title":"Gemini.Types.Response.GenerateContentResponse.token_usage/1","doc":"Get token usage information from the response.","ref":"Gemini.Types.Response.GenerateContentResponse.html#token_usage/1"},{"type":"module","title":"Gemini.Types.Response.GroundingAttribution","doc":"Grounding attribution information.","ref":"Gemini.Types.Response.GroundingAttribution.html"},{"type":"type","title":"Gemini.Types.Response.GroundingAttribution.t/0","doc":"","ref":"Gemini.Types.Response.GroundingAttribution.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.GroundingAttributionSourceId","doc":"Grounding attribution source ID.","ref":"Gemini.Types.Response.GroundingAttributionSourceId.html"},{"type":"type","title":"Gemini.Types.Response.GroundingAttributionSourceId.t/0","doc":"","ref":"Gemini.Types.Response.GroundingAttributionSourceId.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.GroundingPassageId","doc":"Grounding passage ID.","ref":"Gemini.Types.Response.GroundingPassageId.html"},{"type":"type","title":"Gemini.Types.Response.GroundingPassageId.t/0","doc":"","ref":"Gemini.Types.Response.GroundingPassageId.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.ListModelsResponse","doc":"Response structure for listing models.\n\nContains the list of models and pagination information.","ref":"Gemini.Types.Response.ListModelsResponse.html"},{"type":"function","title":"Gemini.Types.Response.ListModelsResponse.filter_models/2","doc":"Filter models by a predicate function.","ref":"Gemini.Types.Response.ListModelsResponse.html#filter_models/2"},{"type":"function","title":"Gemini.Types.Response.ListModelsResponse.group_models/2","doc":"Group models by a classification function.","ref":"Gemini.Types.Response.ListModelsResponse.html#group_models/2"},{"type":"function","title":"Gemini.Types.Response.ListModelsResponse.has_next_page?/1","doc":"Check if there are more pages available.","ref":"Gemini.Types.Response.ListModelsResponse.html#has_next_page?/1"},{"type":"function","title":"Gemini.Types.Response.ListModelsResponse.model_count/1","doc":"Get the total number of models in this response.","ref":"Gemini.Types.Response.ListModelsResponse.html#model_count/1"},{"type":"function","title":"Gemini.Types.Response.ListModelsResponse.model_names/1","doc":"Extract model names from the response.","ref":"Gemini.Types.Response.ListModelsResponse.html#model_names/1"},{"type":"type","title":"Gemini.Types.Response.ListModelsResponse.t/0","doc":"","ref":"Gemini.Types.Response.ListModelsResponse.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.Model","doc":"Model information response structure.\n\nRepresents the complete model metadata returned by the Gemini API,\nincluding capabilities, token limits, and generation parameters.","ref":"Gemini.Types.Response.Model.html"},{"type":"function","title":"Gemini.Types.Response.Model.capabilities_summary/1","doc":"Generate a comprehensive capabilities summary.","ref":"Gemini.Types.Response.Model.html#capabilities_summary/1"},{"type":"function","title":"Example Response - Gemini.Types.Response.Model.capabilities_summary/1","doc":"%{\n      supports_streaming: true,\n      supports_token_counting: true,\n      supports_embeddings: false,\n      has_temperature: true,\n      has_top_k: true,\n      has_top_p: false,\n      method_count: 3,\n      input_capacity: :very_large,\n      output_capacity: :medium\n    }","ref":"Gemini.Types.Response.Model.html#capabilities_summary/1-example-response"},{"type":"type","title":"Gemini.Types.Response.Model.capability_level/0","doc":"Model capability level based on features and limits","ref":"Gemini.Types.Response.Model.html#t:capability_level/0"},{"type":"function","title":"Gemini.Types.Response.Model.capability_score/1","doc":"Calculate a capability score for model comparison.\n\nHigher scores indicate more capable models.","ref":"Gemini.Types.Response.Model.html#capability_score/1"},{"type":"type","title":"Gemini.Types.Response.Model.capacity_tier/0","doc":"Token capacity classification","ref":"Gemini.Types.Response.Model.html#t:capacity_tier/0"},{"type":"function","title":"Gemini.Types.Response.Model.compare_capabilities/2","doc":"Compare two models by capability.\n\nReturns :lt, :eq, or :gt based on capability scores.","ref":"Gemini.Types.Response.Model.html#compare_capabilities/2"},{"type":"function","title":"Gemini.Types.Response.Model.effective_base_id/1","doc":"Get the effective base model ID.\n\nPrefers the base_model_id field, but falls back to extracting\nfrom the name if base_model_id is nil.","ref":"Gemini.Types.Response.Model.html#effective_base_id/1"},{"type":"function","title":"Examples - Gemini.Types.Response.Model.effective_base_id/1","doc":"iex> Model.effective_base_id(%Model{base_model_id: \"gemini-2.0-flash\"})\n    \"gemini-2.0-flash\"\n\n    iex> Model.effective_base_id(%Model{name: \"models/gemini-1.5-pro\", base_model_id: nil})\n    \"gemini-1.5-pro\"","ref":"Gemini.Types.Response.Model.html#effective_base_id/1-examples"},{"type":"function","title":"Gemini.Types.Response.Model.has_advanced_params?/1","doc":"Check if model has advanced generation parameters.\n\nReturns true if the model supports temperature, top_p, or top_k parameters.","ref":"Gemini.Types.Response.Model.html#has_advanced_params?/1"},{"type":"function","title":"Gemini.Types.Response.Model.input_capacity_tier/1","doc":"Classify model's input token capacity.","ref":"Gemini.Types.Response.Model.html#input_capacity_tier/1"},{"type":"function","title":"Examples - Gemini.Types.Response.Model.input_capacity_tier/1","doc":"iex> Model.input_capacity_tier(%Model{input_token_limit: 2_000_000})\n    :very_large\n\n    iex> Model.input_capacity_tier(%Model{input_token_limit: 30_000})\n    :medium","ref":"Gemini.Types.Response.Model.html#input_capacity_tier/1-examples"},{"type":"function","title":"Gemini.Types.Response.Model.is_latest_version?/1","doc":"Check if this appears to be the latest version of a model.\n\nHeuristic based on name patterns (no version suffix, \"latest\" in name).","ref":"Gemini.Types.Response.Model.html#is_latest_version?/1"},{"type":"function","title":"Gemini.Types.Response.Model.model_family/1","doc":"Extract model family from the base model ID.","ref":"Gemini.Types.Response.Model.html#model_family/1"},{"type":"function","title":"Examples - Gemini.Types.Response.Model.model_family/1","doc":"iex> Model.model_family(%Model{base_model_id: \"gemini-2.0-flash\"})\n    \"gemini\"\n\n    iex> Model.model_family(%Model{base_model_id: \"text-embedding-004\"})\n    \"text\"","ref":"Gemini.Types.Response.Model.html#model_family/1-examples"},{"type":"function","title":"Gemini.Types.Response.Model.output_capacity_tier/1","doc":"Classify model's output token capacity.","ref":"Gemini.Types.Response.Model.html#output_capacity_tier/1"},{"type":"function","title":"Gemini.Types.Response.Model.production_ready?/1","doc":"Determine if model is suitable for production use.\n\nBased on capability, capacity, and stability indicators.","ref":"Gemini.Types.Response.Model.html#production_ready?/1"},{"type":"function","title":"Gemini.Types.Response.Model.supports_embeddings?/1","doc":"Check if model supports embeddings.","ref":"Gemini.Types.Response.Model.html#supports_embeddings?/1"},{"type":"function","title":"Gemini.Types.Response.Model.supports_method?/2","doc":"Check if model supports a specific generation method.","ref":"Gemini.Types.Response.Model.html#supports_method?/2"},{"type":"function","title":"Examples - Gemini.Types.Response.Model.supports_method?/2","doc":"iex> Model.supports_method?(model, \"generateContent\")\n    true\n\n    iex> Model.supports_method?(model, \"nonexistentMethod\")\n    false","ref":"Gemini.Types.Response.Model.html#supports_method?/2-examples"},{"type":"function","title":"Gemini.Types.Response.Model.supports_streaming?/1","doc":"Check if model supports streaming content generation.","ref":"Gemini.Types.Response.Model.html#supports_streaming?/1"},{"type":"function","title":"Gemini.Types.Response.Model.supports_token_counting?/1","doc":"Check if model supports token counting.","ref":"Gemini.Types.Response.Model.html#supports_token_counting?/1"},{"type":"type","title":"Gemini.Types.Response.Model.t/0","doc":"","ref":"Gemini.Types.Response.Model.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.PromptFeedback","doc":"Prompt feedback information.","ref":"Gemini.Types.Response.PromptFeedback.html"},{"type":"type","title":"Gemini.Types.Response.PromptFeedback.t/0","doc":"","ref":"Gemini.Types.Response.PromptFeedback.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.SafetyRating","doc":"Safety rating for content.","ref":"Gemini.Types.Response.SafetyRating.html"},{"type":"type","title":"Gemini.Types.Response.SafetyRating.t/0","doc":"","ref":"Gemini.Types.Response.SafetyRating.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.SemanticRetrieverChunk","doc":"Semantic retriever chunk information.","ref":"Gemini.Types.Response.SemanticRetrieverChunk.html"},{"type":"type","title":"Gemini.Types.Response.SemanticRetrieverChunk.t/0","doc":"","ref":"Gemini.Types.Response.SemanticRetrieverChunk.html#t:t/0"},{"type":"module","title":"Gemini.Types.Response.UsageMetadata","doc":"Usage metadata for API calls.","ref":"Gemini.Types.Response.UsageMetadata.html"},{"type":"type","title":"Gemini.Types.Response.UsageMetadata.t/0","doc":"","ref":"Gemini.Types.Response.UsageMetadata.html#t:t/0"},{"type":"module","title":"Gemini.Types.SafetySetting","doc":"Safety settings for content generation.","ref":"Gemini.Types.SafetySetting.html"},{"type":"type","title":"Gemini.Types.SafetySetting.category/0","doc":"","ref":"Gemini.Types.SafetySetting.html#t:category/0"},{"type":"function","title":"Gemini.Types.SafetySetting.dangerous_content/1","doc":"Create a safety setting for dangerous content.","ref":"Gemini.Types.SafetySetting.html#dangerous_content/1"},{"type":"function","title":"Gemini.Types.SafetySetting.defaults/0","doc":"Get default safety settings (medium threshold for all categories).","ref":"Gemini.Types.SafetySetting.html#defaults/0"},{"type":"function","title":"Gemini.Types.SafetySetting.harassment/1","doc":"Create a safety setting for harassment content.","ref":"Gemini.Types.SafetySetting.html#harassment/1"},{"type":"function","title":"Gemini.Types.SafetySetting.hate_speech/1","doc":"Create a safety setting for hate speech content.","ref":"Gemini.Types.SafetySetting.html#hate_speech/1"},{"type":"function","title":"Gemini.Types.SafetySetting.permissive/0","doc":"Get permissive safety settings (block only high risk content).","ref":"Gemini.Types.SafetySetting.html#permissive/0"},{"type":"function","title":"Gemini.Types.SafetySetting.sexually_explicit/1","doc":"Create a safety setting for sexually explicit content.","ref":"Gemini.Types.SafetySetting.html#sexually_explicit/1"},{"type":"type","title":"Gemini.Types.SafetySetting.t/0","doc":"","ref":"Gemini.Types.SafetySetting.html#t:t/0"},{"type":"type","title":"Gemini.Types.SafetySetting.threshold/0","doc":"","ref":"Gemini.Types.SafetySetting.html#t:threshold/0"},{"type":"module","title":"Gemini","doc":"# Gemini Elixir Client\n\nA comprehensive Elixir client for Google's Gemini AI API with dual authentication support,\nadvanced streaming capabilities, type safety, and built-in telemetry.","ref":"Gemini.html"},{"type":"module","title":"Features - Gemini","doc":"- **🔐 Dual Authentication**: Seamless support for both Gemini API keys and Vertex AI OAuth/Service Accounts\n- **⚡ Advanced Streaming**: Production-grade Server-Sent Events streaming with real-time processing\n- **🛡️ Type Safety**: Complete type definitions with runtime validation\n- **📊 Built-in Telemetry**: Comprehensive observability and metrics out of the box\n- **💬 Chat Sessions**: Multi-turn conversation management with state persistence\n- **🎭 Multimodal**: Full support for text, image, audio, and video content\n- **🚀 Production Ready**: Robust error handling, retry logic, and performance optimizations","ref":"Gemini.html#module-features"},{"type":"module","title":"Quick Start - Gemini","doc":"","ref":"Gemini.html#module-quick-start"},{"type":"module","title":"Installation - Gemini","doc":"Add to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:gemini, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"Gemini.html#module-installation"},{"type":"module","title":"Basic Configuration - Gemini","doc":"Configure your API key in `config/runtime.exs`:\n\n```elixir\nimport Config\n\nconfig :gemini,\n  api_key: System.get_env(\"GEMINI_API_KEY\")\n```\n\nOr set the environment variable:\n\n```bash\nexport GEMINI_API_KEY=\"your_api_key_here\"\n```","ref":"Gemini.html#module-basic-configuration"},{"type":"module","title":"Simple Usage - Gemini","doc":"```elixir\n# Basic text generation\n{:ok, response} = Gemini.generate(\"Tell me about Elixir programming\")\n{:ok, text} = Gemini.extract_text(response)\nIO.puts(text)\n\n# With options\n{:ok, response} = Gemini.generate(\"Explain quantum computing\", [\n  model: \"gemini-1.5-pro\",\n  temperature: 0.7,\n  max_output_tokens: 1000\n])\n```","ref":"Gemini.html#module-simple-usage"},{"type":"module","title":"Streaming - Gemini","doc":"```elixir\n# Start a streaming session\n{:ok, stream_id} = Gemini.stream_generate(\"Write a long story\", [\n  on_chunk: fn chunk -> IO.write(chunk) end,\n  on_complete: fn -> IO.puts(\"\\n✅ Complete!\") end\n])\n```","ref":"Gemini.html#module-streaming"},{"type":"module","title":"Authentication - Gemini","doc":"This client supports two authentication methods:","ref":"Gemini.html#module-authentication"},{"type":"module","title":"1. Gemini API Key (Simple) - Gemini","doc":"Best for development and simple applications:\n\n```elixir\n# Environment variable (recommended)\nexport GEMINI_API_KEY=\"your_api_key\"\n\n# Application config\nconfig :gemini, api_key: \"your_api_key\"\n\n# Per-request override\nGemini.generate(\"Hello\", api_key: \"specific_key\")\n```","ref":"Gemini.html#module-1-gemini-api-key-simple"},{"type":"module","title":"2. Vertex AI (Production) - Gemini","doc":"Best for production Google Cloud applications:\n\n```elixir\n# Service Account JSON file\nexport VERTEX_SERVICE_ACCOUNT=\"/path/to/service-account.json\"\nexport VERTEX_PROJECT_ID=\"your-gcp-project\"\nexport VERTEX_LOCATION=\"us-central1\"\n\n# Application config\nconfig :gemini, :auth,\n  type: :vertex_ai,\n  credentials: %{\n    service_account_key: System.get_env(\"VERTEX_SERVICE_ACCOUNT\"),\n    project_id: System.get_env(\"VERTEX_PROJECT_ID\"),\n    location: \"us-central1\"\n  }\n```","ref":"Gemini.html#module-2-vertex-ai-production"},{"type":"module","title":"Error Handling - Gemini","doc":"The client provides detailed error information with recovery suggestions:\n\n```elixir\ncase Gemini.generate(\"Hello world\") do\n  {:ok, response} ->\n    {:ok, text} = Gemini.extract_text(response)\n\n  {:error, %Gemini.Error{type: :rate_limit} = error} ->\n    IO.puts(\"Rate limited. Retry after: #{error.retry_after}\")\n\n  {:error, %Gemini.Error{type: :authentication} = error} ->\n    IO.puts(\"Auth error: #{error.message}\")\n\n  {:error, error} ->\n    IO.puts(\"Unexpected error: #{inspect(error)}\")\nend\n```","ref":"Gemini.html#module-error-handling"},{"type":"module","title":"Advanced Features - Gemini","doc":"","ref":"Gemini.html#module-advanced-features"},{"type":"module","title":"Multimodal Content - Gemini","doc":"```elixir\ncontent = [\n  %{type: \"text\", text: \"What's in this image?\"},\n  %{type: \"image\", source: %{type: \"base64\", data: base64_image}}\n]\n\n{:ok, response} = Gemini.generate(content)\n```","ref":"Gemini.html#module-multimodal-content"},{"type":"module","title":"Model Management - Gemini","doc":"```elixir\n# List available models\n{:ok, models} = Gemini.list_models()\n\n# Get model details\n{:ok, model_info} = Gemini.get_model(\"gemini-1.5-pro\")\n\n# Count tokens\n{:ok, token_count} = Gemini.count_tokens(\"Your text\", model: \"gemini-1.5-pro\")\n```\n\nThis module provides backward-compatible access to the Gemini API while routing\nrequests through the unified coordinator for maximum flexibility and performance.","ref":"Gemini.html#module-model-management"},{"type":"function","title":"Gemini.chat/1","doc":"Start a new chat session.","ref":"Gemini.html#chat/1"},{"type":"function","title":"Gemini.configure/2","doc":"Configure authentication for the client.","ref":"Gemini.html#configure/2"},{"type":"function","title":"Examples - Gemini.configure/2","doc":"# Gemini API\n    Gemini.configure(:gemini, %{api_key: \"your_api_key\"})\n\n    # Vertex AI\n    Gemini.configure(:vertex_ai, %{\n      service_account_key: \"/path/to/key.json\",\n      project_id: \"your-project\",\n      location: \"us-central1\"\n    })","ref":"Gemini.html#configure/2-examples"},{"type":"function","title":"Gemini.count_tokens/2","doc":"Count tokens in the given content.","ref":"Gemini.html#count_tokens/2"},{"type":"function","title":"Gemini.extract_text/1","doc":"Extract text from a GenerateContentResponse or raw streaming data.","ref":"Gemini.html#extract_text/1"},{"type":"function","title":"Gemini.generate/2","doc":"Generate content using the configured authentication.","ref":"Gemini.html#generate/2"},{"type":"function","title":"Gemini.get_model/1","doc":"Get information about a specific model.","ref":"Gemini.html#get_model/1"},{"type":"function","title":"Gemini.get_stream_status/1","doc":"Get stream status.","ref":"Gemini.html#get_stream_status/1"},{"type":"function","title":"Gemini.list_models/1","doc":"List available models.","ref":"Gemini.html#list_models/1"},{"type":"function","title":"Gemini.model_exists?/1","doc":"Check if a model exists.","ref":"Gemini.html#model_exists?/1"},{"type":"function","title":"Gemini.send_message/2","doc":"Send a message in a chat session.","ref":"Gemini.html#send_message/2"},{"type":"function","title":"Gemini.start_link/0","doc":"Start the streaming manager (for compatibility).","ref":"Gemini.html#start_link/0"},{"type":"function","title":"Gemini.start_stream/2","doc":"Start a managed streaming session.","ref":"Gemini.html#start_stream/2"},{"type":"function","title":"Gemini.stream_generate/2","doc":"Generate content with streaming response (synchronous collection).","ref":"Gemini.html#stream_generate/2"},{"type":"function","title":"Gemini.subscribe_stream/1","doc":"Subscribe to streaming events.","ref":"Gemini.html#subscribe_stream/1"},{"type":"function","title":"Gemini.text/2","doc":"Generate text content and return only the text.","ref":"Gemini.html#text/2"},{"type":"module","title":"Gemini.APIs.Coordinator","doc":"Coordinates API calls across different authentication strategies and endpoints.\n\nProvides a unified interface that can route requests to either Gemini API or Vertex AI\nbased on configuration, while maintaining the same interface.\n\nThis module acts as the main entry point for all Gemini API operations,\nautomatically handling authentication strategy selection and request routing.","ref":"Gemini.APIs.Coordinator.html"},{"type":"module","title":"Features - Gemini.APIs.Coordinator","doc":"- Unified API for content generation across auth strategies\n- Automatic auth strategy selection based on configuration\n- Per-request auth strategy override capability\n- Consistent error handling and response format\n- Support for both streaming and non-streaming operations\n- Model listing and token counting functionality","ref":"Gemini.APIs.Coordinator.html#module-features"},{"type":"module","title":"Usage - Gemini.APIs.Coordinator","doc":"# Use default auth strategy\n    {:ok, response} = Coordinator.generate_content(\"Hello world\")\n\n    # Override auth strategy for specific request\n    {:ok, response} = Coordinator.generate_content(\"Hello world\", auth: :vertex_ai)\n\n    # Start streaming with specific auth\n    {:ok, stream_id} = Coordinator.stream_generate_content(\"Tell me a story\", auth: :gemini)","ref":"Gemini.APIs.Coordinator.html#module-usage"},{"type":"type","title":"Gemini.APIs.Coordinator.api_result/1","doc":"","ref":"Gemini.APIs.Coordinator.html#t:api_result/1"},{"type":"type","title":"Gemini.APIs.Coordinator.auth_strategy/0","doc":"","ref":"Gemini.APIs.Coordinator.html#t:auth_strategy/0"},{"type":"function","title":"Gemini.APIs.Coordinator.count_tokens/2","doc":"Count tokens in the given input.","ref":"Gemini.APIs.Coordinator.html#count_tokens/2"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.count_tokens/2","doc":"- `input`: String or GenerateContentRequest to count tokens for\n- `opts`: Options including model and auth strategy","ref":"Gemini.APIs.Coordinator.html#count_tokens/2-parameters"},{"type":"function","title":"Options - Gemini.APIs.Coordinator.count_tokens/2","doc":"- `:model`: Model to use for token counting (defaults to \"gemini-2.0-flash\")\n- `:auth`: Authentication strategy (`:gemini` or `:vertex_ai`)","ref":"Gemini.APIs.Coordinator.html#count_tokens/2-options"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.count_tokens/2","doc":"{:ok, count} = Coordinator.count_tokens(\"Hello world\")\n    {:ok, count} = Coordinator.count_tokens(\"Complex text\", model: \"gemini-1.5-pro\", auth: :vertex_ai)","ref":"Gemini.APIs.Coordinator.html#count_tokens/2-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.extract_text/1","doc":"Extract text content from a GenerateContentResponse.","ref":"Gemini.APIs.Coordinator.html#extract_text/1"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.extract_text/1","doc":"{:ok, response} = Coordinator.generate_content(\"Hello\")\n    {:ok, text} = Coordinator.extract_text(response)","ref":"Gemini.APIs.Coordinator.html#extract_text/1-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.generate_content/2","doc":"Generate content using the specified model and input.","ref":"Gemini.APIs.Coordinator.html#generate_content/2"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.generate_content/2","doc":"- `input`: String prompt or GenerateContentRequest struct\n- `opts`: Options including model, auth strategy, and generation config","ref":"Gemini.APIs.Coordinator.html#generate_content/2-parameters"},{"type":"function","title":"Options - Gemini.APIs.Coordinator.generate_content/2","doc":"- `:model`: Model to use (defaults to \"gemini-2.0-flash\")\n- `:auth`: Authentication strategy (`:gemini` or `:vertex_ai`)\n- `:temperature`: Generation temperature (0.0-1.0)\n- `:max_output_tokens`: Maximum tokens to generate\n- `:top_p`: Top-p sampling parameter\n- `:top_k`: Top-k sampling parameter\n- `:safety_settings`: List of safety settings","ref":"Gemini.APIs.Coordinator.html#generate_content/2-options"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.generate_content/2","doc":"# Simple text generation\n    {:ok, response} = Coordinator.generate_content(\"What is AI?\")\n\n    # With specific model and auth\n    {:ok, response} = Coordinator.generate_content(\n      \"Explain quantum computing\",\n      model: \"gemini-2.0-flash\",\n      auth: :vertex_ai,\n      temperature: 0.7\n    )\n\n    # Using request struct\n    request = %GenerateContentRequest{...}\n    {:ok, response} = Coordinator.generate_content(request)","ref":"Gemini.APIs.Coordinator.html#generate_content/2-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.get_model/2","doc":"Get information about a specific model.","ref":"Gemini.APIs.Coordinator.html#get_model/2"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.get_model/2","doc":"- `model_name`: Name of the model to retrieve\n- `opts`: Options including auth strategy","ref":"Gemini.APIs.Coordinator.html#get_model/2-parameters"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.get_model/2","doc":"{:ok, model} = Coordinator.get_model(\"gemini-2.0-flash\")\n    {:ok, model} = Coordinator.get_model(\"gemini-1.5-pro\", auth: :vertex_ai)","ref":"Gemini.APIs.Coordinator.html#get_model/2-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.list_models/1","doc":"List available models for the specified authentication strategy.","ref":"Gemini.APIs.Coordinator.html#list_models/1"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.list_models/1","doc":"- `opts`: Options including auth strategy and pagination","ref":"Gemini.APIs.Coordinator.html#list_models/1-parameters"},{"type":"function","title":"Options - Gemini.APIs.Coordinator.list_models/1","doc":"- `:auth`: Authentication strategy (`:gemini` or `:vertex_ai`)\n- `:page_size`: Number of models per page\n- `:page_token`: Pagination token for next page","ref":"Gemini.APIs.Coordinator.html#list_models/1-options"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.list_models/1","doc":"# List models with default auth\n    {:ok, models_response} = Coordinator.list_models()\n\n    # List models with specific auth strategy\n    {:ok, models_response} = Coordinator.list_models(auth: :vertex_ai)\n\n    # With pagination\n    {:ok, models_response} = Coordinator.list_models(\n      auth: :gemini,\n      page_size: 50,\n      page_token: \"next_page_token\"\n    )","ref":"Gemini.APIs.Coordinator.html#list_models/1-examples"},{"type":"type","title":"Gemini.APIs.Coordinator.request_opts/0","doc":"","ref":"Gemini.APIs.Coordinator.html#t:request_opts/0"},{"type":"function","title":"Gemini.APIs.Coordinator.stop_stream/1","doc":"Stop a streaming content generation.","ref":"Gemini.APIs.Coordinator.html#stop_stream/1"},{"type":"function","title":"Gemini.APIs.Coordinator.stream_generate_content/2","doc":"Stream content generation with real-time response chunks.","ref":"Gemini.APIs.Coordinator.html#stream_generate_content/2"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.stream_generate_content/2","doc":"- `input`: String prompt or GenerateContentRequest struct  \n- `opts`: Options including model, auth strategy, and generation config","ref":"Gemini.APIs.Coordinator.html#stream_generate_content/2-parameters"},{"type":"function","title":"Returns - Gemini.APIs.Coordinator.stream_generate_content/2","doc":"- `{:ok, stream_id}`: Stream started successfully\n- `{:error, reason}`: Failed to start stream\n\nAfter starting the stream, subscribe to receive events:\n\n    {:ok, stream_id} = Coordinator.stream_generate_content(\"Tell me a story\")\n    :ok = Coordinator.subscribe_stream(stream_id)\n\n    # Handle incoming messages\n    receive do\n      {:stream_event, ^stream_id, event} -> \n        IO.inspect(event, label: \"Stream Event\")\n      {:stream_complete, ^stream_id} -> \n        IO.puts(\"Stream completed\")\n      {:stream_error, ^stream_id, stream_error} -> \n        IO.puts(\"Stream error: #{inspect(stream_error)}\")\n    end","ref":"Gemini.APIs.Coordinator.html#stream_generate_content/2-returns"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.stream_generate_content/2","doc":"# Basic streaming\n    {:ok, stream_id} = Coordinator.stream_generate_content(\"Write a poem\")\n\n    # With specific configuration\n    {:ok, stream_id} = Coordinator.stream_generate_content(\n      \"Explain machine learning\",\n      model: \"gemini-2.0-flash\",\n      auth: :gemini,\n      temperature: 0.8,\n      max_output_tokens: 1000\n    )","ref":"Gemini.APIs.Coordinator.html#stream_generate_content/2-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.stream_status/1","doc":"Get the status of a streaming content generation.","ref":"Gemini.APIs.Coordinator.html#stream_status/1"},{"type":"function","title":"Gemini.APIs.Coordinator.subscribe_stream/2","doc":"Subscribe to a streaming content generation.","ref":"Gemini.APIs.Coordinator.html#subscribe_stream/2"},{"type":"function","title":"Parameters - Gemini.APIs.Coordinator.subscribe_stream/2","doc":"- `stream_id`: ID of the stream to subscribe to\n- `subscriber_pid`: Process to receive stream events (defaults to current process)","ref":"Gemini.APIs.Coordinator.html#subscribe_stream/2-parameters"},{"type":"function","title":"Examples - Gemini.APIs.Coordinator.subscribe_stream/2","doc":"{:ok, stream_id} = Coordinator.stream_generate_content(\"Hello\")\n    :ok = Coordinator.subscribe_stream(stream_id)\n\n    # In a different process\n    :ok = Coordinator.subscribe_stream(stream_id, target_pid)","ref":"Gemini.APIs.Coordinator.html#subscribe_stream/2-examples"},{"type":"function","title":"Gemini.APIs.Coordinator.unsubscribe_stream/2","doc":"Unsubscribe from a streaming content generation.","ref":"Gemini.APIs.Coordinator.html#unsubscribe_stream/2"},{"type":"module","title":"Gemini.Auth","doc":"Authentication strategy behavior and implementations for Gemini and Vertex AI.\n\nThis module provides a unified interface for different authentication methods:\n- Gemini API: Simple API key authentication\n- Vertex AI: OAuth2/Service Account authentication","ref":"Gemini.Auth.html"},{"type":"type","title":"Gemini.Auth.auth_type/0","doc":"","ref":"Gemini.Auth.html#t:auth_type/0"},{"type":"function","title":"Gemini.Auth.authenticate/2","doc":"Authenticate using the given strategy and configuration.","ref":"Gemini.Auth.html#authenticate/2"},{"type":"function","title":"Gemini.Auth.base_url/2","doc":"Get base URL using the given strategy and configuration.","ref":"Gemini.Auth.html#base_url/2"},{"type":"function","title":"Gemini.Auth.build_headers/2","doc":"Build authenticated headers for the given strategy and credentials.","ref":"Gemini.Auth.html#build_headers/2"},{"type":"function","title":"Gemini.Auth.build_path/4","doc":"Build the full path for an API endpoint.","ref":"Gemini.Auth.html#build_path/4"},{"type":"type","title":"Gemini.Auth.credentials/0","doc":"","ref":"Gemini.Auth.html#t:credentials/0"},{"type":"function","title":"Gemini.Auth.get_base_url/2","doc":"Get the base URL for the given strategy and credentials.","ref":"Gemini.Auth.html#get_base_url/2"},{"type":"function","title":"Gemini.Auth.get_strategy/1","doc":"Get the appropriate authentication strategy based on configuration.","ref":"Gemini.Auth.html#get_strategy/1"},{"type":"function","title":"Gemini.Auth.refresh_credentials/2","doc":"Refresh credentials if needed (mainly for Vertex AI OAuth tokens).","ref":"Gemini.Auth.html#refresh_credentials/2"},{"type":"function","title":"Gemini.Auth.strategy/1","doc":"Get the appropriate authentication strategy based on configuration.\n(Alias for get_strategy/1 for backward compatibility)","ref":"Gemini.Auth.html#strategy/1"},{"type":"module","title":"Gemini.Auth.GeminiStrategy","doc":"Authentication strategy for Google Gemini API using API key.\n\nThis strategy uses the simple x-goog-api-key header authentication\nmethod used by the Gemini API.","ref":"Gemini.Auth.GeminiStrategy.html"},{"type":"function","title":"Gemini.Auth.GeminiStrategy.authenticate/1","doc":"Authenticate with Gemini API using API key.","ref":"Gemini.Auth.GeminiStrategy.html#authenticate/1"},{"type":"module","title":"Gemini.Auth.MultiAuthCoordinator","doc":"Coordinates multiple authentication strategies for concurrent usage.\n\nEnables per-request auth strategy selection while maintaining\nindependent credential management and request routing.\n\nThis module serves as the central coordination point for the Gemini Unified\nImplementation's multi-auth capability, allowing applications to use both\nGemini API and Vertex AI authentication strategies simultaneously.","ref":"Gemini.Auth.MultiAuthCoordinator.html"},{"type":"type","title":"Gemini.Auth.MultiAuthCoordinator.auth_result/0","doc":"","ref":"Gemini.Auth.MultiAuthCoordinator.html#t:auth_result/0"},{"type":"type","title":"Gemini.Auth.MultiAuthCoordinator.auth_strategy/0","doc":"","ref":"Gemini.Auth.MultiAuthCoordinator.html#t:auth_strategy/0"},{"type":"function","title":"Gemini.Auth.MultiAuthCoordinator.coordinate_auth/2","doc":"Coordinates authentication for the specified strategy.\n\nThis is the main entry point for multi-auth coordination. It routes\nauthentication requests to the appropriate strategy while maintaining\nindependent credential management.","ref":"Gemini.Auth.MultiAuthCoordinator.html#coordinate_auth/2"},{"type":"function","title":"Parameters - Gemini.Auth.MultiAuthCoordinator.coordinate_auth/2","doc":"- `strategy`: The authentication strategy (`:gemini` or `:vertex_ai`)\n- `opts`: Request options (may include configuration overrides)","ref":"Gemini.Auth.MultiAuthCoordinator.html#coordinate_auth/2-parameters"},{"type":"function","title":"Returns - Gemini.Auth.MultiAuthCoordinator.coordinate_auth/2","doc":"- `{:ok, strategy, headers}` on successful authentication\n- `{:error, reason}` on authentication failure","ref":"Gemini.Auth.MultiAuthCoordinator.html#coordinate_auth/2-returns"},{"type":"function","title":"Examples - Gemini.Auth.MultiAuthCoordinator.coordinate_auth/2","doc":"# Coordinate Gemini API authentication\n    {:ok, :gemini, headers} = MultiAuthCoordinator.coordinate_auth(:gemini, [])\n    \n    # Coordinate Vertex AI authentication\n    {:ok, :vertex_ai, headers} = MultiAuthCoordinator.coordinate_auth(:vertex_ai, [])\n    \n    # With configuration overrides\n    {:ok, :gemini, headers} = MultiAuthCoordinator.coordinate_auth(:gemini, [api_key: \"override\"])","ref":"Gemini.Auth.MultiAuthCoordinator.html#coordinate_auth/2-examples"},{"type":"type","title":"Gemini.Auth.MultiAuthCoordinator.credentials/0","doc":"","ref":"Gemini.Auth.MultiAuthCoordinator.html#t:credentials/0"},{"type":"function","title":"Gemini.Auth.MultiAuthCoordinator.get_credentials/2","doc":"Retrieves credentials for the specified authentication strategy.\n\nLoads credentials from configuration, with optional overrides from request options.","ref":"Gemini.Auth.MultiAuthCoordinator.html#get_credentials/2"},{"type":"function","title":"Parameters - Gemini.Auth.MultiAuthCoordinator.get_credentials/2","doc":"- `strategy`: The authentication strategy\n- `opts`: Optional configuration overrides","ref":"Gemini.Auth.MultiAuthCoordinator.html#get_credentials/2-parameters"},{"type":"function","title":"Returns - Gemini.Auth.MultiAuthCoordinator.get_credentials/2","doc":"- `{:ok, credentials}` on success\n- `{:error, reason}` on failure","ref":"Gemini.Auth.MultiAuthCoordinator.html#get_credentials/2-returns"},{"type":"function","title":"Gemini.Auth.MultiAuthCoordinator.refresh_credentials/1","doc":"Refreshes credentials for the specified authentication strategy.\n\nFor strategies that support credential refresh (like Vertex AI OAuth tokens),\nthis function will generate fresh credentials. For strategies that don't\nneed refresh (like Gemini API keys), it returns the existing credentials.","ref":"Gemini.Auth.MultiAuthCoordinator.html#refresh_credentials/1"},{"type":"function","title":"Parameters - Gemini.Auth.MultiAuthCoordinator.refresh_credentials/1","doc":"- `strategy`: The authentication strategy","ref":"Gemini.Auth.MultiAuthCoordinator.html#refresh_credentials/1-parameters"},{"type":"function","title":"Returns - Gemini.Auth.MultiAuthCoordinator.refresh_credentials/1","doc":"- `{:ok, refreshed_credentials}` on success\n- `{:error, reason}` on failure","ref":"Gemini.Auth.MultiAuthCoordinator.html#refresh_credentials/1-returns"},{"type":"type","title":"Gemini.Auth.MultiAuthCoordinator.request_opts/0","doc":"","ref":"Gemini.Auth.MultiAuthCoordinator.html#t:request_opts/0"},{"type":"type","title":"Gemini.Auth.MultiAuthCoordinator.t/0","doc":"","ref":"Gemini.Auth.MultiAuthCoordinator.html#t:t/0"},{"type":"function","title":"Gemini.Auth.MultiAuthCoordinator.validate_auth_config/1","doc":"Validates configuration for the specified authentication strategy.\n\nChecks that all required configuration is present and valid for the\ngiven strategy.","ref":"Gemini.Auth.MultiAuthCoordinator.html#validate_auth_config/1"},{"type":"function","title":"Parameters - Gemini.Auth.MultiAuthCoordinator.validate_auth_config/1","doc":"- `strategy`: The authentication strategy to validate","ref":"Gemini.Auth.MultiAuthCoordinator.html#validate_auth_config/1-parameters"},{"type":"function","title":"Returns - Gemini.Auth.MultiAuthCoordinator.validate_auth_config/1","doc":"- `:ok` if configuration is valid\n- `{:error, reason}` if configuration is invalid or missing","ref":"Gemini.Auth.MultiAuthCoordinator.html#validate_auth_config/1-returns"},{"type":"module","title":"Gemini.Auth.VertexStrategy","doc":"Authentication strategy for Google Vertex AI using OAuth2/Service Account.\n\nThis strategy supports multiple authentication methods:\n- Service Account JSON file (via VERTEX_JSON_FILE environment variable)\n- OAuth2 access tokens\n- Application Default Credentials (ADC)\n\nBased on the Vertex AI documentation, this strategy can generate self-signed JWTs\nfor authenticated endpoints and standard Bearer tokens for regular API calls.","ref":"Gemini.Auth.VertexStrategy.html"},{"type":"function","title":"Gemini.Auth.VertexStrategy.authenticate/1","doc":"Authenticate with Vertex AI using various methods.\n\nSupports the following authentication methods:\n- OAuth2 with project_id and location\n- Service Account with key file path\n- Service Account with key data\n- Direct access token","ref":"Gemini.Auth.VertexStrategy.html#authenticate/1"},{"type":"function","title":"Gemini.Auth.VertexStrategy.create_signed_jwt/4","doc":"Create a signed JWT for authenticated Vertex AI endpoints.\n\nThis is used for Vector Search endpoints with JWT authentication as described in v1.md.","ref":"Gemini.Auth.VertexStrategy.html#create_signed_jwt/4"},{"type":"function","title":"Parameters - Gemini.Auth.VertexStrategy.create_signed_jwt/4","doc":"- `service_account_email`: The service account email (issuer)\n- `audience`: The audience specified during index deployment\n- `credentials`: The credentials map containing authentication info\n- `opts`: Additional options for JWT creation","ref":"Gemini.Auth.VertexStrategy.html#create_signed_jwt/4-parameters"},{"type":"function","title":"Examples - Gemini.Auth.VertexStrategy.create_signed_jwt/4","doc":"iex> credentials = %{service_account_key: \"/path/to/key.json\"}\n    iex> {:ok, jwt} = Gemini.Auth.VertexStrategy.create_signed_jwt(\n    ...>   \"my-service@project.iam.gserviceaccount.com\",\n    ...>   \"my-app-audience\",\n    ...>   credentials\n    ...> )","ref":"Gemini.Auth.VertexStrategy.html#create_signed_jwt/4-examples"},{"type":"function","title":"Gemini.Auth.VertexStrategy.headers/1","doc":"Get authentication headers for Vertex AI requests.\n\nSupports multiple credential types:\n- %{access_token: token} - Direct access token\n- %{service_account_key: path} - Service account JSON file path\n- %{service_account_data: data} - Service account JSON data\n- %{jwt_token: token} - Pre-signed JWT token","ref":"Gemini.Auth.VertexStrategy.html#headers/1"},{"type":"module","title":"Gemini.SSE.Parser","doc":"Server-Sent Events (SSE) parser for streaming responses.\n\nHandles partial chunks and maintains state across multiple calls.\nProperly parses SSE format with incremental data.","ref":"Gemini.SSE.Parser.html"},{"type":"function","title":"Gemini.SSE.Parser.extract_text/1","doc":"Extract text content from a streaming event.","ref":"Gemini.SSE.Parser.html#extract_text/1"},{"type":"function","title":"Gemini.SSE.Parser.finalize/1","doc":"Finalize parsing and return any remaining events in buffer.\n\nCall this when the stream is complete to get any final partial events.","ref":"Gemini.SSE.Parser.html#finalize/1"},{"type":"function","title":"Gemini.SSE.Parser.new/0","doc":"Create a new SSE parser state.","ref":"Gemini.SSE.Parser.html#new/0"},{"type":"function","title":"Gemini.SSE.Parser.parse_chunk/2","doc":"Parse incoming SSE chunk and return events + updated state.","ref":"Gemini.SSE.Parser.html#parse_chunk/2"},{"type":"function","title":"Examples - Gemini.SSE.Parser.parse_chunk/2","doc":"iex> parser = SSE.Parser.new()\n    iex> chunk = \"data: {\\\"text\\\": \\\"hello\\\"}\n\n\"\n    iex> {:ok, events, new_parser} = SSE.Parser.parse_chunk(chunk, parser)\n    iex> length(events)\n    1","ref":"Gemini.SSE.Parser.html#parse_chunk/2-examples"},{"type":"type","title":"Gemini.SSE.Parser.parse_result/0","doc":"","ref":"Gemini.SSE.Parser.html#t:parse_result/0"},{"type":"function","title":"Gemini.SSE.Parser.stream_done?/1","doc":"Check if an event indicates the stream is done.","ref":"Gemini.SSE.Parser.html#stream_done?/1"},{"type":"type","title":"Gemini.SSE.Parser.t/0","doc":"","ref":"Gemini.SSE.Parser.html#t:t/0"},{"type":"module","title":"Gemini.Streaming.UnifiedManager","doc":"Unified streaming manager that supports multiple authentication strategies.\n\nThis manager extends the excellent ManagerV2 functionality with multi-auth support,\nallowing concurrent usage of both Gemini API and Vertex AI authentication strategies\nwithin the same application.\n\nFeatures:\n- All capabilities from ManagerV2 (HTTP streaming, resource management, etc.)\n- Multi-authentication strategy support via MultiAuthCoordinator\n- Per-stream authentication strategy selection\n- Concurrent usage of multiple auth strategies","ref":"Gemini.Streaming.UnifiedManager.html"},{"type":"type","title":"Gemini.Streaming.UnifiedManager.auth_strategy/0","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#t:auth_strategy/0"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Gemini.Streaming.UnifiedManager.html#child_spec/1"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.get_stats/0","doc":"Get manager statistics (ManagerV2 compatibility).","ref":"Gemini.Streaming.UnifiedManager.html#get_stats/0"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.get_stream_info/1","doc":"Get stream information (ManagerV2 compatibility).","ref":"Gemini.Streaming.UnifiedManager.html#get_stream_info/1"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.list_streams/0","doc":"List all active streams.","ref":"Gemini.Streaming.UnifiedManager.html#list_streams/0"},{"type":"type","title":"Gemini.Streaming.UnifiedManager.manager_state/0","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#t:manager_state/0"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.start_link/1","doc":"Start the unified streaming manager.","ref":"Gemini.Streaming.UnifiedManager.html#start_link/1"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.start_stream/3","doc":"Start a new stream.","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3"},{"type":"function","title":"API Variants - Gemini.Streaming.UnifiedManager.start_stream/3","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3-api-variants"},{"type":"function","title":"New API: start_stream(model, request_body, opts) - Gemini.Streaming.UnifiedManager.start_stream/3","doc":"- `model`: The model to use for generation\n- `request_body`: The request body for content generation\n- `opts`: Options including auth strategy and other config","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3-new-api-start_stream-model-request_body-opts"},{"type":"function","title":"Legacy API: start_stream(contents, opts, subscriber_pid) - ManagerV2 compatibility - Gemini.Streaming.UnifiedManager.start_stream/3","doc":"- `contents`: Content to stream (string or list of Content structs)\n- `opts`: Generation options (model, generation_config, etc.)\n- `subscriber_pid`: Process to receive stream events","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3-legacy-api-start_stream-contents-opts-subscriber_pid-managerv2-compatibility"},{"type":"function","title":"Options - Gemini.Streaming.UnifiedManager.start_stream/3","doc":"- `:auth`: Authentication strategy (`:gemini` or `:vertex_ai`)\n- `:timeout`: Request timeout in milliseconds\n- Other options passed to the streaming request","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3-options"},{"type":"function","title":"Examples - Gemini.Streaming.UnifiedManager.start_stream/3","doc":"# New API with Gemini auth\n    {:ok, stream_id} = UnifiedManager.start_stream(\n      \"gemini-2.0-flash\",\n      %{contents: [%{parts: [%{text: \"Hello\"}]}]},\n      auth: :gemini\n    )\n\n    # Legacy API for ManagerV2 compatibility\n    {:ok, stream_id} = UnifiedManager.start_stream(\"Hello\", [model: \"gemini-2.0-flash\"], self())","ref":"Gemini.Streaming.UnifiedManager.html#start_stream/3-examples"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.stop_stream/1","doc":"Stop a stream.","ref":"Gemini.Streaming.UnifiedManager.html#stop_stream/1"},{"type":"type","title":"Gemini.Streaming.UnifiedManager.stream_id/0","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#t:stream_id/0"},{"type":"type","title":"Gemini.Streaming.UnifiedManager.stream_state/0","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#t:stream_state/0"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.stream_status/1","doc":"Get the status of a stream.","ref":"Gemini.Streaming.UnifiedManager.html#stream_status/1"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.subscribe/2","doc":"Subscribe to a stream to receive events.","ref":"Gemini.Streaming.UnifiedManager.html#subscribe/2"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.subscribe_stream/2","doc":"Subscribe to stream events (ManagerV2 compatibility).","ref":"Gemini.Streaming.UnifiedManager.html#subscribe_stream/2"},{"type":"type","title":"Gemini.Streaming.UnifiedManager.subscriber_ref/0","doc":"","ref":"Gemini.Streaming.UnifiedManager.html#t:subscriber_ref/0"},{"type":"function","title":"Gemini.Streaming.UnifiedManager.unsubscribe/2","doc":"Unsubscribe from a stream.","ref":"Gemini.Streaming.UnifiedManager.html#unsubscribe/2"},{"type":"module","title":"Gemini.Client","doc":"Main client module that delegates to the appropriate HTTP client implementation.\n\nThis module provides a unified interface for making HTTP requests to the Gemini API,\nabstracting away the specific implementation details of the underlying HTTP client.","ref":"Gemini.Client.html"},{"type":"function","title":"Gemini.Client.get/2","doc":"Make a GET request using the configured authentication.","ref":"Gemini.Client.html#get/2"},{"type":"function","title":"Parameters - Gemini.Client.get/2","doc":"- `path` - The API path to request\n- `opts` - Optional keyword list of request options","ref":"Gemini.Client.html#get/2-parameters"},{"type":"function","title":"Returns - Gemini.Client.get/2","doc":"- `{:ok, response}` - Successful response\n- `{:error, Error.t()}` - Error details","ref":"Gemini.Client.html#get/2-returns"},{"type":"function","title":"Gemini.Client.post/3","doc":"Make a POST request using the configured authentication.","ref":"Gemini.Client.html#post/3"},{"type":"function","title":"Parameters - Gemini.Client.post/3","doc":"- `path` - The API path to request\n- `body` - The request body (will be JSON encoded)\n- `opts` - Optional keyword list of request options","ref":"Gemini.Client.html#post/3-parameters"},{"type":"function","title":"Returns - Gemini.Client.post/3","doc":"- `{:ok, response}` - Successful response\n- `{:error, Error.t()}` - Error details","ref":"Gemini.Client.html#post/3-returns"},{"type":"function","title":"Gemini.Client.request/5","doc":"Make an authenticated HTTP request.","ref":"Gemini.Client.html#request/5"},{"type":"function","title":"Parameters - Gemini.Client.request/5","doc":"- `method` - HTTP method (:get, :post, etc.)\n- `path` - The API path to request\n- `body` - The request body (nil for GET requests)\n- `auth_config` - Authentication configuration\n- `opts` - Optional keyword list of request options","ref":"Gemini.Client.html#request/5-parameters"},{"type":"function","title":"Returns - Gemini.Client.request/5","doc":"- `{:ok, response}` - Successful response\n- `{:error, Error.t()}` - Error details","ref":"Gemini.Client.html#request/5-returns"},{"type":"function","title":"Gemini.Client.stream_post/3","doc":"Stream a POST request for Server-Sent Events using configured authentication.","ref":"Gemini.Client.html#stream_post/3"},{"type":"function","title":"Parameters - Gemini.Client.stream_post/3","doc":"- `path` - The API path to request\n- `body` - The request body (will be JSON encoded)\n- `opts` - Optional keyword list of request options","ref":"Gemini.Client.html#stream_post/3-parameters"},{"type":"function","title":"Returns - Gemini.Client.stream_post/3","doc":"- `{:ok, events}` - Successful stream response with parsed events\n- `{:error, Error.t()}` - Error details","ref":"Gemini.Client.html#stream_post/3-returns"},{"type":"function","title":"Gemini.Client.stream_post_with_auth/4","doc":"Stream a POST request with specific authentication configuration.","ref":"Gemini.Client.html#stream_post_with_auth/4"},{"type":"function","title":"Parameters - Gemini.Client.stream_post_with_auth/4","doc":"- `path` - The API path to request\n- `body` - The request body (will be JSON encoded)\n- `auth_config` - Authentication configuration\n- `opts` - Optional keyword list of request options","ref":"Gemini.Client.html#stream_post_with_auth/4-parameters"},{"type":"function","title":"Returns - Gemini.Client.stream_post_with_auth/4","doc":"- `{:ok, events}` - Successful stream response with parsed events\n- `{:error, Error.t()}` - Error details","ref":"Gemini.Client.html#stream_post_with_auth/4-returns"},{"type":"module","title":"Gemini.Client.HTTP","doc":"Unified HTTP client for both Gemini and Vertex AI APIs using Req.\n\nSupports multiple authentication strategies and provides both\nregular and streaming request capabilities.","ref":"Gemini.Client.HTTP.html"},{"type":"function","title":"Gemini.Client.HTTP.get/2","doc":"Make a GET request using the configured authentication.","ref":"Gemini.Client.HTTP.html#get/2"},{"type":"function","title":"Gemini.Client.HTTP.post/3","doc":"Make a POST request using the configured authentication.","ref":"Gemini.Client.HTTP.html#post/3"},{"type":"function","title":"Gemini.Client.HTTP.request/5","doc":"Make an authenticated HTTP request.","ref":"Gemini.Client.HTTP.html#request/5"},{"type":"function","title":"Gemini.Client.HTTP.stream_post/3","doc":"Stream a POST request for Server-Sent Events using configured authentication.","ref":"Gemini.Client.HTTP.html#stream_post/3"},{"type":"function","title":"Gemini.Client.HTTP.stream_post_raw/4","doc":"Raw streaming POST with full URL (used by streaming manager).","ref":"Gemini.Client.HTTP.html#stream_post_raw/4"},{"type":"function","title":"Gemini.Client.HTTP.stream_post_with_auth/4","doc":"Stream a POST request with specific authentication configuration.","ref":"Gemini.Client.HTTP.html#stream_post_with_auth/4"},{"type":"module","title":"Gemini.Client.HTTPStreaming","doc":"HTTP client for streaming Server-Sent Events (SSE) from Gemini API.\n\nProvides proper streaming support with:\n- Incremental SSE parsing\n- Connection management\n- Error handling and retries\n- Backpressure support","ref":"Gemini.Client.HTTPStreaming.html"},{"type":"type","title":"Gemini.Client.HTTPStreaming.stream_callback/0","doc":"","ref":"Gemini.Client.HTTPStreaming.html#t:stream_callback/0"},{"type":"type","title":"Gemini.Client.HTTPStreaming.stream_event/0","doc":"","ref":"Gemini.Client.HTTPStreaming.html#t:stream_event/0"},{"type":"function","title":"Gemini.Client.HTTPStreaming.stream_sse/5","doc":"Start an SSE stream with a callback function.","ref":"Gemini.Client.HTTPStreaming.html#stream_sse/5"},{"type":"function","title":"Parameters - Gemini.Client.HTTPStreaming.stream_sse/5","doc":"- `url` - Full URL for the streaming endpoint\n- `headers` - HTTP headers including authentication\n- `body` - Request body (will be JSON encoded)\n- `callback` - Function called for each event\n- `opts` - Options including timeout, retry settings","ref":"Gemini.Client.HTTPStreaming.html#stream_sse/5-parameters"},{"type":"function","title":"Examples - Gemini.Client.HTTPStreaming.stream_sse/5","doc":"callback = fn\n      %{type: :data, data: data} ->\n        IO.puts(\"Received data\")\n        :ok\n      %{type: :complete} ->\n        IO.puts(\"Stream complete\")\n        :ok\n      %{type: :error, error: _error} ->\n        IO.puts(\"Stream error\")\n        :stop\n    end\n\n    HTTPStreaming.stream_sse(url, headers, body, callback)","ref":"Gemini.Client.HTTPStreaming.html#stream_sse/5-examples"},{"type":"function","title":"Gemini.Client.HTTPStreaming.stream_to_process/6","doc":"Start an SSE stream that sends events to a GenServer process.\n\nEvents are sent as messages: {:stream_event, stream_id, event}","ref":"Gemini.Client.HTTPStreaming.html#stream_to_process/6"},{"type":"module","title":"Gemini.Types.Content","doc":"Content type for Gemini API requests and responses.","ref":"Gemini.Types.Content.html"},{"type":"function","title":"Gemini.Types.Content.image/2","doc":"Create content with an image from a file path.","ref":"Gemini.Types.Content.html#image/2"},{"type":"function","title":"Gemini.Types.Content.multimodal/4","doc":"Create content with text and image.","ref":"Gemini.Types.Content.html#multimodal/4"},{"type":"type","title":"Gemini.Types.Content.parts/0","doc":"Ordered parts that constitute a single message.","ref":"Gemini.Types.Content.html#t:parts/0"},{"type":"type","title":"Gemini.Types.Content.role/0","doc":"The role of the content creator.","ref":"Gemini.Types.Content.html#t:role/0"},{"type":"type","title":"Gemini.Types.Content.t/0","doc":"","ref":"Gemini.Types.Content.html#t:t/0"},{"type":"function","title":"Gemini.Types.Content.text/2","doc":"Create content with text.","ref":"Gemini.Types.Content.html#text/2"},{"type":"module","title":"Gemini.Types.Response","doc":"Response types for the Gemini API.","ref":"Gemini.Types.Response.html"},{"type":"module","title":"Gemini.Config","doc":"Unified configuration management for both Gemini and Vertex AI authentication.\n\nSupports multiple authentication strategies:\n- Gemini: API key authentication\n- Vertex AI: OAuth2 or Service Account authentication","ref":"Gemini.Config.html"},{"type":"function","title":"Gemini.Config.api_key/0","doc":"Get the API key from environment or application config.\n(Legacy function for backward compatibility)","ref":"Gemini.Config.html#api_key/0"},{"type":"function","title":"Gemini.Config.auth_config/0","doc":"Get the authentication configuration.\n\nReturns a map with the authentication type and credentials.\nPriority order:\n1. Environment variables\n2. Application configuration\n3. Default to Gemini with API key","ref":"Gemini.Config.html#auth_config/0"},{"type":"type","title":"Gemini.Config.auth_config/0","doc":"","ref":"Gemini.Config.html#t:auth_config/0"},{"type":"function","title":"Gemini.Config.base_url/0","doc":"Get the base URL for the current authentication type.\n(Legacy function - now determined by auth strategy)","ref":"Gemini.Config.html#base_url/0"},{"type":"function","title":"Gemini.Config.default_model/0","doc":"Get the default model to use.","ref":"Gemini.Config.html#default_model/0"},{"type":"function","title":"Gemini.Config.detect_auth_type/0","doc":"Detect authentication type based on environment variables.","ref":"Gemini.Config.html#detect_auth_type/0"},{"type":"function","title":"Gemini.Config.detect_auth_type/1","doc":"Detect authentication type based on configuration map.","ref":"Gemini.Config.html#detect_auth_type/1"},{"type":"function","title":"Gemini.Config.get/0","doc":"Get configuration based on environment variables and application config.\nReturns a structured configuration map.","ref":"Gemini.Config.html#get/0"},{"type":"function","title":"Gemini.Config.get/1","doc":"Get configuration with overrides.","ref":"Gemini.Config.html#get/1"},{"type":"function","title":"Gemini.Config.get_auth_config/1","doc":"Get authentication configuration for a specific strategy.","ref":"Gemini.Config.html#get_auth_config/1"},{"type":"function","title":"Parameters - Gemini.Config.get_auth_config/1","doc":"- `strategy`: The authentication strategy (`:gemini` or `:vertex_ai`)","ref":"Gemini.Config.html#get_auth_config/1-parameters"},{"type":"function","title":"Returns - Gemini.Config.get_auth_config/1","doc":"- A map containing configuration for the specified strategy\n- Returns empty map if no configuration found","ref":"Gemini.Config.html#get_auth_config/1-returns"},{"type":"function","title":"Examples - Gemini.Config.get_auth_config/1","doc":"iex> Gemini.Config.get_auth_config(:gemini)\n    %{api_key: \"your_api_key\"}\n\n    iex> Gemini.Config.get_auth_config(:vertex_ai)\n    %{project_id: \"your-project\", location: \"us-central1\"}","ref":"Gemini.Config.html#get_auth_config/1-examples"},{"type":"function","title":"Gemini.Config.telemetry_enabled?/0","doc":"Check if telemetry is enabled.\n\nDetermines whether telemetry events should be emitted based on the\napplication configuration. Telemetry is enabled by default unless\nexplicitly disabled.","ref":"Gemini.Config.html#telemetry_enabled?/0"},{"type":"function","title":"Configuration - Gemini.Config.telemetry_enabled?/0","doc":"Set `:telemetry_enabled` to `false` in your application config to disable:\n\n    config :gemini, telemetry_enabled: false","ref":"Gemini.Config.html#telemetry_enabled?/0-configuration"},{"type":"function","title":"Returns - Gemini.Config.telemetry_enabled?/0","doc":"- `true` - Telemetry is enabled (default)\n- `false` - Telemetry is explicitly disabled","ref":"Gemini.Config.html#telemetry_enabled?/0-returns"},{"type":"function","title":"Examples - Gemini.Config.telemetry_enabled?/0","doc":"iex> # Default behavior (telemetry enabled)\n    iex> Gemini.Config.telemetry_enabled?()\n    true\n\n    iex> # Explicitly disabled\n    iex> Application.put_env(:gemini, :telemetry_enabled, false)\n    iex> Gemini.Config.telemetry_enabled?()\n    false\n\n    iex> # Any other value defaults to enabled\n    iex> Application.put_env(:gemini, :telemetry_enabled, :maybe)\n    iex> Gemini.Config.telemetry_enabled?()\n    true","ref":"Gemini.Config.html#telemetry_enabled?/0-examples"},{"type":"function","title":"Gemini.Config.timeout/0","doc":"Get HTTP timeout in milliseconds.","ref":"Gemini.Config.html#timeout/0"},{"type":"function","title":"Gemini.Config.validate!/0","doc":"Validate that required configuration is present.","ref":"Gemini.Config.html#validate!/0"},{"type":"module","title":"Gemini.Error","doc":"Standardized error structure for Gemini client.","ref":"Gemini.Error.html"},{"type":"function","title":"Gemini.Error.api_error/3","doc":"Create an API error from Gemini response.","ref":"Gemini.Error.html#api_error/3"},{"type":"type","title":"Gemini.Error.api_reason/0","doc":"API-specific error code or reason, if provided by Gemini.","ref":"Gemini.Error.html#t:api_reason/0"},{"type":"function","title":"Gemini.Error.config_error/2","doc":"Create a configuration error.","ref":"Gemini.Error.html#config_error/2"},{"type":"type","title":"Gemini.Error.error_details/0","doc":"Additional details or context about the error.","ref":"Gemini.Error.html#t:error_details/0"},{"type":"type","title":"Gemini.Error.error_message/0","doc":"A human-readable message describing the error.","ref":"Gemini.Error.html#t:error_message/0"},{"type":"type","title":"Gemini.Error.error_type/0","doc":"The type of error.","ref":"Gemini.Error.html#t:error_type/0"},{"type":"function","title":"Gemini.Error.http_error/3","doc":"Create an HTTP error.","ref":"Gemini.Error.html#http_error/3"},{"type":"type","title":"Gemini.Error.http_status/0","doc":"The HTTP status code, if the error originated from an HTTP response.","ref":"Gemini.Error.html#t:http_status/0"},{"type":"function","title":"Gemini.Error.invalid_response/2","doc":"Create an invalid response error.","ref":"Gemini.Error.html#invalid_response/2"},{"type":"function","title":"Gemini.Error.network_error/2","doc":"Create a network/connection error.","ref":"Gemini.Error.html#network_error/2"},{"type":"function","title":"Gemini.Error.new/3","doc":"Create a new error with type and message.","ref":"Gemini.Error.html#new/3"},{"type":"type","title":"Gemini.Error.original_error/0","doc":"The original error term, if this error is wrapping another.","ref":"Gemini.Error.html#t:original_error/0"},{"type":"function","title":"Gemini.Error.serialization_error/2","doc":"Create a JSON serialization/deserialization error.","ref":"Gemini.Error.html#serialization_error/2"},{"type":"type","title":"Gemini.Error.t/0","doc":"","ref":"Gemini.Error.html#t:t/0"},{"type":"function","title":"Gemini.Error.validation_error/2","doc":"Create a request validation error.","ref":"Gemini.Error.html#validation_error/2"},{"type":"module","title":"Gemini.Telemetry","doc":"Telemetry instrumentation helpers for Gemini library.\n\nThis module provides functions to emit telemetry events for requests, streaming,\nand other operations throughout the library. It supports the standard telemetry\nevents defined in the Gemini library specification:\n\n- `[:gemini, :request, :start]` - HTTP request started\n- `[:gemini, :request, :stop]` - HTTP request completed successfully\n- `[:gemini, :request, :exception]` - HTTP request failed with exception\n- `[:gemini, :stream, :start]` - Streaming request started\n- `[:gemini, :stream, :chunk]` - Streaming chunk received\n- `[:gemini, :stream, :stop]` - Streaming request completed\n- `[:gemini, :stream, :exception]` - Streaming request failed with exception\n\nAll telemetry events respect the global telemetry configuration and can be\ndisabled by setting `telemetry_enabled: false` in the application config.","ref":"Gemini.Telemetry.html"},{"type":"module","title":"Types - Gemini.Telemetry","doc":"The module works with several key data types for telemetry metadata and measurements.","ref":"Gemini.Telemetry.html#module-types"},{"type":"function","title":"Gemini.Telemetry.build_request_metadata/3","doc":"Build base metadata for HTTP requests with additional context.\n\nCreates a standardized metadata map for telemetry events related to\nHTTP requests, including URL, method, model, and other contextual information.","ref":"Gemini.Telemetry.html#build_request_metadata/3"},{"type":"function","title":"Parameters - Gemini.Telemetry.build_request_metadata/3","doc":"- `url` - The request URL\n- `method` - The HTTP method (atom)\n- `opts` - Optional keyword list with additional metadata","ref":"Gemini.Telemetry.html#build_request_metadata/3-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.build_request_metadata/3","doc":"A map containing standardized request metadata.","ref":"Gemini.Telemetry.html#build_request_metadata/3-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.build_request_metadata/3","doc":"iex> metadata = Gemini.Telemetry.build_request_metadata(\"/api/generate\", :post, model: \"gemini-pro\")\n    iex> metadata.url\n    \"/api/generate\"\n    iex> metadata.method\n    :post\n    iex> metadata.model\n    \"gemini-pro\"","ref":"Gemini.Telemetry.html#build_request_metadata/3-examples"},{"type":"function","title":"Gemini.Telemetry.build_stream_metadata/4","doc":"Build base metadata for streaming requests with additional context.\n\nCreates a standardized metadata map for telemetry events related to\nstreaming requests, including all standard request metadata plus\nstream-specific information like stream ID.","ref":"Gemini.Telemetry.html#build_stream_metadata/4"},{"type":"function","title":"Parameters - Gemini.Telemetry.build_stream_metadata/4","doc":"- `url` - The request URL\n- `method` - The HTTP method (atom)\n- `stream_id` - Unique identifier for the stream\n- `opts` - Optional keyword list with additional metadata","ref":"Gemini.Telemetry.html#build_stream_metadata/4-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.build_stream_metadata/4","doc":"A map containing standardized streaming metadata.","ref":"Gemini.Telemetry.html#build_stream_metadata/4-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.build_stream_metadata/4","doc":"iex> stream_id = \"abc123def456\"\n    iex> metadata = Gemini.Telemetry.build_stream_metadata(\"/api/stream\", :post, stream_id)\n    iex> metadata.stream_id\n    \"abc123def456\"\n    iex> metadata.url\n    \"/api/stream\"","ref":"Gemini.Telemetry.html#build_stream_metadata/4-examples"},{"type":"function","title":"Gemini.Telemetry.calculate_duration/1","doc":"Calculate duration in milliseconds from start time.\n\nComputes the elapsed time between a start time (in native units)\nand the current time, returning the duration in milliseconds.","ref":"Gemini.Telemetry.html#calculate_duration/1"},{"type":"function","title":"Parameters - Gemini.Telemetry.calculate_duration/1","doc":"- `start_time` - Start time in native time units (from `System.monotonic_time/0`)","ref":"Gemini.Telemetry.html#calculate_duration/1-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.calculate_duration/1","doc":"Duration in milliseconds as an integer.","ref":"Gemini.Telemetry.html#calculate_duration/1-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.calculate_duration/1","doc":"iex> start_time = System.monotonic_time()\n    iex> :timer.sleep(10)  # Sleep for 10ms\n    iex> duration = Gemini.Telemetry.calculate_duration(start_time)\n    iex> duration >= 10\n    true","ref":"Gemini.Telemetry.html#calculate_duration/1-examples"},{"type":"function","title":"Gemini.Telemetry.classify_contents/1","doc":"Classify content types for telemetry metadata.\n\nAnalyzes the content structure to determine if it contains only text,\nmultimodal data (text + images/other media), or unknown content types.\nThis classification helps with telemetry analysis and monitoring.","ref":"Gemini.Telemetry.html#classify_contents/1"},{"type":"function","title":"Parameters - Gemini.Telemetry.classify_contents/1","doc":"- `contents` - The content to classify (string, list, or other)","ref":"Gemini.Telemetry.html#classify_contents/1-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.classify_contents/1","doc":"- `:text` - For plain text content\n- `:multimodal` - For content containing non-text elements\n- `:unknown` - For unrecognized content types","ref":"Gemini.Telemetry.html#classify_contents/1-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.classify_contents/1","doc":"iex> Gemini.Telemetry.classify_contents(\"Hello world\")\n    :text\n\n    iex> Gemini.Telemetry.classify_contents([%{parts: [%{text: \"Hello\"}]}])\n    :text\n\n    iex> Gemini.Telemetry.classify_contents([%{parts: [%{text: \"Hello\"}, %{image: \"data\"}]}])\n    :multimodal\n\n    iex> Gemini.Telemetry.classify_contents(%{unknown: \"format\"})\n    :unknown","ref":"Gemini.Telemetry.html#classify_contents/1-examples"},{"type":"type","title":"Gemini.Telemetry.content_type/0","doc":"","ref":"Gemini.Telemetry.html#t:content_type/0"},{"type":"function","title":"Gemini.Telemetry.execute/3","doc":"Execute a telemetry event if telemetry is enabled.\n\nThis function conditionally emits telemetry events based on the global\ntelemetry configuration. If telemetry is disabled, the function returns\nimmediately without executing the event.","ref":"Gemini.Telemetry.html#execute/3"},{"type":"function","title":"Parameters - Gemini.Telemetry.execute/3","doc":"- `event` - A list of atoms representing the telemetry event name\n- `measurements` - A map of numeric measurements (e.g., duration, size)\n- `metadata` - A map of contextual information about the event","ref":"Gemini.Telemetry.html#execute/3-parameters"},{"type":"function","title":"Examples - Gemini.Telemetry.execute/3","doc":"iex> Gemini.Telemetry.execute([:gemini, :request, :start], %{}, %{url: \"/api\"})\n    :ok\n\n    iex> # When telemetry is disabled, no event is emitted\n    iex> Application.put_env(:gemini, :telemetry_enabled, false)\n    iex> Gemini.Telemetry.execute([:gemini, :request, :start], %{}, %{})\n    :ok","ref":"Gemini.Telemetry.html#execute/3-examples"},{"type":"function","title":"Gemini.Telemetry.extract_model/1","doc":"Extract model name from options or use default.\n\nRetrieves the model name from a keyword list of options, falling back\nto the system default model if not specified.","ref":"Gemini.Telemetry.html#extract_model/1"},{"type":"function","title":"Parameters - Gemini.Telemetry.extract_model/1","doc":"- `opts` - Keyword list of options that may contain a `:model` key","ref":"Gemini.Telemetry.html#extract_model/1-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.extract_model/1","doc":"The model name as a string.","ref":"Gemini.Telemetry.html#extract_model/1-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.extract_model/1","doc":"iex> Gemini.Telemetry.extract_model(model: \"gemini-pro\")\n    \"gemini-pro\"\n\n    iex> Gemini.Telemetry.extract_model([])\n    \"gemini-2.0-flash\"  # default model\n\n    iex> Gemini.Telemetry.extract_model(\"not a keyword list\")\n    \"gemini-2.0-flash\"  # fallback to default","ref":"Gemini.Telemetry.html#extract_model/1-examples"},{"type":"function","title":"Gemini.Telemetry.generate_stream_id/0","doc":"Generate unique stream IDs for telemetry tracking.\n\nCreates a cryptographically secure random identifier for tracking\nstreaming operations across multiple telemetry events.","ref":"Gemini.Telemetry.html#generate_stream_id/0"},{"type":"function","title":"Returns - Gemini.Telemetry.generate_stream_id/0","doc":"A 16-character lowercase hexadecimal string representing a unique stream ID.","ref":"Gemini.Telemetry.html#generate_stream_id/0-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.generate_stream_id/0","doc":"iex> stream_id = Gemini.Telemetry.generate_stream_id()\n    iex> is_binary(stream_id) and byte_size(stream_id) == 16\n    true\n\n    iex> # Stream IDs should be unique\n    iex> id1 = Gemini.Telemetry.generate_stream_id()\n    iex> id2 = Gemini.Telemetry.generate_stream_id()\n    iex> id1 != id2\n    true","ref":"Gemini.Telemetry.html#generate_stream_id/0-examples"},{"type":"function","title":"Gemini.Telemetry.has_non_text_parts?/1","doc":"Check if content has non-text parts (for multimodal classification).\n\nExamines a content structure to determine if it contains any non-text\nelements such as images, audio, or other media types.","ref":"Gemini.Telemetry.html#has_non_text_parts?/1"},{"type":"function","title":"Parameters - Gemini.Telemetry.has_non_text_parts?/1","doc":"- `content` - A content structure with parts to examine","ref":"Gemini.Telemetry.html#has_non_text_parts?/1-parameters"},{"type":"function","title":"Returns - Gemini.Telemetry.has_non_text_parts?/1","doc":"- `true` - If the content contains non-text parts\n- `false` - If the content contains only text or is not recognized","ref":"Gemini.Telemetry.html#has_non_text_parts?/1-returns"},{"type":"function","title":"Examples - Gemini.Telemetry.has_non_text_parts?/1","doc":"iex> Gemini.Telemetry.has_non_text_parts?(%{parts: [%{text: \"Hello\"}]})\n    false\n\n    iex> Gemini.Telemetry.has_non_text_parts?(%{parts: [%{text: \"Hello\"}, %{image: \"data\"}]})\n    true\n\n    iex> Gemini.Telemetry.has_non_text_parts?(\"not a content structure\")\n    false","ref":"Gemini.Telemetry.html#has_non_text_parts?/1-examples"},{"type":"type","title":"Gemini.Telemetry.http_method/0","doc":"","ref":"Gemini.Telemetry.html#t:http_method/0"},{"type":"type","title":"Gemini.Telemetry.stream_id/0","doc":"","ref":"Gemini.Telemetry.html#t:stream_id/0"},{"type":"type","title":"Gemini.Telemetry.telemetry_event/0","doc":"","ref":"Gemini.Telemetry.html#t:telemetry_event/0"},{"type":"type","title":"Gemini.Telemetry.telemetry_measurements/0","doc":"","ref":"Gemini.Telemetry.html#t:telemetry_measurements/0"},{"type":"type","title":"Gemini.Telemetry.telemetry_metadata/0","doc":"","ref":"Gemini.Telemetry.html#t:telemetry_metadata/0"},{"type":"extras","title":"Gemini Elixir Client","doc":"# Gemini Elixir Client\n\n[![CI](https://github.com/nshkrdotcom/gemini_ex/actions/workflows/elixir.yaml/badge.svg)](https://github.com/nshkrdotcom/gemini_ex/actions/workflows/elixir.yaml)\n[![Elixir](https://img.shields.io/badge/elixir-1.18.3-purple.svg)](https://elixir-lang.org)\n[![OTP](https://img.shields.io/badge/otp-27.3.3-blue.svg)](https://www.erlang.org)\n[![Hex.pm](https://img.shields.io/hexpm/v/gemini.svg)](https://hex.pm/packages/gemini)\n[![Documentation](https://img.shields.io/badge/docs-hexdocs-purple.svg)](https://hexdocs.pm/gemini)\n[![License](https://img.shields.io/badge/license-MIT-green.svg)](https://github.com/nshkrdotcom/gemini_ex/blob/main/LICENSE)\n\nA comprehensive Elixir client for Google's Gemini AI API with dual authentication support, advanced streaming capabilities, type safety, and built-in telemetry.","ref":"readme.html"},{"type":"extras","title":"✨ Features - Gemini Elixir Client","doc":"- **🔐 Dual Authentication**: Seamless support for both Gemini API keys and Vertex AI OAuth/Service Accounts\n- **⚡ Advanced Streaming**: Production-grade Server-Sent Events streaming with real-time processing\n- **🛡️ Type Safety**: Complete type definitions with runtime validation\n- **📊 Built-in Telemetry**: Comprehensive observability and metrics out of the box\n- **💬 Chat Sessions**: Multi-turn conversation management with state persistence\n- **🎭 Multimodal**: Full support for text, image, audio, and video content\n- **🚀 Production Ready**: Robust error handling, retry logic, and performance optimizations\n- **🔧 Flexible Configuration**: Environment variables, application config, and per-request overrides","ref":"readme.html#features"},{"type":"extras","title":"📦 Installation - Gemini Elixir Client","doc":"Add `gemini` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:gemini, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"🚀 Quick Start - Gemini Elixir Client","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Basic Configuration - Gemini Elixir Client","doc":"Configure your API key in `config/runtime.exs`:\n\n```elixir\nimport Config\n\nconfig :gemini,\n  api_key: System.get_env(\"GEMINI_API_KEY\")\n```\n\nOr set the environment variable:\n\n```bash\nexport GEMINI_API_KEY=\"your_api_key_here\"\n```","ref":"readme.html#basic-configuration"},{"type":"extras","title":"Simple Content Generation - Gemini Elixir Client","doc":"```elixir\n# Basic text generation\n{:ok, response} = Gemini.generate(\"Tell me about Elixir programming\")\n{:ok, text} = Gemini.extract_text(response)\nIO.puts(text)\n\n# With options\n{:ok, response} = Gemini.generate(\"Explain quantum computing\", [\n  model: \"gemini-1.5-pro\",\n  temperature: 0.7,\n  max_output_tokens: 1000\n])\n```","ref":"readme.html#simple-content-generation"},{"type":"extras","title":"Advanced Streaming - Gemini Elixir Client","doc":"```elixir\n# Start a streaming session\n{:ok, stream_id} = Gemini.stream_generate(\"Write a long story about AI\", [\n  on_chunk: fn chunk -> IO.write(chunk) end,\n  on_complete: fn -> IO.puts(\"\\n✅ Stream complete!\") end,\n  on_error: fn error -> IO.puts(\"❌ Error: #{inspect(error)}\") end\n])\n\n# Stream management\nGemini.Streaming.pause_stream(stream_id)\nGemini.Streaming.resume_stream(stream_id)\nGemini.Streaming.stop_stream(stream_id)\n```","ref":"readme.html#advanced-streaming"},{"type":"extras","title":"Multi-turn Conversations - Gemini Elixir Client","doc":"```elixir\n# Create a chat session\n{:ok, session} = Gemini.create_chat_session([\n  model: \"gemini-1.5-pro\",\n  system_instruction: \"You are a helpful programming assistant.\"\n])\n\n# Send messages\n{:ok, response1} = Gemini.send_message(session, \"What is functional programming?\")\n{:ok, response2} = Gemini.send_message(session, \"Show me an example in Elixir\")\n\n# Get conversation history\nhistory = Gemini.get_conversation_history(session)\n```","ref":"readme.html#multi-turn-conversations"},{"type":"extras","title":"🔐 Authentication - Gemini Elixir Client","doc":"","ref":"readme.html#authentication"},{"type":"extras","title":"Gemini API Key (Recommended for Development) - Gemini Elixir Client","doc":"```elixir\n# Environment variable (recommended)\nexport GEMINI_API_KEY=\"your_api_key\"\n\n# Application config\nconfig :gemini, api_key: \"your_api_key\"\n\n# Per-request override\nGemini.generate(\"Hello\", api_key: \"specific_key\")\n```","ref":"readme.html#gemini-api-key-recommended-for-development"},{"type":"extras","title":"Vertex AI (Recommended for Production) - Gemini Elixir Client","doc":"```elixir\n# Service Account JSON file\nexport VERTEX_SERVICE_ACCOUNT=\"/path/to/service-account.json\"\nexport VERTEX_PROJECT_ID=\"your-gcp-project\"\nexport VERTEX_LOCATION=\"us-central1\"\n\n# Application config\nconfig :gemini, :auth,\n  type: :vertex_ai,\n  credentials: %{\n    service_account_key: System.get_env(\"VERTEX_SERVICE_ACCOUNT\"),\n    project_id: System.get_env(\"VERTEX_PROJECT_ID\"),\n    location: \"us-central1\"\n  }\n```","ref":"readme.html#vertex-ai-recommended-for-production"},{"type":"extras","title":"📚 Documentation - Gemini Elixir Client","doc":"- **[API Reference](https://hexdocs.pm/gemini)** - Complete function documentation\n- **[Architecture Guide](https://hexdocs.pm/gemini/architecture.html)** - System design and components\n- **[Authentication System](https://hexdocs.pm/gemini/authentication_system.html)** - Detailed auth configuration\n- **[Examples](https://github.com/nshkrdotcom/gemini_ex/tree/main/examples)** - Working code examples","ref":"readme.html#documentation"},{"type":"extras","title":"🏗️ Architecture - Gemini Elixir Client","doc":"The library features a modular, layered architecture:\n\n- **Authentication Layer**: Multi-strategy auth with automatic credential resolution\n- **Coordination Layer**: Unified API coordinator for all operations\n- **Streaming Layer**: Advanced SSE processing with state management\n- **HTTP Layer**: Dual client system for standard and streaming requests\n- **Type Layer**: Comprehensive schemas with runtime validation","ref":"readme.html#architecture"},{"type":"extras","title":"🔧 Advanced Usage - Gemini Elixir Client","doc":"","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Custom Model Configuration - Gemini Elixir Client","doc":"```elixir\n# List available models\n{:ok, models} = Gemini.list_models()\n\n# Get model details\n{:ok, model_info} = Gemini.get_model(\"gemini-1.5-pro\")\n\n# Count tokens\n{:ok, token_count} = Gemini.count_tokens(\"Your text here\", model: \"gemini-1.5-pro\")\n```","ref":"readme.html#custom-model-configuration"},{"type":"extras","title":"Multimodal Content - Gemini Elixir Client","doc":"```elixir\n# Text with images\ncontent = [\n  %{type: \"text\", text: \"What's in this image?\"},\n  %{type: \"image\", source: %{type: \"base64\", data: base64_image}}\n]\n\n{:ok, response} = Gemini.generate(content)\n```","ref":"readme.html#multimodal-content"},{"type":"extras","title":"Error Handling - Gemini Elixir Client","doc":"```elixir\ncase Gemini.generate(\"Hello world\") do\n  {:ok, response} -> \n    # Handle success\n    {:ok, text} = Gemini.extract_text(response)\n    \n  {:error, %Gemini.Error{type: :rate_limit} = error} -> \n    # Handle rate limiting\n    IO.puts(\"Rate limited. Retry after: #{error.retry_after}\")\n    \n  {:error, %Gemini.Error{type: :authentication} = error} -> \n    # Handle auth errors\n    IO.puts(\"Auth error: #{error.message}\")\n    \n  {:error, error} -> \n    # Handle other errors\n    IO.puts(\"Unexpected error: #{inspect(error)}\")\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"🧪 Testing - Gemini Elixir Client","doc":"```bash\n# Run all tests\nmix test\n\n# Run with coverage\nmix test --cover\n\n# Run integration tests (requires API key)\nGEMINI_API_KEY=\"your_key\" mix test --only integration\n```","ref":"readme.html#testing"},{"type":"extras","title":"🤝 Contributing - Gemini Elixir Client","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Commit your changes (`git commit -m 'Add amazing feature'`)\n4. Push to the branch (`git push origin feature/amazing-feature`)\n5. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"📄 License - Gemini Elixir Client","doc":"This project is licensed under the MIT License - see the [LICENSE](https://github.com/nshkrdotcom/gemini_ex/blob/main/LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"🙏 Acknowledgments - Gemini Elixir Client","doc":"- Google AI team for the Gemini API\n- Elixir community for excellent tooling and libraries\n- Contributors and maintainers","ref":"readme.html#acknowledgments"},{"type":"extras","title":"Gemini Unified Client Architecture","doc":"# Gemini Unified Client Architecture\n\nThis document provides a high-level overview of the Gemini Unified Client architecture based on the codebase in `lib/gemini`.","ref":"architecture.html"},{"type":"extras","title":"System Overview - Gemini Unified Client Architecture","doc":"The Gemini Unified Client is designed as a modular, extensible system that provides a unified interface for interacting with both Gemini API and Vertex AI services while maintaining backward compatibility and supporting advanced features like streaming, multi-authentication, and telemetry.","ref":"architecture.html#system-overview"},{"type":"extras","title":"High-Level Architecture Diagram - Gemini Unified Client Architecture","doc":"```mermaid\ngraph LR\n    subgraph \"Application Layer\"\n        APP[Application Code]\n        MAIN[Gemini Main Module]\n    end\n\n    subgraph \"Coordination Layer\"\n        COORD[APIs.Coordinator]\n        MAUTH[Auth.MultiAuthCoordinator]\n    end\n\n    subgraph \"Client Layer\"\n        UC[Client.UnifiedClient]\n        HTTP[Client.HTTP]\n        STREAM[Client.HTTPStreaming]\n    end\n\n    subgraph \"API Layer\"\n        GEN[APIs.Generate]\n        EGEN[APIs.EnhancedGenerate]\n        MOD[APIs.Models]\n        EMOD[APIs.EnhancedModels]\n        TOK[APIs.Tokens]\n    end\n\n    subgraph \"Streaming System\"\n        UMAN[Streaming.UnifiedManager]\n        MAN[Streaming.Manager]\n        MANV2[Streaming.ManagerV2]\n        SSE[SSE.Parser]\n    end\n\n    subgraph \"Authentication System\"\n        AUTH[Auth]\n        GMAUTH[Gemini API Auth]\n        VAAUTH[Vertex AI Auth]\n    end\n\n    subgraph \"Infrastructure\"\n        CONF[Config]\n        ERR[Error]\n        TEL[Telemetry]\n        TYPES[Types/*]\n        UTILS[Utils/*]\n    end\n\n    subgraph \"Application Supervision\"\n        APPSUP[Application]\n        SUP[Supervisor]\n    end\n\n    subgraph \"External Services\"\n        GAPI[Gemini API]\n        VAI[Vertex AI]\n    end\n\n    %% Application Flow\n    APP --> MAIN\n    MAIN --> COORD\n\n    %% Coordination Layer\n    COORD --> MAUTH\n    COORD --> UC\n    COORD --> UMAN\n\n    %% Authentication Coordination\n    MAUTH --> GMAUTH\n    MAUTH --> VAAUTH\n\n    %% Client Layer\n    UC --> HTTP\n    UC --> STREAM\n    UC --> AUTH\n\n    %% API Layer Routing\n    COORD --> GEN\n    COORD --> EGEN\n    COORD --> MOD\n    COORD --> EMOD\n    COORD --> TOK\n\n    %% Streaming Management\n    UMAN --> MAN\n    UMAN --> MANV2\n    UMAN --> SSE\n    UMAN --> STREAM\n\n    %% Infrastructure Dependencies\n    UC --> CONF\n    UC --> ERR\n    UC --> TEL\n    COORD --> TYPES\n    AUTH --> CONF\n    UMAN --> TEL\n\n    %% Supervision Tree\n    APPSUP --> SUP\n    SUP --> UMAN\n\n    %% External Connections\n    HTTP --> GAPI\n    HTTP --> VAI\n    STREAM --> GAPI\n    STREAM --> VAI\n\n    %% Styling\n    classDef primary fill:#6B46C1,stroke:#4C1D95,stroke-width:3px,color:#FFFFFF\n    classDef secondary fill:#9333EA,stroke:#6B21A8,stroke-width:2px,color:#FFFFFF\n    classDef tertiary fill:#A855F7,stroke:#7C2D12,stroke-width:2px,color:#FFFFFF\n    classDef api fill:#EF4444,stroke:#B91C1C,stroke-width:2px,color:#FFFFFF\n    classDef coordinator fill:#10B981,stroke:#047857,stroke-width:2px,color:#FFFFFF\n    classDef strategy fill:#F59E0B,stroke:#D97706,stroke-width:2px,color:#FFFFFF\n    classDef config fill:#3B82F6,stroke:#1D4ED8,stroke-width:2px,color:#FFFFFF\n    classDef behavior fill:#8B5CF6,stroke:#7C3AED,stroke-width:3px,color:#FFFFFF,stroke-dasharray: 5 5\n\n    %% Apply classes\n    class APP,MAIN primary\n    class COORD,MAUTH coordinator\n    class UC,HTTP,STREAM secondary\n    class GEN,EGEN,MOD,EMOD,TOK tertiary\n    class UMAN,MAN,MANV2,SSE secondary\n    class AUTH behavior\n    class GMAUTH,VAAUTH strategy\n    class CONF,ERR,TEL,TYPES,UTILS config\n    class APPSUP,SUP primary\n    class GAPI,VAI api\n\n\n```","ref":"architecture.html#high-level-architecture-diagram"},{"type":"extras","title":"Core Components - Gemini Unified Client Architecture","doc":"","ref":"architecture.html#core-components"},{"type":"extras","title":"1. Application Layer - Gemini Unified Client Architecture","doc":"- **Gemini Main Module**: Primary entry point providing backward-compatible API\n- **Application Code**: User-facing interface for all Gemini operations","ref":"architecture.html#1-application-layer"},{"type":"extras","title":"2. Coordination Layer - Gemini Unified Client Architecture","doc":"- **APIs.Coordinator**: Central orchestrator that routes requests and manages unified API operations\n- **Auth.MultiAuthCoordinator**: Manages multiple authentication strategies concurrently","ref":"architecture.html#2-coordination-layer"},{"type":"extras","title":"3. Client Layer - Gemini Unified Client Architecture","doc":"- **Client.UnifiedClient**: Unified HTTP client with comprehensive error handling and response parsing\n- **Client.HTTP**: Standard HTTP client for request/response operations\n- **Client.HTTPStreaming**: Specialized client for streaming operations","ref":"architecture.html#3-client-layer"},{"type":"extras","title":"4. API Layer - Gemini Unified Client Architecture","doc":"- **APIs.Generate**: Core content generation functionality\n- **APIs.EnhancedGenerate**: Enhanced generation with additional features\n- **APIs.Models**: Model listing and management\n- **APIs.EnhancedModels**: Enhanced model operations\n- **APIs.Tokens**: Token counting and management","ref":"architecture.html#4-api-layer"},{"type":"extras","title":"5. Streaming System - Gemini Unified Client Architecture","doc":"- **Streaming.UnifiedManager**: Main streaming manager with multi-auth support\n- **Streaming.Manager**: Base streaming manager\n- **Streaming.ManagerV2**: Enhanced streaming manager\n- **SSE.Parser**: Server-Sent Events parser for streaming responses","ref":"architecture.html#5-streaming-system"},{"type":"extras","title":"6. Authentication System - Gemini Unified Client Architecture","doc":"- **Auth**: Base authentication module\n- **Multi-Authentication Support**: Concurrent support for both Gemini API and Vertex AI authentication","ref":"architecture.html#6-authentication-system"},{"type":"extras","title":"7. Infrastructure - Gemini Unified Client Architecture","doc":"- **Config**: Configuration management\n- **Error**: Error handling and classification\n- **Telemetry**: Instrumentation and monitoring\n- **Types**: Type definitions and data structures\n- **Utils**: Utility functions and helpers","ref":"architecture.html#7-infrastructure"},{"type":"extras","title":"Key Architectural Principles - Gemini Unified Client Architecture","doc":"","ref":"architecture.html#key-architectural-principles"},{"type":"extras","title":"1. Unified Interface - Gemini Unified Client Architecture","doc":"The system provides a single, consistent API that abstracts away the differences between Gemini API and Vertex AI, allowing users to switch authentication strategies without changing their code.","ref":"architecture.html#1-unified-interface"},{"type":"extras","title":"2. Multi-Authentication Support - Gemini Unified Client Architecture","doc":"The architecture supports concurrent usage of multiple authentication strategies within the same application, enabling flexible deployment scenarios.","ref":"architecture.html#2-multi-authentication-support"},{"type":"extras","title":"3. Modular Design - Gemini Unified Client Architecture","doc":"Each layer has clearly defined responsibilities and interfaces, making the system maintainable and extensible.","ref":"architecture.html#3-modular-design"},{"type":"extras","title":"4. Streaming-First Architecture - Gemini Unified Client Architecture","doc":"Streaming capabilities are built into the core architecture rather than being an afterthought, providing robust real-time functionality.","ref":"architecture.html#4-streaming-first-architecture"},{"type":"extras","title":"5. Comprehensive Error Handling - Gemini Unified Client Architecture","doc":"Error handling is centralized and consistent across all components, providing clear error classification and recovery strategies.","ref":"architecture.html#5-comprehensive-error-handling"},{"type":"extras","title":"6. Telemetry Integration - Gemini Unified Client Architecture","doc":"Built-in telemetry support provides visibility into system performance and behavior across all operations.","ref":"architecture.html#6-telemetry-integration"},{"type":"extras","title":"Data Flow - Gemini Unified Client Architecture","doc":"1. **Request Initiation**: Application code calls the main Gemini module\n2. **Coordination**: The Coordinator determines the appropriate authentication strategy and API endpoint\n3. **Authentication**: MultiAuthCoordinator handles authentication for the selected strategy\n4. **Client Execution**: UnifiedClient executes the HTTP request with proper authentication\n5. **Response Processing**: Responses are parsed, validated, and returned through the coordination layer\n6. **Streaming Handling**: For streaming requests, UnifiedManager manages the stream lifecycle\n7. **Telemetry**: All operations emit telemetry events for monitoring and observability","ref":"architecture.html#data-flow"},{"type":"extras","title":"Supervision Strategy - Gemini Unified Client Architecture","doc":"The system uses OTP supervision principles with the Application module starting a supervisor that manages the UnifiedManager for streaming operations, ensuring fault tolerance and automatic recovery.","ref":"architecture.html#supervision-strategy"},{"type":"extras","title":"Configuration Management - Gemini Unified Client Architecture","doc":"The Config module provides centralized configuration management that supports both global settings and per-request overrides, enabling flexible deployment and testing scenarios.","ref":"architecture.html#configuration-management"},{"type":"extras","title":"AUTHENTICATION_SYSTEM","doc":"Of course. Here is a detailed technical documentation for the authentication system of the provided Elixir library, complete with a Mermaid diagram to visualize the architecture.\n\n---","ref":"authentication_system.html"},{"type":"extras","title":"Technical Documentation: Gemini Elixir Library Authentication System - AUTHENTICATION_SYSTEM","doc":"","ref":"authentication_system.html#technical-documentation-gemini-elixir-library-authentication-system"},{"type":"extras","title":"1. Overview - AUTHENTICATION_SYSTEM","doc":"The authentication system in this Elixir library is a sophisticated, multi-layered architecture designed for maximum flexibility and robustness. It allows developers to seamlessly switch between different Google AI authentication schemes without changing their application code.\n\nThe system is built on two core principles:\n\n1.  **Dual-Strategy Support:** It natively supports both the simple **Gemini API Key** (`:gemini`) and the more complex **Vertex AI OAuth/JWT** (`:vertex_ai`) authentication methods.\n2.  **Hierarchical Configuration:** It provides a clear and predictable order of precedence for sourcing credentials, prioritizing per-request overrides, then environment variables, and finally Elixir application configuration.\n\nThis design makes the library easy to use for simple scripts while being powerful enough for complex, multi-tenant production applications.","ref":"authentication_system.html#1-overview"},{"type":"extras","title":"2. Core Concepts: Authentication Strategies - AUTHENTICATION_SYSTEM","doc":"The system revolves around two distinct, pluggable strategies:\n\n*   **:gemini Strategy**:\n    *   **Method**: Uses a static API key.\n    *   **Mechanism**: Adds the key to the `x-goog-api-key` HTTP header.\n    *   **Use Case**: Ideal for quickstarts, personal projects, and scenarios where a simple, long-lived key is sufficient.\n\n*   **:vertex_ai Strategy**:\n    *   **Method**: Uses Google Cloud's standard Identity and Access Management (IAM).\n    *   **Mechanism**: Generates short-lived OAuth2 Bearer tokens or self-signed JWTs from a service account.\n    *   **Use Case**: The standard for production Google Cloud Platform applications, providing more secure, auditable, and role-based access.","ref":"authentication_system.html#2-core-concepts-authentication-strategies"},{"type":"extras","title":"3. System Architecture - AUTHENTICATION_SYSTEM","doc":"The authentication system is composed of several distinct layers, each with a clear responsibility. This separation of concerns is key to its flexibility.\n\n#### Mermaid Diagram of the Authentication Architecture\n\n```mermaid\ngraph TD\n    subgraph S1 [\"User-Facing API\"]\n        A[\"User Call e.g., Coordinator.stream_generate_content(..., opts)\"]\n    end\n\n    subgraph S2 [\"Coordination & Logic\"]\n        B[\"APIs.Coordinator\"]\n        C[\"Streaming.UnifiedManager\"]\n        D{\"Auth.MultiAuthCoordinator\"}\n    end\n\n    subgraph S3 [\"Auth Implementation\"]\n        E[\"Config\"]\n        F[\"Auth.Strategy (Behavior)\"]\n        G((\"GeminiStrategy\"))\n        H((\"VertexStrategy\"))\n    end\n    \n    subgraph S4 [\"HTTP Client\"]\n        I[\"Client.HTTP / HTTPStreaming\"]\n    end\n\n    A --> B\n    B --> C\n    C -->|\"1 Requests auth coordination (passes opts)\"| D\n    D -->|\"2 Gets credentials for strategy\"| E\n    E -->|\"Reads ENV & app config\"| D\n    D -->|\"3 Selects strategy implementation\"| F\n    F --> G\n    F --> H\n    D -->|\"4 Uses strategy to build headers\"| G\n    G -->|\"Returns Gemini headers\"| D\n    D -->|\"5 Returns final headers\"| C\n    C -->|\"6 Makes request with final headers\"| I\n\n    %% Styling\n    classDef primary fill:#6B46C1,stroke:#4C1D95,stroke-width:3px,color:#FFFFFF\n    classDef secondary fill:#9333EA,stroke:#6B21A8,stroke-width:2px,color:#FFFFFF\n    classDef tertiary fill:#A855F7,stroke:#7C2D12,stroke-width:2px,color:#FFFFFF\n    classDef api fill:#EF4444,stroke:#B91C1C,stroke-width:2px,color:#FFFFFF\n    classDef coordinator fill:#10B981,stroke:#047857,stroke-width:2px,color:#FFFFFF\n    classDef strategy fill:#F59E0B,stroke:#D97706,stroke-width:2px,color:#FFFFFF\n    classDef config fill:#3B82F6,stroke:#1D4ED8,stroke-width:2px,color:#FFFFFF\n    classDef behavior fill:#8B5CF6,stroke:#7C3AED,stroke-width:3px,color:#FFFFFF,stroke-dasharray: 5 5\n    \n    %% Apply classes\n    class A api\n    class B,C primary\n    class D coordinator\n    class E config\n    class F behavior\n    class G,H strategy\n    class I secondary\n    \n    %% Subgraph styling\n    style S1 fill:#F9FAFB,stroke:#EF4444,stroke-width:3px\n    style S2 fill:#FEFEFE,stroke:#6B46C1,stroke-width:3px\n    style S3 fill:#F3F4F6,stroke:#10B981,stroke-width:3px\n    style S4 fill:#F8FAFC,stroke:#9333EA,stroke-width:3px\n```\n\n#### Architectural Layers Explained\n\n**1. Configuration Layer (`gemini/config.ex`)**\n\n*   **Purpose**: To detect and load the *default* authentication credentials for the application.\n*   **Mechanism**: The `Gemini.Config.auth_config/0` function establishes a strict priority order for finding credentials:\n    1.  **Environment Variables**: Checks for `GEMINI_API_KEY`, `VERTEX_SERVICE_ACCOUNT`, etc. This is the highest priority.\n    2.  **Application Config**: If no environment variables are found, it checks for `:gemini, :auth` or `:gemini, :api_key` in the Elixir application environment (e.g., `config/runtime.exs`).\n*   **Code Example (`gemini/config.ex`):**\n    ```elixir\n    def auth_config do\n      cond do\n        gemini_api_key() ->\n          %{type: :gemini, credentials: %{api_key: gemini_api_key()}}\n        # ... other checks for Vertex, etc.\n        true ->\n          Application.get_env(:gemini, :auth) # ...\n      end\n    end\n    ```\n\n**2. Abstraction Layer (`gemini/auth.ex`)**\n\n*   **Purpose**: To define a common interface for all authentication strategies.\n*   **Mechanism**: It defines the `Gemini.Auth.Strategy` behavior. Any module that implements this behavior must define functions like `headers/1` and `base_url/1`. This allows the rest of the system to interact with any authentication strategy in a uniform way.\n*   **Code Example (`gemini/auth.ex`):**\n    ```elixir\n    defmodule Strategy do\n      @callback headers(credentials :: map()) :: [{String.t(), String.t()}]\n      @callback base_url(credentials :: map()) :: String.t()\n      # ...\n    end\n    ```\n\n**3. Implementation Layer (`gemini/auth/*_strategy.ex`)**\n\n*   **Purpose**: To provide the concrete logic for each authentication strategy.\n*   **Mechanism**:\n    *   `GeminiStrategy` implements the `Strategy` behavior for API Key auth. Its `headers/1` function creates the `x-goog-api-key` header.\n    *   `VertexStrategy` implements the same behavior for Vertex AI, handling the complexities of generating OAuth tokens.\n*   **Code Example (`gemini/auth/gemini_strategy.ex`):**\n    ```elixir\n    def headers(%{api_key: api_key}) do\n      [\n        {\"Content-Type\", \"application/json\"},\n        {\"x-goog-api-key\", api_key}\n      ]\n    end\n    ```\n\n**4. Coordination Layer (`gemini/auth/multi_auth_coordinator.ex`)**\n\n*   **Purpose**: To be the central brain of the system, handling per-request logic and orchestrating the other layers. This is what enables a user to override the global configuration for a single API call.\n*   **Mechanism**: The key function is `get_credentials(strategy, opts)`. It first checks the `opts` keyword list (passed from the user's function call) for overriding credentials (like `:api_key`). If an override is not present, it falls back to the default credentials loaded by the `Config` layer.\n*   **Code Example (`gemini/auth/multi_auth_coordinator.ex`):**\n    ```elixir\n    def get_credentials(:gemini, opts) do\n      base_config = Config.get_auth_config(:gemini)\n      \n      # Prioritize opts, then fall back to base_config\n      api_key = Keyword.get(opts, :api_key, base_config[:api_key])\n      \n      # ... validation\n    end\n    ```","ref":"authentication_system.html#3-system-architecture"},{"type":"extras","title":"4. End-to-End Authentication Flow - AUTHENTICATION_SYSTEM","doc":"Let's trace a user's call to see how these layers interact.\n\n**Scenario:** A user wants to make a single streaming call with a specific, temporary API key, overriding their global configuration.\n\n1.  **User Call**: The developer calls a high-level function in `APIs.Coordinator`, passing the `:api_key` in the `opts`.\n    ```elixir\n    Coordinator.stream_generate_content(\n      \"Tell me a story\",\n      auth: :gemini,\n      api_key: \"aiza-temporary-key-for-this-call\"\n    )\n    ```\n\n2.  **Delegation**: `APIs.Coordinator` delegates this call to the `Streaming.UnifiedManager`, passing along the `opts` unchanged.\n\n3.  **Auth Coordination**: The `UnifiedManager` needs to get HTTP headers. It calls `MultiAuthCoordinator.coordinate_auth(:gemini, opts)`.\n\n4.  **Credential Resolution**: The `MultiAuthCoordinator`'s `get_credentials/2` function is executed.\n    *   It sees `api_key: \"aiza-temporary-key-for-this-call\"` inside the `opts`.\n    *   It **uses this key** and ignores whatever might be configured in environment variables or `config/runtime.exs`.\n\n5.  **Header Generation**: The `MultiAuthCoordinator` now has the correct credentials. It invokes the `:gemini` strategy (`GeminiStrategy.headers/1`) with these credentials.\n\n6.  **Header Creation**: `GeminiStrategy` creates the final list of headers: `[{\"x-goog-api-key\", \"aiza-temporary-key-for-this-call\"}, ...]`.\n\n7.  **HTTP Request**: The headers are returned up to the `UnifiedManager`, which then passes them to the `Client.HTTPStreaming` module to make the final, correctly authenticated HTTP request.","ref":"authentication_system.html#4-end-to-end-authentication-flow"},{"type":"extras","title":"5. Configuration Guide for Library Users - AUTHENTICATION_SYSTEM","doc":"To use the library, configure your credentials using one of the following methods, listed in order of precedence (Method 1 overrides Method 2, which overrides Method 3).\n\n#### Method 1: Per-Request Override\n\nFor functions that support it (primarily in the streaming API), you can provide the authentication details directly in the `opts` of the function call.\n\n```elixir\n# This API key is used for this call only\nopts = [\n  auth: :gemini,\n  api_key: \"your_specific_api_key\"\n]\n{:ok, stream_id} = Gemini.APIs.Coordinator.stream_generate_content(\"Hello\", opts)\n```\n\n#### Method 2: Environment Variables (Recommended for Production)\n\nThis is the most secure and standard way to provide credentials in production, staging, and CI/CD environments.\n\n*   **For Gemini API Key:**\n    ```bash\n    export GEMINI_API_KEY=\"your-google-ai-api-key\"\n    ```\n\n*   **For Vertex AI (Service Account):**\n    ```bash\n    export VERTEX_SERVICE_ACCOUNT=\"/path/to/your/service-account.json\"\n    export VERTEX_PROJECT_ID=\"your-gcp-project-id\"\n    export VERTEX_LOCATION=\"us-central1\"\n    ```\n\nThe library will automatically detect and use these variables.\n\n#### Method 3: Application Configuration (Standard Elixir Way)\n\nConfigure the library in your project's `config/runtime.exs` file. This is the canonical approach for managing secrets within an Elixir application.\n\n*   **For Gemini API Key:**\n    ```elixir\n    # config/runtime.exs\n    import Config\n\n    # Best practice: Read from an environment variable\n    config :gemini,\n      api_key: System.fetch_env!(\"GEMINI_API_KEY\")\n    ```\n\n*   **For Vertex AI (Service Account):**\n    ```elixir\n    # config/runtime.exs\n    import Config\n\n    config :gemini, :auth,\n      type: :vertex_ai,\n      credentials: %{\n        service_account_key: System.fetch_env!(\"VERTEX_SERVICE_ACCOUNT_PATH\"),\n        project_id: System.fetch_env!(\"GCP_PROJECT_ID\"),\n        location: \"us-central1\"\n      }\n    ```\n\n---","ref":"authentication_system.html#5-configuration-guide-for-library-users"},{"type":"extras","title":"Gemini Elixir Streaming Architecture","doc":"# Gemini Elixir Streaming Architecture\n\n**Technical Deep Dive: Server-Sent Events (SSE) Streaming Implementation**\n\n---","ref":"streaming_architecture.html"},{"type":"extras","title":"Table of Contents - Gemini Elixir Streaming Architecture","doc":"1. [Executive Summary](#executive-summary)\n2. [Problem Analysis](#problem-analysis)\n3. [Architecture Overview](#architecture-overview)\n4. [Technical Design Decisions](#technical-design-decisions)\n5. [Component Deep Dive](#component-deep-dive)\n6. [Performance Characteristics](#performance-characteristics)\n7. [Error Handling Strategy](#error-handling-strategy)\n8. [Testing Strategy](#testing-strategy)\n9. [Production Considerations](#production-considerations)\n10. [Future Enhancements](#future-enhancements)\n\n---","ref":"streaming_architecture.html#table-of-contents"},{"type":"extras","title":"Executive Summary - Gemini Elixir Streaming Architecture","doc":"This document details the complete redesign and implementation of streaming functionality for the Gemini Elixir client library. The original implementation was fundamentally broken due to misunderstanding of HTTP streaming protocols and improper state management. Our solution provides a production-ready streaming system with proper SSE parsing, resource management, and error recovery.","ref":"streaming_architecture.html#executive-summary"},{"type":"extras","title":"Key Achievements - Gemini Elixir Streaming Architecture","doc":"- **🔧 Fixed Core Issues**: Proper SSE parsing with stateful chunk handling\n- **📈 Performance**: Supports 100+ concurrent streams with automatic resource management\n- **🛡️ Reliability**: Comprehensive error handling with automatic retries and fallback\n- **🔄 Scalability**: GenServer-based architecture with subscriber pattern\n- **📊 Observability**: Built-in metrics, logging, and monitoring capabilities\n\n---","ref":"streaming_architecture.html#key-achievements"},{"type":"extras","title":"Problem Analysis - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#problem-analysis"},{"type":"extras","title":"Original Implementation Issues - Gemini Elixir Streaming Architecture","doc":"#### 1. **HTTP Streaming Misunderstanding**\n```elixir\n# ❌ BROKEN: Original implementation\nreq_opts = [\n  into: :self,  # Wrong - this is for file downloads, not SSE\n  # ...\n]\n\ncase Req.post(req_opts) do\n  {:ok, %Req.Response{body: body}} ->\n    events = parse_sse_stream(body)  # Body is complete response, not streaming\n```\n\n**Problem**: The `into: :self` option in Req is designed for downloading large files, not for processing Server-Sent Events. SSE requires maintaining an open connection and processing data as it arrives in chunks.\n\n#### 2. **SSE Parsing Assumptions**\n```elixir\n# ❌ BROKEN: Assumes complete SSE data\ndefp parse_sse_stream(data) when is_binary(data) do\n  data\n  |> String.split(\"\\n\\n\")  # Assumes all events are complete\n```\n\n**Problem**: SSE events arrive incrementally. A single event might be split across multiple TCP packets, requiring stateful parsing with buffering.\n\n#### 3. **GenServer Lifecycle Issues**\n```elixir\n# ❌ BROKEN: Process immediately exits\nspawn_link(fn ->\n  case HTTP.stream_post_raw(...) do\n    {:ok, events} -> \n      # Send all events at once, then exit\n      send(manager_pid, {:stream_complete, stream_id})\n  end\nend)\n```\n\n**Problem**: Real streaming requires persistent connections. The original implementation spawned processes that immediately terminated after receiving a complete HTTP response.","ref":"streaming_architecture.html#original-implementation-issues"},{"type":"extras","title":"Impact Assessment - Gemini Elixir Streaming Architecture","doc":"- **Functionality**: 0% - Streaming completely non-functional\n- **Resource Management**: Critical memory leaks and race conditions\n- **Error Handling**: No recovery mechanisms\n- **Production Readiness**: Completely unsuitable\n\n---","ref":"streaming_architecture.html#impact-assessment"},{"type":"extras","title":"Architecture Overview - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#architecture-overview"},{"type":"extras","title":"System Components - Gemini Elixir Streaming Architecture","doc":"```mermaid\ngraph LR\n    A[Client Application] --> B[Gemini.Streaming.ManagerV2]\n    B --> C[Gemini.Client.HTTPStreaming]\n    C --> D[Gemini.SSE.Parser]\n    C --> E[Req HTTP Client]\n    E --> F[Gemini API]\n    \n    B --> G[Subscriber 1]\n    B --> H[Subscriber 2]\n    B --> I[Subscriber N]\n    \n    D --> J[Event Buffer]\n    D --> K[JSON Parser]\n    \n    subgraph S1 [\" \"]\n        direction TB\n        ST1[🔄 Stream State Management]\n        B\n        L[Process Monitors]\n        M[Resource Cleanup]\n        N[Statistics]\n    end\n    \n    subgraph S2 [\" \"]\n        direction TB\n        ST2[🌐 HTTP Layer]\n        C\n        E\n        O[Connection Pool]\n        P[Retry Logic]\n    end\n    \n    subgraph S3 [\" \"]\n        direction TB\n        ST3[⚡ SSE Processing]\n        D\n        J\n        K\n        Q[Event Validation]\n    end\n\n    %% Styling\n    classDef primary fill:#6B46C1,stroke:#4C1D95,stroke-width:3px,color:#FFFFFF\n    classDef secondary fill:#9333EA,stroke:#6B21A8,stroke-width:2px,color:#FFFFFF\n    classDef tertiary fill:#A855F7,stroke:#7C2D12,stroke-width:2px,color:#FFFFFF\n    classDef api fill:#EF4444,stroke:#B91C1C,stroke-width:2px,color:#FFFFFF\n    classDef subscriber fill:#10B981,stroke:#047857,stroke-width:2px,color:#FFFFFF\n    classDef subgraphTitle fill:#1E1B4B,stroke:#312E81,stroke-width:2px,color:#FFFFFF\n    \n    %% Apply classes\n    class A primary\n    class B primary\n    class C,E secondary\n    class D,J,K tertiary\n    class F api\n    class G,H,I subscriber\n    class L,M,N,O,P,Q tertiary\n    class ST1,ST2,ST3 subgraphTitle\n    \n    %% Subgraph styling\n    style S1 fill:#F3F4F6,stroke:#6B46C1,stroke-width:3px\n    style S2 fill:#F9FAFB,stroke:#9333EA,stroke-width:3px\n    style S3 fill:#FEFEFE,stroke:#A855F7,stroke-width:3px\n```","ref":"streaming_architecture.html#system-components"},{"type":"extras","title":"Data Flow - Gemini Elixir Streaming Architecture","doc":"1. **Stream Initiation**: Client requests stream via `ManagerV2.start_stream/3`\n2. **Authentication**: Manager builds authenticated request using unified auth system\n3. **HTTP Streaming**: `HTTPStreaming` establishes persistent SSE connection\n4. **Chunk Processing**: `SSE.Parser` processes incoming data chunks with state management\n5. **Event Distribution**: Manager notifies all subscribers of parsed events\n6. **Lifecycle Management**: Automatic cleanup when streams complete or subscribers disconnect\n\n---","ref":"streaming_architecture.html#data-flow"},{"type":"extras","title":"Technical Design Decisions - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#technical-design-decisions"},{"type":"extras","title":"1. GenServer for Stream Management - Gemini Elixir Streaming Architecture","doc":"#### **Decision**: Use GenServer for centralized stream management\n\n#### **Rationale**:\n\n**State Management**: GenServer provides reliable state management for complex stream lifecycle:\n```elixir\n@type stream_state :: %{\n  stream_id: stream_id(),\n  stream_pid: pid() | nil,\n  model: String.t(),\n  status: :starting | :active | :completed | :error | :stopped,\n  subscribers: [subscriber_ref()],\n  events_count: non_neg_integer(),\n  # ... additional metadata\n}\n```\n\n**Process Monitoring**: GenServer can monitor multiple subscriber processes and clean up when they die:\n```elixir\ndef handle_info({:DOWN, monitor_ref, :process, pid, _reason}, state) do\n  # Automatic cleanup of dead subscribers\n  # Remove stream if no subscribers remain\nend\n```\n\n**Concurrency Control**: Single GenServer provides serialized access to stream state, preventing race conditions:\n```elixir\n# All operations go through GenServer calls/casts\ndef handle_call({:subscribe, stream_id, subscriber_pid}, _from, state)\n```\n\n**OTP Supervision**: GenServer integrates with OTP supervision trees for fault tolerance:\n```elixir\n# Supervisor can restart manager if it crashes\nchildren = [{Gemini.Streaming.ManagerV2, []}]\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\n#### **Alternatives Considered**:\n\n- **Registry-based**: Would require external coordination for cleanup\n- **Agent-based**: No built-in monitoring capabilities\n- **Pure functional**: Complex state management without GenServer benefits\n- **GenStage**: Overkill for current requirements, planned for future enhancement","ref":"streaming_architecture.html#1-genserver-for-stream-management"},{"type":"extras","title":"2. Stateful SSE Parser - Gemini Elixir Streaming Architecture","doc":"#### **Decision**: Implement stateful parser with buffer management\n\n#### **Rationale**:\n\n**Chunk Boundaries**: SSE events can be split across TCP packets:\n```elixir\n# Chunk 1: \"data: {\\\"text\\\": \\\"hel\"\n# Chunk 2: \"lo world\\\"}\\n\\n\"\n# Parser must buffer partial events\n```\n\n**Memory Efficiency**: Process events as they arrive rather than accumulating all data:\n```elixir\ndefstruct buffer: \"\", events: []\n\ndef parse_chunk(chunk, %__MODULE__{buffer: buffer} = state) do\n  full_data = buffer <> chunk\n  {events, remaining_buffer} = extract_events(full_data)\n  # Process events immediately, keep only incomplete data\nend\n```\n\n**Error Isolation**: Bad events don't corrupt the entire stream:\n```elixir\ndef parse_event(event_data) do\n  case parse_json_data(data) do\n    {:ok, parsed_data} -> build_event(parsed_data)\n    {:error, _} -> nil  # Skip malformed events\n  end\nend\n```","ref":"streaming_architecture.html#2-stateful-sse-parser"},{"type":"extras","title":"3. Callback-based HTTP Streaming - Gemini Elixir Streaming Architecture","doc":"#### **Decision**: Use Req with streaming callbacks rather than complete response handling\n\n#### **Rationale**:\n\n**Real-time Processing**: Events processed as they arrive:\n```elixir\nreq_opts = [\n  into: fn message, acc ->\n    handle_stream_message(message, acc, callback)\n  end\n]\n```\n\n**Memory Efficiency**: No accumulation of large response bodies:\n```elixir\ndefp handle_stream_message({:data, chunk}, parser, callback) do\n  case Parser.parse_chunk(chunk, parser) do\n    {:ok, events, new_parser} ->\n      Enum.each(events, &callback.(%{type: :data, data: &1.data}))\n      new_parser\n  end\nend\n```\n\n**Backpressure**: Callback return values can control stream flow:\n```elixir\ncase callback.(stream_event) do\n  :ok -> :continue\n  :stop -> throw({:stop_stream, :requested})\nend\n```","ref":"streaming_architecture.html#3-callback-based-http-streaming"},{"type":"extras","title":"4. Subscriber Pattern - Gemini Elixir Streaming Architecture","doc":"#### **Decision**: Multiple subscribers per stream with individual monitoring\n\n#### **Rationale**:\n\n**Resource Sharing**: Multiple processes can receive the same stream data:\n```elixir\n# Dashboard, logger, and analytics can all subscribe to same stream\n:ok = ManagerV2.subscribe_stream(stream_id, dashboard_pid)\n:ok = ManagerV2.subscribe_stream(stream_id, logger_pid)\n:ok = ManagerV2.subscribe_stream(stream_id, analytics_pid)\n```\n\n**Automatic Cleanup**: Streams stop when all subscribers disconnect:\n```elixir\ndef handle_info({:DOWN, monitor_ref, :process, pid, _reason}, state) do\n  # Remove dead subscriber\n  # Stop stream if no subscribers remain\nend\n```\n\n**Process Isolation**: Subscriber failures don't affect other subscribers or the stream:\n```elixir\n# Each subscriber is monitored independently\nmonitor_ref = Process.monitor(subscriber_pid)\nsubscriber_ref = {subscriber_pid, monitor_ref}\n```\n\n---","ref":"streaming_architecture.html#4-subscriber-pattern"},{"type":"extras","title":"Component Deep Dive - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#component-deep-dive"},{"type":"extras","title":"Gemini.SSE.Parser - Gemini Elixir Streaming Architecture","doc":"#### **Purpose**: Stateful parsing of Server-Sent Events with proper chunk handling\n\n#### **Key Features**:\n\n```elixir\ndefmodule Gemini.SSE.Parser do\n  defstruct buffer: \"\", events: []\n  \n  # Core parsing with state management\n  def parse_chunk(chunk, parser) do\n    full_data = parser.buffer <> chunk\n    {events, remaining_buffer} = extract_events(full_data)\n    parsed_events = Enum.map(events, &parse_event/1) |> Enum.filter(&(&1 != nil))\n    new_state = %{parser | buffer: remaining_buffer}\n    {:ok, parsed_events, new_state}\n  end\nend\n```\n\n#### **Technical Details**:\n\n**Event Extraction**:\n```elixir\ndefp extract_events(data) do\n  parts = String.split(data, \"\\n\\n\")\n  case parts do\n    [single_part] -> {[], single_part}  # No complete events\n    multiple_parts ->\n      {complete_events, [remaining]} = Enum.split(multiple_parts, -1)\n      {complete_events, remaining}\n  end\nend\n```\n\n**JSON Parsing with Error Handling**:\n```elixir\ndefp parse_json_data(\"[DONE]\"), do: {:ok, %{done: true}}\ndefp parse_json_data(json_string) do\n  case Jason.decode(json_string) do\n    {:ok, data} -> {:ok, data}\n    {:error, _} -> {:error, :invalid_json}\n  end\nend\n```\n\n**Performance Optimizations**:\n- Early filtering of malformed events\n- Minimal string operations\n- Efficient buffer management\n- Unicode-safe string handling","ref":"streaming_architecture.html#gemini-sse-parser"},{"type":"extras","title":"Gemini.Client.HTTPStreaming - Gemini Elixir Streaming Architecture","doc":"#### **Purpose**: HTTP client for establishing and managing SSE connections\n\n#### **Key Features**:\n\n```elixir\ndef stream_sse(url, headers, body, callback, opts \\\\ []) do\n  timeout = Keyword.get(opts, :timeout, 30_000)\n  max_retries = Keyword.get(opts, :max_retries, 3)\n  stream_with_retries(url, headers, body, callback, timeout, max_retries, 0)\nend\n```\n\n#### **Connection Management**:\n\n**Retry Logic with Exponential Backoff**:\n```elixir\ndefp stream_with_retries(url, headers, body, callback, timeout, max_retries, attempt) do\n  case do_stream(url, headers, body, callback, timeout) do\n    {:ok, :completed} -> {:ok, :completed}\n    {:error, error} when attempt  \n      delay = min(1000 * :math.pow(2, attempt), 10_000) |> round()\n      Process.sleep(delay)\n      stream_with_retries(url, headers, body, callback, timeout, max_retries, attempt + 1)\n    {:error, error} -> {:error, error}\n  end\nend\n```\n\n**SSE-specific Headers**:\n```elixir\ndefp add_sse_headers(headers) do\n  sse_headers = [\n    {\"Accept\", \"text/event-stream\"},\n    {\"Cache-Control\", \"no-cache\"}\n  ]\n  # Merge without duplicates\n  headers ++ filter_new_headers(sse_headers, headers)\nend\n```\n\n**Stream Processing**:\n```elixir\nreq_opts = [\n  method: :post,\n  url: sse_url,\n  headers: headers,\n  json: body,\n  into: fn message, acc ->\n    handle_stream_message(message, acc, callback)\n  end\n]\n```","ref":"streaming_architecture.html#gemini-client-httpstreaming"},{"type":"extras","title":"Gemini.Streaming.ManagerV2 - Gemini Elixir Streaming Architecture","doc":"#### **Purpose**: Central GenServer for managing stream lifecycle and subscribers\n\n#### **State Structure**:\n```elixir\n@type manager_state :: %{\n  streams: %{stream_id() => stream_state()},\n  stream_counter: non_neg_integer(),\n  max_streams: pos_integer(),\n  default_timeout: pos_integer()\n}\n\n@type stream_state :: %{\n  stream_id: stream_id(),\n  stream_pid: pid() | nil,\n  model: String.t(),\n  request_body: map(),\n  status: :starting | :active | :completed | :error | :stopped,\n  error: term() | nil,\n  started_at: DateTime.t(),\n  subscribers: [subscriber_ref()],\n  events_count: non_neg_integer(),\n  last_event_at: DateTime.t() | nil,\n  config: keyword()\n}\n```\n\n#### **Resource Management**:\n\n**Stream Limits**:\n```elixir\ndef handle_call({:start_stream, contents, opts, subscriber_pid}, _from, state) do\n  if map_size(state.streams) >= state.max_streams do\n    {:reply, {:error, :max_streams_exceeded}, state}\n  else\n    # Proceed with stream creation\n  end\nend\n```\n\n**Automatic Cleanup**:\n```elixir\ndef handle_info({:stream_complete, stream_id}, state) do\n  # Update status and notify subscribers\n  updated_stream = %{stream_state | status: :completed}\n  notify_subscribers(updated_stream.subscribers, {:stream_complete, stream_id})\n  \n  # Schedule cleanup after delay\n  Process.send_after(self(), {:cleanup_stream, stream_id}, 5_000)\n  # ...\nend\n```\n\n**Subscriber Monitoring**:\n```elixir\ndef handle_call({:subscribe, stream_id, subscriber_pid}, _from, state) do\n  monitor_ref = Process.monitor(subscriber_pid)\n  subscriber_ref = {subscriber_pid, monitor_ref}\n  \n  updated_stream = %{\n    stream_state | \n    subscribers: [subscriber_ref | stream_state.subscribers]\n  }\n  # ...\nend\n```\n\n---","ref":"streaming_architecture.html#gemini-streaming-managerv2"},{"type":"extras","title":"Performance Characteristics - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#performance-characteristics"},{"type":"extras","title":"Benchmarks - Gemini Elixir Streaming Architecture","doc":"#### **Concurrent Streams**\n- **Target**: 100 concurrent streams\n- **Memory per stream**: ~2KB baseline + event data\n- **CPU overhead**: <1% per stream on modern hardware\n- **Network**: Limited by API rate limits, not client\n\n#### **Event Processing**\n- **Parser throughput**: >10,000 events/second\n- **Latency**: <1ms from chunk receipt to subscriber notification\n- **Memory efficiency**: O(1) buffer size regardless of stream length\n\n#### **Resource Usage**\n```elixir\n# Example resource footprint\n%{\n  total_streams: 50,\n  memory_per_stream: \"~2KB\",\n  total_memory_overhead: \"~100KB\",\n  cpu_usage: \"<0.5%\",\n  network_connections: 50\n}\n```","ref":"streaming_architecture.html#benchmarks"},{"type":"extras","title":"Scaling Characteristics - Gemini Elixir Streaming Architecture","doc":"#### **Vertical Scaling**\n- **Memory**: Linear with number of active streams\n- **CPU**: Negligible until >1000 concurrent streams\n- **Network**: Limited by upstream API, not client\n\n#### **Horizontal Scaling**\n- **Process distribution**: GenServer can be distributed across nodes\n- **Load balancing**: Multiple manager instances with consistent hashing\n- **State replication**: Stream state can be replicated for fault tolerance","ref":"streaming_architecture.html#scaling-characteristics"},{"type":"extras","title":"Performance Monitoring - Gemini Elixir Streaming Architecture","doc":"```elixir\ndef get_performance_stats() do\n  %{\n    streams_by_status: count_streams_by_status(),\n    average_events_per_stream: calculate_average_events(),\n    memory_usage: :erlang.system_info(:total_memory),\n    process_count: :erlang.system_info(:process_count),\n    uptime_seconds: uptime_in_seconds()\n  }\nend\n```\n\n---","ref":"streaming_architecture.html#performance-monitoring"},{"type":"extras","title":"Error Handling Strategy - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#error-handling-strategy"},{"type":"extras","title":"Error Categories - Gemini Elixir Streaming Architecture","doc":"#### **1. Network Errors**\n```elixir\n# Connection failures, timeouts, DNS issues\n{:error, %Req.TransportError{reason: :timeout}} ->\n  # Automatic retry with exponential backoff\n  retry_stream_with_backoff(url, headers, body, callback, attempt + 1)\n```\n\n#### **2. API Errors**\n```elixir\n# HTTP 4xx/5xx responses\n{:ok, %Req.Response{status: 429}} ->\n  # Rate limiting - longer backoff\n  {:error, Error.api_error(429, \"Rate limited\")}\n\n{:ok, %Req.Response{status: 401}} ->\n  # Auth failure - no retry\n  {:error, Error.api_error(401, \"Authentication failed\")}\n```\n\n#### **3. Parse Errors**\n```elixir\n# Malformed SSE events\ndefp parse_event(event_data) do\n  case parse_json_data(data) do\n    {:ok, parsed_data} -> build_event(parsed_data)\n    {:error, _} -> nil  # Skip malformed events, continue stream\n  end\nend\n```\n\n#### **4. Resource Errors**\n```elixir\n# Stream limits, memory pressure\ndef handle_call({:start_stream, _, _, _}, _from, state) do\n  if map_size(state.streams) >= state.max_streams do\n    {:reply, {:error, :max_streams_exceeded}, state}\n  end\nend\n```","ref":"streaming_architecture.html#error-categories"},{"type":"extras","title":"Recovery Strategies - Gemini Elixir Streaming Architecture","doc":"#### **Automatic Retry**\n```elixir\ndefp stream_with_retries(url, headers, body, callback, timeout, max_retries, attempt) do\n  case do_stream(url, headers, body, callback, timeout) do\n    {:error, error} when attempt  \n      delay = min(1000 * :math.pow(2, attempt), 10_000) |> round()\n      Logger.warning(\"Stream retry #{attempt + 1}/#{max_retries}: #{inspect(error)}\")\n      Process.sleep(delay)\n      stream_with_retries(url, headers, body, callback, timeout, max_retries, attempt + 1)\n    \n    result -> result\n  end\nend\n```\n\n#### **Graceful Degradation**\n```elixir\n# Fallback to non-streaming when streaming fails\ncase ManagerV2.start_stream(contents, opts, self()) do\n  {:error, _streaming_error} ->\n    Logger.info(\"Streaming failed, falling back to regular generation\")\n    Gemini.generate(contents, opts)\nend\n```\n\n#### **Circuit Breaker Pattern**\n```elixir\n# Future enhancement - stop trying after repeated failures\ndefmodule CircuitBreaker do\n  # Track failure rates and disable endpoints temporarily\n  def should_attempt_stream?(endpoint) do\n    failure_rate = get_failure_rate(endpoint)\n    failure_rate < 0.5  # 50% threshold\n  end\nend\n```","ref":"streaming_architecture.html#recovery-strategies"},{"type":"extras","title":"Error Propagation - Gemini Elixir Streaming Architecture","doc":"#### **Structured Error Types**\n```elixir\ndefmodule Gemini.Error do\n  @type t :: %__MODULE__{\n    type: :network_error | :api_error | :parse_error | :resource_error,\n    message: String.t(),\n    http_status: integer() | nil,\n    retryable: boolean(),\n    details: map()\n  }\nend\n```\n\n#### **Subscriber Notification**\n```elixir\n# Errors propagated to all subscribers\ndef handle_info({:stream_error, stream_id, error}, state) do\n  notify_subscribers(stream_state.subscribers, {:stream_error, stream_id, error})\n  updated_stream = %{stream_state | status: :error, error: error}\n  # ...\nend\n```\n\n---","ref":"streaming_architecture.html#error-propagation"},{"type":"extras","title":"Testing Strategy - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#testing-strategy"},{"type":"extras","title":"Test Categories - Gemini Elixir Streaming Architecture","doc":"#### **1. Unit Tests**\n\n**SSE Parser Tests**:\n```elixir\ndescribe \"SSE Parser edge cases\" do\n  test \"handles partial events across chunks\" do\n    parser = Parser.new()\n    \n    # First chunk - incomplete\n    chunk1 = \"data: {\\\"text\\\": \\\"hel\"\n    {:ok, events1, parser1} = Parser.parse_chunk(chunk1, parser)\n    assert events1 == []\n    \n    # Second chunk - completes event\n    chunk2 = \"lo\\\"}\\n\\n\"\n    {:ok, events2, parser2} = Parser.parse_chunk(chunk2, parser1)\n    assert length(events2) == 1\n    assert hd(events2).data == %{\"text\" => \"hello\"}\n  end\nend\n```\n\n**HTTP Streaming Tests**:\n```elixir\ndescribe \"HTTP streaming with mocks\" do\n  test \"handles connection failures with retry\" do\n    # Mock Req to fail first N attempts\n    # Verify exponential backoff timing\n    # Confirm eventual success\n  end\nend\n```\n\n**Manager State Tests**:\n```elixir\ndescribe \"Streaming Manager state management\" do\n  test \"cleans up when subscribers die\" do\n    {:ok, stream_id} = ManagerV2.start_stream(contents, [], self())\n    subscriber = spawn(fn -> :ok end)\n    :ok = ManagerV2.subscribe_stream(stream_id, subscriber)\n    \n    Process.exit(subscriber, :kill)\n    Process.sleep(50)  # Allow DOWN message processing\n    \n    # Verify cleanup\n    assert {:error, :stream_not_found} = ManagerV2.get_stream_info(stream_id)\n  end\nend\n```\n\n#### **2. Integration Tests**\n\n**Real API Tests**:\n```elixir\n@tag :integration\ntest \"end-to-end streaming with real API\" do\n  {:ok, stream_id} = ManagerV2.start_stream(\"Count from 1 to 5\", [], self())\n  \n  events = collect_stream_events(stream_id, 10_000)\n  \n  assert length(events) > 0\n  assert List.last(events).type in [:complete, :error]\nend\n```\n\n**Error Scenario Tests**:\n```elixir\n@tag :integration  \ntest \"streaming with invalid model triggers error\" do\n  {:ok, stream_id} = ManagerV2.start_stream(\"Hello\", [model: \"invalid\"], self())\n  \n  events = collect_stream_events(stream_id, 5_000)\n  error_events = Enum.filter(events, &(&1.type == :error))\n  \n  assert length(error_events) > 0\nend\n```\n\n#### **3. Performance Tests**\n\n**Load Testing**:\n```elixir\ntest \"handles 100 concurrent streams\" do\n  stream_ids = \n    1..100\n    |> Enum.map(fn i ->\n      {:ok, stream_id} = ManagerV2.start_stream(\"Test #{i}\", [], self())\n      stream_id\n    end)\n  \n  stats = ManagerV2.get_stats()\n  assert stats.total_streams == 100\n  \n  # Verify all streams eventually complete or error\n  assert_all_streams_finish(stream_ids, 60_000)\nend\n```\n\n**Memory Testing**:\n```elixir\ntest \"memory usage remains stable during long streams\" do\n  initial_memory = :erlang.memory(:total)\n  \n  {:ok, stream_id} = ManagerV2.start_stream(\"Very long content...\", [], self())\n  \n  # Process many events\n  events = collect_stream_events(stream_id, 30_000)\n  \n  final_memory = :erlang.memory(:total)\n  memory_growth = final_memory - initial_memory\n  \n  # Memory growth should be minimal (< 10MB for test)\n  assert memory_growth < 10_000_000\nend\n```\n\n#### **4. Property-Based Testing**\n\n**SSE Parser Properties**:\n```elixir\nproperty \"parser handles arbitrary chunk boundaries\" do\n  check all event_data <- valid_sse_event(),\n            chunk_sizes <- list_of(positive_integer()) do\n    \n    # Split event data at arbitrary boundaries\n    chunks = split_into_chunks(event_data, chunk_sizes)\n    \n    # Parse incrementally\n    {events, _parser} = parse_chunks_incrementally(chunks)\n    \n    # Should produce same result as parsing complete data\n    {:ok, expected_events, _} = Parser.parse_chunk(event_data, Parser.new())\n    assert events == expected_events\n  end\nend\n```","ref":"streaming_architecture.html#test-categories"},{"type":"extras","title":"Test Infrastructure - Gemini Elixir Streaming Architecture","doc":"#### **Mock Servers**:\n```elixir\ndefmodule MockSSEServer do\n  use Plug.Router\n  \n  plug :match\n  plug :dispatch\n  \n  post \"/stream\" do\n    conn\n    |> put_resp_header(\"content-type\", \"text/event-stream\")\n    |> send_chunked(200)\n    |> send_mock_events()\n  end\n  \n  defp send_mock_events(conn) do\n    events = [\n      \"data: {\\\"text\\\": \\\"Hello\\\"}\\n\\n\",\n      \"data: {\\\"text\\\": \\\" world\\\"}\\n\\n\",\n      \"data: [DONE]\\n\\n\"\n    ]\n    \n    Enum.reduce(events, conn, fn event, acc ->\n      {:ok, acc} = chunk(acc, event)\n      Process.sleep(100)  # Simulate streaming delay\n      acc\n    end)\n  end\nend\n```\n\n---","ref":"streaming_architecture.html#test-infrastructure"},{"type":"extras","title":"Production Considerations - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#production-considerations"},{"type":"extras","title":"Deployment Architecture - Gemini Elixir Streaming Architecture","doc":"#### **Process Supervision**\n```elixir\ndefmodule Gemini.Application do\n  use Application\n  \n  def start(_type, _args) do\n    children = [\n      # HTTP connection pool\n      {Finch, name: GeminiHTTP, pools: %{default: [size: 100]}},\n      \n      # Streaming manager with config\n      {Gemini.Streaming.ManagerV2, [\n        max_streams: 500,\n        default_timeout: 30_000,\n        cleanup_interval: 60_000\n      ]},\n      \n      # Metrics collector\n      {Gemini.Metrics.Collector, []},\n      \n      # Health check endpoint\n      {Gemini.HealthCheck, []}\n    ]\n    \n    opts = [strategy: :one_for_one, name: Gemini.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\n#### **Configuration Management**\n```elixir\n# config/prod.exs\nconfig :gemini,\n  streaming: [\n    max_concurrent_streams: 200,\n    default_timeout: 45_000,\n    max_retries: 5,\n    retry_base_delay: 1000,\n    max_retry_delay: 30_000,\n    enable_metrics: true,\n    log_level: :info\n  ]\n```","ref":"streaming_architecture.html#deployment-architecture"},{"type":"extras","title":"Monitoring and Observability - Gemini Elixir Streaming Architecture","doc":"#### **Metrics Collection**\n```elixir\ndefmodule Gemini.Metrics do\n  use GenServer\n  \n  # Track key metrics\n  def record_stream_start(stream_id, model) do\n    :telemetry.execute([:gemini, :stream, :start], %{count: 1}, %{\n      stream_id: stream_id,\n      model: model\n    })\n  end\n  \n  def record_event_processed(stream_id, event_size) do\n    :telemetry.execute([:gemini, :stream, :event], %{\n      count: 1,\n      size: event_size\n    }, %{stream_id: stream_id})\n  end\nend\n```\n\n#### **Health Checks**\n```elixir\ndefmodule Gemini.HealthCheck do\n  def health_status do\n    manager_stats = ManagerV2.get_stats()\n    \n    %{\n      status: determine_health_status(manager_stats),\n      streams: %{\n        total: manager_stats.total_streams,\n        active: manager_stats.streams_by_status[:active] || 0,\n        max_allowed: manager_stats.max_streams\n      },\n      memory: %{\n        total: :erlang.memory(:total),\n        processes: :erlang.memory(:processes)\n      },\n      uptime_seconds: uptime_seconds()\n    }\n  end\n  \n  defp determine_health_status(stats) do\n    utilization = stats.total_streams / stats.max_streams\n    \n    cond do\n      utilization > 0.9 -> :degraded\n      utilization > 0.7 -> :warning  \n      true -> :healthy\n    end\n  end\nend\n```\n\n#### **Logging Strategy**\n```elixir\n# Structured logging with correlation IDs\nLogger.info(\"Stream started\", \n  stream_id: stream_id,\n  model: model,\n  subscriber_count: 1,\n  correlation_id: correlation_id\n)\n\nLogger.warning(\"Stream retry attempt\",\n  stream_id: stream_id,\n  attempt: attempt,\n  error: inspect(error),\n  correlation_id: correlation_id\n)\n```","ref":"streaming_architecture.html#monitoring-and-observability"},{"type":"extras","title":"Security Considerations - Gemini Elixir Streaming Architecture","doc":"#### **Rate Limiting**\n```elixir\ndefmodule Gemini.RateLimit do\n  def check_rate_limit(client_id) do\n    key = \"stream_limit:#{client_id}\"\n    current = :ets.update_counter(:rate_limits, key, 1, {key, 0})\n    \n    if current > max_streams_per_client() do\n      {:error, :rate_limited}\n    else\n      :ok\n    end\n  end\nend\n```\n\n#### **Resource Protection**\n```elixir\n# Memory pressure handling\ndef handle_info(:check_memory_pressure, state) do\n  total_memory = :erlang.memory(:total)\n  \n  if total_memory > memory_threshold() do\n    Logger.warning(\"Memory pressure detected, stopping oldest streams\")\n    stop_oldest_streams(state, 10)\n  end\n  \n  # Schedule next check\n  Process.send_after(self(), :check_memory_pressure, 30_000)\n  {:noreply, state}\nend\n```\n\n#### **Input Validation**\n```elixir\ndef validate_stream_request(contents, opts) do\n  with :ok <- validate_content_size(contents),\n       :ok <- validate_model_name(opts[:model]),\n       :ok <- validate_timeout(opts[:timeout]) do\n    :ok\n  else\n    {:error, reason} -> {:error, {:validation_failed, reason}}\n  end\nend\n```","ref":"streaming_architecture.html#security-considerations"},{"type":"extras","title":"Capacity Planning - Gemini Elixir Streaming Architecture","doc":"#### **Resource Requirements**\n```elixir\n# Estimated resource usage per stream\n%{\n  memory_baseline: \"2KB\",\n  memory_per_event: \"~500 bytes\",\n  cpu_per_stream: \"<0.1%\",\n  network_connection: \"1 HTTP/2 stream\",\n  file_descriptors: \"1 per stream\"\n}\n```\n\n#### **Scaling Guidelines**\n- **Small deployment**: 50-100 concurrent streams\n- **Medium deployment**: 200-500 concurrent streams  \n- **Large deployment**: 1000+ concurrent streams (requires clustering)\n\n#### **Bottleneck Analysis**\n1. **Memory**: Primary constraint - linear with active streams\n2. **Network**: API rate limits before client limits\n3. **CPU**: Minimal until >1000 streams\n4. **File descriptors**: 1 per stream + overhead\n\n---","ref":"streaming_architecture.html#capacity-planning"},{"type":"extras","title":"Future Enhancements - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#future-enhancements"},{"type":"extras","title":"Phase 1: Performance Optimization (Next 3 months) - Gemini Elixir Streaming Architecture","doc":"#### **GenStage Integration**\n```elixir\ndefmodule Gemini.Streaming.Producer do\n  use GenStage\n  \n  # Replace GenServer with GenStage for backpressure\n  def handle_demand(demand, state) when demand > 0 do\n    events = fetch_events(state, demand)\n    {:noreply, events, state}\n  end\nend\n\ndefmodule Gemini.Streaming.Consumer do\n  use GenStage\n  \n  # Consumer can control rate of event processing\n  def handle_events(events, _from, state) do\n    process_events_with_backpressure(events, state)\n    {:noreply, [], state}\n  end\nend\n```\n\n**Benefits**:\n- Automatic backpressure handling\n- Better resource utilization\n- Support for slow consumers\n- Built-in buffering strategies\n\n#### **Connection Pooling**\n```elixir\ndefmodule Gemini.ConnectionPool do\n  use GenServer\n  \n  # Pool HTTP connections for better performance\n  def get_connection(endpoint) do\n    case :poolboy.checkout(pool_name(endpoint)) do\n      :full -> {:error, :pool_exhausted}\n      conn -> {:ok, conn}\n    end\n  end\n  \n  # HTTP/2 multiplexing for multiple streams per connection\n  def multiplex_streams(connection, requests) do\n    # Send multiple stream requests over single HTTP/2 connection\n  end\nend\n```\n\n**Performance Impact**:\n- 50% reduction in connection establishment time\n- Support for 10x more concurrent streams\n- Lower memory usage per stream\n\n#### **Event Batching**\n```elixir\ndefmodule Gemini.EventBatcher do\n  # Batch multiple events for efficient processing\n  def handle_info(:flush_batch, %{batch: events} = state) when length(events) > 0 do\n    notify_subscribers_batch(state.subscribers, events)\n    schedule_next_flush()\n    {:noreply, %{state | batch: []}}\n  end\n  \n  def add_event(event, %{batch: batch} = state) do\n    new_batch = [event | batch]\n    \n    if length(new_batch) >= batch_size() do\n      notify_subscribers_batch(state.subscribers, Enum.reverse(new_batch))\n      %{state | batch: []}\n    else\n      %{state | batch: new_batch}\n    end\n  end\nend\n```","ref":"streaming_architecture.html#phase-1-performance-optimization-next-3-months"},{"type":"extras","title":"Phase 2: Advanced Features (Next 6 months) - Gemini Elixir Streaming Architecture","doc":"#### **Circuit Breaker Pattern**\n```elixir\ndefmodule Gemini.CircuitBreaker do\n  use GenServer\n  \n  @type state :: :closed | :open | :half_open\n  \n  def check_availability(endpoint) do\n    case get_state(endpoint) do\n      :closed -> :available\n      :open -> {:unavailable, get_retry_after(endpoint)}\n      :half_open -> :probe_allowed\n    end\n  end\n  \n  def record_success(endpoint) do\n    GenServer.cast(__MODULE__, {:success, endpoint})\n  end\n  \n  def record_failure(endpoint, error) do\n    GenServer.cast(__MODULE__, {:failure, endpoint, error})\n  end\n  \n  # State transitions based on failure rates\n  def handle_cast({:failure, endpoint, error}, state) do\n    failure_count = increment_failure_count(endpoint)\n    \n    if failure_count >= failure_threshold() do\n      transition_to_open(endpoint)\n      schedule_half_open_probe(endpoint)\n    end\n    \n    {:noreply, state}\n  end\nend\n```\n\n#### **Stream Persistence and Replay**\n```elixir\ndefmodule Gemini.StreamPersistence do\n  # Persist stream events for replay capability\n  def persist_event(stream_id, event) do\n    :ets.insert(:stream_events, {{stream_id, event.sequence}, event})\n  end\n  \n  def replay_stream(stream_id, from_sequence \\\\ 0) do\n    :ets.select(:stream_events, [\n      {{{stream_id, :\"$1\"}, :\"$2\"}, [{:>=, :\"$1\", from_sequence}], [:\"$2\"]}\n    ])\n    |> Enum.sort_by(& &1.sequence)\n  end\n  \n  # Subscriber can request replay from specific point\n  def subscribe_with_replay(stream_id, subscriber_pid, from_sequence \\\\ 0) do\n    # Send historical events first\n    historical_events = replay_stream(stream_id, from_sequence)\n    Enum.each(historical_events, fn event ->\n      send(subscriber_pid, {:stream_event, stream_id, event})\n    end)\n    \n    # Then subscribe to live events\n    ManagerV2.subscribe_stream(stream_id, subscriber_pid)\n  end\nend\n```\n\n#### **Advanced Error Recovery**\n```elixir\ndefmodule Gemini.ErrorRecovery do\n  # Intelligent retry with different strategies\n  def determine_retry_strategy(error) do\n    case error do\n      %{type: :network_error, details: %{reason: :timeout}} ->\n        {:exponential_backoff, max_retries: 5}\n      \n      %{type: :api_error, http_status: 429} ->\n        {:rate_limit_backoff, respect_retry_after: true}\n      \n      %{type: :api_error, http_status: 503} ->\n        {:circuit_breaker, endpoint_cooldown: 300_000}\n      \n      %{type: :parse_error} ->\n        {:skip_and_continue, log_level: :warning}\n      \n      _ ->\n        {:fail_fast, notify_subscribers: true}\n    end\n  end\n  \n  # Fallback strategies\n  def apply_fallback(original_request, error) do\n    case determine_fallback_strategy(error) do\n      :non_streaming ->\n        Logger.info(\"Falling back to non-streaming generation\")\n        Gemini.generate(original_request.contents, original_request.opts)\n      \n      :different_model ->\n        fallback_opts = Keyword.put(original_request.opts, :model, \"gemini-2.0-flash\")\n        Gemini.generate(original_request.contents, fallback_opts)\n      \n      :cached_response ->\n        lookup_cached_response(original_request)\n      \n      :no_fallback ->\n        {:error, error}\n    end\n  end\nend\n```","ref":"streaming_architecture.html#phase-2-advanced-features-next-6-months"},{"type":"extras","title":"Phase 3: Enterprise Features (Next 12 months) - Gemini Elixir Streaming Architecture","doc":"#### **Multi-region Support**\n```elixir\ndefmodule Gemini.MultiRegion do\n  # Route requests to different regional endpoints\n  def select_endpoint(opts) do\n    preferred_region = Keyword.get(opts, :region, :auto)\n    \n    case preferred_region do\n      :auto -> select_fastest_endpoint()\n      region -> get_regional_endpoint(region)\n    end\n  end\n  \n  def select_fastest_endpoint do\n    endpoints = [\n      \"us-central1-aiplatform.googleapis.com\",\n      \"europe-west4-aiplatform.googleapis.com\", \n      \"asia-southeast1-aiplatform.googleapis.com\"\n    ]\n    \n    # Health check and latency testing\n    endpoints\n    |> Enum.map(&measure_latency/1)\n    |> Enum.min_by(& &1.latency)\n    |> Map.get(:endpoint)\n  end\n  \n  # Automatic failover between regions\n  def handle_regional_failure(failed_endpoint, request) do\n    alternative_endpoints = get_alternative_endpoints(failed_endpoint)\n    \n    Enum.reduce_while(alternative_endpoints, {:error, :all_regions_failed}, fn endpoint, _acc ->\n      case execute_request(request, endpoint) do\n        {:ok, result} -> {:halt, {:ok, result}}\n        {:error, _} -> {:cont, {:error, :region_failed}}\n      end\n    end)\n  end\nend\n```\n\n#### **WebSocket Support**\n```elixir\ndefmodule Gemini.WebSocket do\n  use WebSockex\n  \n  # Bidirectional streaming over WebSocket\n  def start_link(auth_config, opts \\\\ []) do\n    ws_url = build_websocket_url(auth_config)\n    WebSockex.start_link(ws_url, __MODULE__, %{auth: auth_config}, opts)\n  end\n  \n  def handle_frame({:text, message}, state) do\n    case Jason.decode(message) do\n      {:ok, %{\"type\" => \"stream_event\", \"data\" => data}} ->\n        notify_subscribers(state, data)\n      \n      {:ok, %{\"type\" => \"control\", \"action\" => \"pause\"}} ->\n        # Handle flow control messages\n        {:ok, %{state | paused: true}}\n      \n      {:error, _} ->\n        Logger.warning(\"Invalid WebSocket message: #{message}\")\n    end\n    \n    {:ok, state}\n  end\n  \n  # Send flow control messages\n  def send_backpressure_signal(pid, signal) do\n    message = Jason.encode!(%{type: \"control\", action: signal})\n    WebSockex.send_frame(pid, {:text, message})\n  end\nend\n```\n\n#### **Advanced Analytics**\n```elixir\ndefmodule Gemini.Analytics do\n  use GenServer\n  \n  # Track detailed stream analytics\n  def record_stream_metrics(stream_id, metrics) do\n    GenServer.cast(__MODULE__, {:record_metrics, stream_id, metrics})\n  end\n  \n  def handle_cast({:record_metrics, stream_id, metrics}, state) do\n    # Store in time-series database\n    :telemetry.execute([:gemini, :stream, :detailed], metrics, %{\n      stream_id: stream_id,\n      timestamp: System.system_time(:millisecond)\n    })\n    \n    # Update aggregated statistics\n    updated_stats = update_aggregated_stats(state.stats, metrics)\n    \n    {:noreply, %{state | stats: updated_stats}}\n  end\n  \n  # Generate insights\n  def generate_usage_report(time_range) do\n    %{\n      total_streams: count_streams_in_range(time_range),\n      success_rate: calculate_success_rate(time_range),\n      average_duration: calculate_average_duration(time_range),\n      peak_concurrent: find_peak_concurrent(time_range),\n      error_breakdown: categorize_errors(time_range),\n      cost_analysis: calculate_costs(time_range),\n      recommendations: generate_recommendations()\n    }\n  end\n  \n  defp generate_recommendations do\n    # AI-driven recommendations based on usage patterns\n    [\n      \"Consider using batch requests for short responses\",\n      \"Peak usage at 2-4 PM - consider scaling\",\n      \"Model X has 15% higher error rate - investigate\"\n    ]\n  end\nend\n```","ref":"streaming_architecture.html#phase-3-enterprise-features-next-12-months"},{"type":"extras","title":"Phase 4: Research and Innovation (Future) - Gemini Elixir Streaming Architecture","doc":"#### **Adaptive Streaming**\n```elixir\ndefmodule Gemini.AdaptiveStreaming do\n  # Automatically adjust streaming parameters based on network conditions\n  def adapt_to_network_conditions(stream_state) do\n    network_quality = measure_network_quality()\n    \n    case network_quality do\n      :excellent -> \n        %{chunk_size: :large, buffer_size: :minimal, compression: :none}\n      \n      :good -> \n        %{chunk_size: :medium, buffer_size: :small, compression: :light}\n      \n      :poor -> \n        %{chunk_size: :small, buffer_size: :large, compression: :aggressive}\n    end\n  end\n  \n  # Predict and pre-buffer content\n  def predictive_buffering(user_patterns, current_request) do\n    likely_next_requests = predict_next_requests(user_patterns, current_request)\n    \n    Enum.each(likely_next_requests, fn request ->\n      # Pre-start streams for predicted requests\n      Task.start(fn -> pre_warm_stream(request) end)\n    end)\n  end\nend\n```\n\n#### **Edge Computing Integration**\n```elixir\ndefmodule Gemini.EdgeComputing do\n  # Process streams closer to users\n  def find_edge_node(user_location) do\n    edge_nodes = [\n      %{location: \"san_francisco\", capacity: 0.3},\n      %{location: \"new_york\", capacity: 0.7},\n      %{location: \"london\", capacity: 0.5}\n    ]\n    \n    edge_nodes\n    |> Enum.filter(&(&1.capacity < 0.8))  # Available capacity\n    |> Enum.min_by(&calculate_distance(user_location, &1.location))\n  end\n  \n  # Intelligent request routing\n  def route_request(request, user_context) do\n    case determine_processing_location(request, user_context) do\n      :cloud -> process_in_cloud(request)\n      :edge -> process_at_edge(request, user_context.location)\n      :hybrid -> process_hybrid(request, user_context)\n    end\n  end\nend\n```\n\n---","ref":"streaming_architecture.html#phase-4-research-and-innovation-future"},{"type":"extras","title":"Migration Guide - Gemini Elixir Streaming Architecture","doc":"","ref":"streaming_architecture.html#migration-guide"},{"type":"extras","title":"From Current Implementation to V2 - Gemini Elixir Streaming Architecture","doc":"#### **Step 1: Parallel Deployment (Week 1-2)**\n```elixir\n# config/config.exs\nconfig :gemini,\n  streaming_version: :v1,  # Keep existing for now\n  enable_v2_preview: true  # Enable V2 for testing\n```\n\n#### **Step 2: Feature Flag Migration (Week 3-4)**\n```elixir\ndef start_stream(contents, opts, subscriber_pid) do\n  case Application.get_env(:gemini, :streaming_version) do\n    :v2 -> ManagerV2.start_stream(contents, opts, subscriber_pid)\n    :v1 -> ManagerV1.start_stream(contents, opts, subscriber_pid)  # Fallback\n  end\nend\n```\n\n#### **Step 3: Gradual Rollout (Week 5-8)**\n```elixir\n# Percentage-based rollout\ndef determine_streaming_version(user_id) do\n  hash = :erlang.phash2(user_id, 100)\n  rollout_percentage = Application.get_env(:gemini, :v2_rollout_percentage, 0)\n  \n  if hash < rollout_percentage do\n    :v2\n  else\n    :v1\n  end\nend\n```\n\n#### **Step 4: Full Migration (Week 9-10)**\n```elixir\n# Remove V1 code and feature flags\nconfig :gemini,\n  streaming_version: :v2  # Default for all users\n```","ref":"streaming_architecture.html#from-current-implementation-to-v2"},{"type":"extras","title":"Breaking Changes - Gemini Elixir Streaming Architecture","doc":"#### **API Changes**\n```elixir\n# V1 (deprecated)\n{:ok, events} = Gemini.stream_generate(contents, opts)\n\n# V2 (new)\n{:ok, stream_id} = Gemini.start_stream(contents, opts)\n# Events arrive as messages: {:stream_event, stream_id, event}\n```\n\n#### **Configuration Changes**\n```elixir\n# V1\nconfig :gemini, streaming_timeout: 30_000\n\n# V2\nconfig :gemini,\n  streaming: [\n    max_concurrent_streams: 100,\n    default_timeout: 30_000,\n    max_retries: 3\n  ]\n```","ref":"streaming_architecture.html#breaking-changes"},{"type":"extras","title":"Compatibility Layer - Gemini Elixir Streaming Architecture","doc":"```elixir\ndefmodule Gemini.Legacy do\n  @moduledoc \"\"\"\n  Compatibility layer for V1 streaming API.\n  Provides same interface but uses V2 implementation underneath.\n  \"\"\"\n  \n  def stream_generate(contents, opts \\\\ []) do\n    case ManagerV2.start_stream(contents, opts, self()) do\n      {:ok, stream_id} ->\n        events = collect_all_events(stream_id)\n        {:ok, events}\n      \n      {:error, reason} ->\n        {:error, reason}\n    end\n  end\n  \n  defp collect_all_events(stream_id, acc \\\\ []) do\n    receive do\n      {:stream_event, ^stream_id, event} ->\n        collect_all_events(stream_id, [event | acc])\n      \n      {:stream_complete, ^stream_id} ->\n        Enum.reverse(acc)\n      \n      {:stream_error, ^stream_id, error} ->\n        throw({:error, error})\n    after\n      30_000 -> throw({:error, :timeout})\n    end\n  end\nend\n```\n\n---","ref":"streaming_architecture.html#compatibility-layer"},{"type":"extras","title":"Conclusion - Gemini Elixir Streaming Architecture","doc":"This streaming implementation represents a complete architectural transformation from a fundamentally broken system to a production-ready, scalable solution. The key achievements include:\n\n### **Technical Excellence**\n- **Proper SSE Implementation**: Stateful parsing with buffer management\n- **Resource Management**: GenServer-based lifecycle with automatic cleanup\n- **Error Resilience**: Comprehensive error handling with automatic recovery\n- **Performance**: Support for 100+ concurrent streams with minimal overhead\n\n### **Production Readiness**\n- **Monitoring**: Built-in metrics, health checks, and observability\n- **Security**: Rate limiting, input validation, and resource protection\n- **Scalability**: Horizontal scaling support with connection pooling\n- **Reliability**: Circuit breakers, fallback strategies, and graceful degradation\n\n### **Developer Experience**\n- **Simple API**: Easy-to-use streaming interface\n- **Comprehensive Testing**: Unit, integration, and performance tests\n- **Clear Documentation**: Technical guides and usage examples\n- **Migration Support**: Backward compatibility and gradual rollout\n\n### **Operational Benefits**\n- **Reduced Latency**: Real-time event processing vs. polling\n- **Better Resource Utilization**: Efficient memory and CPU usage\n- **Improved Reliability**: Automatic retry and fallback mechanisms\n- **Enhanced Observability**: Detailed metrics and logging\n\nThe implementation is designed for long-term maintainability and extensibility, with clear upgrade paths for future enhancements like GenStage integration, multi-region support, and advanced analytics.\n\nThis architecture serves as a foundation for building sophisticated real-time applications with the Gemini API, supporting everything from simple chatbots to complex multi-user collaborative platforms.","ref":"streaming_architecture.html#conclusion"},{"type":"extras","title":"Gemini Streaming Architecture","doc":"# Gemini Streaming Architecture\n\nThis document provides a comprehensive overview of the streaming architecture, identifies the current concurrency issue, and proposes solutions for true real-time streaming.","ref":"streaming.html"},{"type":"extras","title":"Current Architecture Overview - Gemini Streaming Architecture","doc":"```mermaid\ngraph TD\n    %% Client Layer\n    CLI[CLI User] --> Demo[streaming_demo.exs]\n    Demo --> |start_stream| GeminiAPI[Gemini API Module]\n    \n    %% Core Streaming Components\n    GeminiAPI --> |start_stream| Manager[ManagerV2 GenServer]\n    Manager --> |stream_to_process| HTTPClient[HTTPStreaming Client]\n    HTTPClient --> |Req.request| ReqLib[Req HTTP Library]\n    ReqLib --> |SSE Connection| API[Gemini API Endpoint]\n    \n    %% Response Processing Chain\n    API --> |SSE Response| ReqLib\n    ReqLib --> |Complete Body| HTTPClient\n    HTTPClient --> |parse_chunk| Parser[SSE Parser]\n    Parser --> |Events| HTTPClient\n    HTTPClient --> |Callback| Manager\n    Manager --> |notify_subscribers| Demo\n    Demo --> |Text Output| CLI\n    \n    %% Detailed Internal Flow\n    subgraph S1 [\" \"]\n        direction TB\n        ST1[🌐 HTTP Streaming Process]\n        HTTPClient --> |spawn process| StreamProcess[Stream Process]\n        StreamProcess --> |do_stream| DoStream[Execute HTTP Request]\n        DoStream --> |Response Body| ParseBody[Parse Complete Body]\n        ParseBody --> |send messages| Manager\n    end\n    \n    subgraph S2 [\" \"]\n        direction TB\n        ST2[⚡ SSE Parser State]\n        Parser --> Buffer[String Buffer]\n        Parser --> Events[Parsed Events]\n        Buffer --> |extract_events| EventList[Event List]\n        EventList --> |parse_event| JSONParse[JSON Parsing]\n    end\n    \n    subgraph S3 [\" \"]\n        direction TB\n        ST3[🔄 Manager State]\n        Manager --> StreamMap[Stream State Map]\n        Manager --> Subscribers[Subscriber List]\n        StreamMap --> StreamState[Individual Stream State]\n        StreamState --> Status[Status: starting/active/complete]\n    end\n    \n    %% Styling\n    classDef primary fill:#6B46C1,stroke:#4C1D95,stroke-width:3px,color:#FFFFFF\n    classDef secondary fill:#9333EA,stroke:#6B21A8,stroke-width:2px,color:#FFFFFF\n    classDef tertiary fill:#A855F7,stroke:#7C2D12,stroke-width:2px,color:#FFFFFF\n    classDef api fill:#EF4444,stroke:#B91C1C,stroke-width:2px,color:#FFFFFF\n    classDef subscriber fill:#10B981,stroke:#047857,stroke-width:2px,color:#FFFFFF\n    classDef subgraphTitle fill:#1E1B4B,stroke:#312E81,stroke-width:2px,color:#FFFFFF\n    classDef problem fill:#DC2626,stroke:#991B1B,stroke-width:3px,color:#FFFFFF\n    classDef solution fill:#059669,stroke:#047857,stroke-width:2px,color:#FFFFFF\n    classDef normal fill:#3B82F6,stroke:#1D4ED8,stroke-width:2px,color:#FFFFFF\n    \n    %% Apply classes\n    class CLI,Demo primary\n    class GeminiAPI,Manager primary\n    class HTTPClient,ReqLib secondary\n    class Parser,Buffer,Events,EventList,JSONParse tertiary\n    class API api\n    class StreamProcess normal\n    class DoStream,ParseBody problem\n    class StreamMap,Subscribers,StreamState,Status solution\n    class ST1,ST2,ST3 subgraphTitle\n    \n    %% Subgraph styling\n    style S1 fill:#F9FAFB,stroke:#9333EA,stroke-width:3px\n    style S2 fill:#FEFEFE,stroke:#A855F7,stroke-width:3px\n    style S3 fill:#F3F4F6,stroke:#059669,stroke-width:3px\n```","ref":"streaming.html#current-architecture-overview"},{"type":"extras","title":"The Core Concurrency Problem - Gemini Streaming Architecture","doc":"","ref":"streaming.html#the-core-concurrency-problem"},{"type":"extras","title":"Issue Identification - Gemini Streaming Architecture","doc":"The current implementation has a **fundamental concurrency flaw** that prevents true real-time streaming:\n\n1. **HTTP Request Blocking**: The `Req.request()` call waits for the **complete response body** before returning\n2. **Batch Event Processing**: All SSE events are parsed **after** the HTTP response completes\n3. **Delayed Notification**: Events are sent to subscribers only **after all data is received**","ref":"streaming.html#issue-identification"},{"type":"extras","title":"Current Flow (Problematic) - Gemini Streaming Architecture","doc":"```mermaid\nsequenceDiagram\n    participant CLI\n    participant Manager\n    participant HTTPClient\n    participant Req\n    participant API\n    \n    CLI->>Manager: start_stream(\"prompt\")\n    Manager->>HTTPClient: stream_to_process(url, headers, body)\n    HTTPClient->>+Req: request(opts)\n    \n    Note over Req,API: HTTP Connection Established\n    API->>Req: data: {\"text\": \"Hello\"}\n    API->>Req: data: {\"text\": \" world\"}  \n    API->>Req: data: {\"text\": \"!\"}\n    API->>Req: Connection closes\n    \n    Req->>-HTTPClient: {:ok, %Response{body: \"complete_sse_data\"}}\n    HTTPClient->>HTTPClient: parse_chunk(complete_body)\n    HTTPClient->>Manager: callback(event1)\n    HTTPClient->>Manager: callback(event2) \n    HTTPClient->>Manager: callback(event3)\n    Manager->>CLI: stream_event(event1)\n    Manager->>CLI: stream_event(event2)\n    Manager->>CLI: stream_event(event3)\n    Manager->>CLI: stream_complete\n    \n    Note over CLI: All text appears at once!\n```","ref":"streaming.html#current-flow-problematic"},{"type":"extras","title":"Solution: True Streaming Architecture - Gemini Streaming Architecture","doc":"","ref":"streaming.html#solution-true-streaming-architecture"},{"type":"extras","title":"Proposed Real-Time Flow - Gemini Streaming Architecture","doc":"```mermaid\nsequenceDiagram\n    participant CLI\n    participant Manager\n    participant HTTPClient\n    participant StreamingReq[Streaming Req]\n    participant API\n    \n    CLI->>Manager: start_stream(\"prompt\")\n    Manager->>HTTPClient: stream_to_process(url, headers, body)\n    HTTPClient->>+StreamingReq: request with streaming callback\n    \n    Note over StreamingReq,API: HTTP Connection Established\n    API->>StreamingReq: data: {\"text\": \"Hello\"}\n    StreamingReq->>HTTPClient: handle_chunk(\"data: {\\\"text\\\": \\\"Hello\\\"}\")\n    HTTPClient->>Manager: callback(event1)\n    Manager->>CLI: stream_event(event1)\n    Note over CLI: \"Hello\" appears immediately\n    \n    API->>StreamingReq: data: {\"text\": \" world\"}\n    StreamingReq->>HTTPClient: handle_chunk(\"data: {\\\"text\\\": \\\" world\\\"}\")\n    HTTPClient->>Manager: callback(event2)\n    Manager->>CLI: stream_event(event2)\n    Note over CLI: \" world\" appears immediately\n    \n    API->>StreamingReq: data: {\"text\": \"!\"}\n    StreamingReq->>HTTPClient: handle_chunk(\"data: {\\\"text\\\": \\\"!\\\"}\")\n    HTTPClient->>Manager: callback(event3)\n    Manager->>CLI: stream_event(event3)\n    Note over CLI: \"!\" appears immediately\n    \n    API->>StreamingReq: Connection closes\n    StreamingReq->>HTTPClient: handle_done()\n    HTTPClient->>Manager: completion_callback()\n    Manager->>CLI: stream_complete\n```","ref":"streaming.html#proposed-real-time-flow"},{"type":"extras","title":"Required Implementation Changes - Gemini Streaming Architecture","doc":"","ref":"streaming.html#required-implementation-changes"},{"type":"extras","title":"1. Fix HTTP Streaming Layer - Gemini Streaming Architecture","doc":"**Current Implementation (Broken)**:\n```elixir\n# lib/gemini/client/http_streaming.ex - CURRENT\ncase Req.request(req_opts) do\n  {:ok, %Req.Response{status: status, body: body}} when status in 200..299 ->\n    # Problem: body contains COMPLETE response\n    case Parser.parse_chunk(body, parser) do\n      {:ok, events, _final_parser} ->\n        # All events processed at once!\n```\n\n**Required Implementation (Real-Time)**:\n```elixir\n# lib/gemini/client/http_streaming.ex - FIXED\nreq_opts = [\n  method: :post,\n  url: url,\n  headers: add_sse_headers(headers),\n  json: body,\n  receive_timeout: timeout,\n  # KEY CHANGE: Stream chunks as they arrive\n  into: fn \n    {:status, status}, acc -> \n      %{acc | status: status}\n    {:headers, headers}, acc -> \n      %{acc | headers: headers}\n    {:data, chunk}, acc -> \n      # Process each chunk immediately!\n      handle_streaming_chunk(chunk, acc, callback)\n    :done, acc -> \n      finalize_stream(acc, callback)\n  end\n]\n```","ref":"streaming.html#1-fix-http-streaming-layer"},{"type":"extras","title":"2. Implement Chunk-by-Chunk Processing - Gemini Streaming Architecture","doc":"```elixir\ndefp handle_streaming_chunk(chunk, %{parser: parser} = acc, callback) do\n  case Parser.parse_chunk(chunk, parser) do\n    {:ok, events, new_parser} ->\n      # Send events immediately as they're parsed\n      Enum.each(events, fn event ->\n        stream_event = %{type: :data, data: event.data, error: nil}\n        callback.(stream_event)\n      end)\n      \n      %{acc | parser: new_parser}\n      \n    {:error, error} ->\n      error_event = %{type: :error, data: nil, error: error}\n      callback.(error_event)\n      acc\n  end\nend\n```","ref":"streaming.html#2-implement-chunk-by-chunk-processing"},{"type":"extras","title":"3. Enhanced SSE Parser for Streaming - Gemini Streaming Architecture","doc":"The current parser already supports stateful chunk processing, but we need to ensure it handles partial events correctly:\n\n```elixir\n# lib/gemini/sse/parser.ex - Verify this works correctly\ndefp extract_events(data) do\n  # Handle both \\r\\n\\r\\n and \\n\\n patterns (FIXED)\n  parts = String.split(data, ~r/\\r?\\n\\r?\\n/)\n  \n  case parts do\n    [] -> {[], \"\"}\n    [single_part] -> {[], single_part}  # No complete events\n    multiple_parts ->\n      {complete_events, [remaining]} = Enum.split(multiple_parts, -1)\n      filtered_events = Enum.filter(complete_events, &(&1 != \"\"))\n      trimmed_remaining = String.trim(remaining)\n      {filtered_events, trimmed_remaining}\n  end\nend\n```","ref":"streaming.html#3-enhanced-sse-parser-for-streaming"},{"type":"extras","title":"4. Manager State Improvements - Gemini Streaming Architecture","doc":"Ensure the manager can handle rapid event processing:\n\n```elixir\n# lib/gemini/streaming/manager_v2.ex\ndef handle_info({:stream_event, stream_id, event}, state) do\n  case Map.get(state.streams, stream_id) do\n    nil ->\n      Logger.warning(\"Received event for unknown stream: #{stream_id}\")\n      {:noreply, state}\n\n    stream_state ->\n      # Update stream state\n      updated_stream = %{\n        stream_state\n        | events_count: stream_state.events_count + 1,\n          last_event_at: DateTime.utc_now(),\n          status: :active\n      }\n\n      # Notify subscribers IMMEDIATELY - no batching\n      notify_subscribers_immediately(updated_stream.subscribers, {:stream_event, stream_id, event})\n\n      new_state = put_in(state.streams[stream_id], updated_stream)\n      {:noreply, new_state}\n  end\nend\n\n# Ensure immediate notification\ndefp notify_subscribers_immediately(subscribers, message) do\n  Enum.each(subscribers, fn {pid, _ref} ->\n    send(pid, message)  # Immediate send, no batching\n  end)\nend\n```","ref":"streaming.html#4-manager-state-improvements"},{"type":"extras","title":"Testing the Fix - Gemini Streaming Architecture","doc":"","ref":"streaming.html#testing-the-fix"},{"type":"extras","title":"1. Update Demo to Show Timing - Gemini Streaming Architecture","doc":"```elixir\n# streaming_demo.exs - Add timing information\ndefp listen_for_events do\n  start_time = System.monotonic_time(:millisecond)\n  \n  receive do\n    {:stream_event, _stream_id, %{type: :data, data: data}} ->\n      current_time = System.monotonic_time(:millisecond)\n      elapsed = current_time - start_time\n      \n      text_content = extract_text_from_stream_data(data)\n      if text_content && text_content != \"\" do\n        IO.write(\"[#{elapsed}ms] #{text_content}\")  # Show timing\n      end\n      listen_for_events()\n      \n    {:stream_complete, _stream_id} ->\n      IO.puts(\"\\n\\n✅ Stream completed!\")\n  after\n    30_000 ->\n      IO.puts(\"\\n⏰ Stream timeout after 30 seconds\")\n  end\nend\n```","ref":"streaming.html#1-update-demo-to-show-timing"},{"type":"extras","title":"2. Performance Verification - Gemini Streaming Architecture","doc":"```elixir\n# Test script to verify real-time behavior\ndefmodule StreamingTest do\n  def test_realtime_performance do\n    start_time = System.monotonic_time(:millisecond)\n    event_times = []\n    \n    {:ok, stream_id} = Gemini.start_stream(\"Count slowly from 1 to 10\")\n    :ok = Gemini.subscribe_stream(stream_id)\n    \n    receive_loop(start_time, event_times)\n  end\n  \n  defp receive_loop(start_time, event_times) do\n    receive do\n      {:stream_event, _, %{type: :data}} ->\n        event_time = System.monotonic_time(:millisecond) - start_time\n        new_times = [event_time | event_times]\n        receive_loop(start_time, new_times)\n        \n      {:stream_complete, _} ->\n        final_times = Enum.reverse(event_times)\n        analyze_timing(final_times)\n    end\n  end\n  \n  defp analyze_timing(times) do\n    intervals = times |> Enum.chunk_every(2, 1, :discard) |> Enum.map(fn [a, b] -> b - a end)\n    \n    IO.puts(\"Event timing analysis:\")\n    IO.puts(\"First event: #{hd(times)}ms\")\n    IO.puts(\"Average interval: #{Enum.sum(intervals) / length(intervals)}ms\")\n    IO.puts(\"All events: #{inspect(times)}\")\n  end\nend\n```","ref":"streaming.html#2-performance-verification"},{"type":"extras","title":"Expected Behavior After Fix - Gemini Streaming Architecture","doc":"1. **Immediate First Response**: First token should appear within 500-1000ms\n2. **Progressive Display**: Each chunk should appear as it's generated, not all at once\n3. **Low Latency**: Gap between API generation and CLI display should be <50ms\n4. **No Buffering**: Text should stream continuously, not in large blocks\n\nThe current issue where \"all text dumps out at the end\" should be completely resolved with this streaming implementation.","ref":"streaming.html#expected-behavior-after-fix"},{"type":"extras","title":"Gemini Telemetry Implementation - Complete","doc":"# Gemini Telemetry Implementation - Complete","ref":"telemetry_implementation.html"},{"type":"extras","title":"🎉 Implementation Status: COMPLETE ✅ - Gemini Telemetry Implementation - Complete","doc":"The Gemini library now includes comprehensive telemetry instrumentation that emits events for all API operations, enabling automatic observability and integration with Foundation's telemetry system.","ref":"telemetry_implementation.html#implementation-status-complete"},{"type":"extras","title":"📊 Implementation Summary - Gemini Telemetry Implementation - Complete","doc":"","ref":"telemetry_implementation.html#implementation-summary"},{"type":"extras","title":"✅ Added Dependencies - Gemini Telemetry Implementation - Complete","doc":"- Added `:telemetry, \"~> 1.2\"` to `mix.exs`","ref":"telemetry_implementation.html#added-dependencies"},{"type":"extras","title":"✅ Telemetry Events Implemented - Gemini Telemetry Implementation - Complete","doc":"#### Request Events\n- `[:gemini, :request, :start]` - Emitted when an API request begins\n- `[:gemini, :request, :stop]` - Emitted when an API request completes successfully  \n- `[:gemini, :request, :exception]` - Emitted when an API request fails\n\n#### Streaming Events\n- `[:gemini, :stream, :start]` - Emitted when a streaming request begins\n- `[:gemini, :stream, :chunk]` - Emitted when a streaming chunk is received\n- `[:gemini, :stream, :stop]` - Emitted when a streaming request completes\n- `[:gemini, :stream, :exception]` - Emitted when a streaming request fails","ref":"telemetry_implementation.html#telemetry-events-implemented"},{"type":"extras","title":"✅ Files Modified/Created - Gemini Telemetry Implementation - Complete","doc":"#### New Files\n- `lib/gemini/telemetry.ex` - Core telemetry helper module\n- `test/gemini/telemetry_test.exs` - Comprehensive test suite\n- `telemetry_demo.exs` - Demonstration script\n- `simple_telemetry_test.exs` - Simple test script\n- `TELEMETRY_IMPLEMENTATION.md` - This documentation\n\n#### Modified Files\n- `mix.exs` - Added telemetry dependency\n- `lib/gemini/config.ex` - Added telemetry configuration support\n- `lib/gemini/client/http.ex` - Instrumented HTTP requests\n- `lib/gemini/client/http_streaming.ex` - Instrumented streaming requests  \n- `lib/gemini/generate.ex` - Enhanced with telemetry metadata","ref":"telemetry_implementation.html#files-modified-created"},{"type":"extras","title":"✅ Key Features Implemented - Gemini Telemetry Implementation - Complete","doc":"#### 1. **Configurable Telemetry**\n```elixir\n# Enable/disable telemetry (default: enabled)\nconfig :gemini, telemetry_enabled: true\n\n# Check status\nGemini.Config.telemetry_enabled?()\n```\n\n#### 2. **Rich Metadata**\nEvery telemetry event includes:\n- `url` - API endpoint URL\n- `method` - HTTP method (:post, :get, etc.)\n- `model` - Gemini model being used\n- `function` - High-level function (:generate_content, :stream_generate, etc.)\n- `contents_type` - Content type (:text, :multimodal, :unknown)\n- `stream_id` - Unique ID for streaming requests\n- `system_time` - System timestamp\n\n#### 3. **Performance Measurements**\n- `duration` - Request duration in milliseconds\n- `status` - HTTP status code\n- `chunk_size` - Size of streaming chunks in bytes\n- `total_chunks` - Total chunks in streaming requests\n- `total_duration` - Total streaming duration\n\n#### 4. **Error Tracking**\n- `reason` - Exception or error details for failed requests\n\n#### 5. **Content Classification**\n- Automatically classifies content as `:text`, `:multimodal`, or `:unknown`\n- Enables content-type specific analytics","ref":"telemetry_implementation.html#key-features-implemented"},{"type":"extras","title":"🧪 Testing - Gemini Telemetry Implementation - Complete","doc":"","ref":"telemetry_implementation.html#testing"},{"type":"extras","title":"Unit Tests - Gemini Telemetry Implementation - Complete","doc":"```bash\n# Run telemetry-specific tests\nmix test test/gemini/telemetry_test.exs\n\n# All tests pass\nmix test\n```","ref":"telemetry_implementation.html#unit-tests"},{"type":"extras","title":"Live API Testing - Gemini Telemetry Implementation - Complete","doc":"```bash\n# Set up authentication\nexport GEMINI_API_KEY=\"your-api-key\"\n\n# Run live tests with telemetry events\nmix test test/live_api_test.exs --include live_api\n```","ref":"telemetry_implementation.html#live-api-testing"},{"type":"extras","title":"Interactive Testing - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# Start IEx session\niex -S mix\n\n# Test configuration\nGemini.Config.telemetry_enabled?()\n# => true\n\n# Test helper functions\nGemini.Telemetry.generate_stream_id()\n# => \"a1b2c3d4e5f6g7h8\"\n\nGemini.Telemetry.classify_contents(\"Hello world\")\n# => :text\n\n# Attach telemetry handler to see events\n:telemetry.attach(\"demo\", [:gemini, :request, :start], fn event, measurements, metadata, _ ->\n  IO.inspect({event, measurements, metadata})\nend, nil)\n\n# Make API call (with valid key) to see telemetry events\nGemini.generate(\"Hello world\")\n```","ref":"telemetry_implementation.html#interactive-testing"},{"type":"extras","title":"🔗 Foundation Integration - Gemini Telemetry Implementation - Complete","doc":"With this implementation, the Foundation library's `Foundation.Integrations.GeminiAdapter` can now automatically capture all Gemini telemetry events:","ref":"telemetry_implementation.html#foundation-integration"},{"type":"extras","title":"Expected Telemetry Flow - Gemini Telemetry Implementation - Complete","doc":"1. **Gemini Library** → Emits telemetry events\n2. **Foundation Adapter** → Captures events automatically  \n3. **Foundation Events** → Stores in event system\n4. **Monitoring/Analytics** → Process stored events","ref":"telemetry_implementation.html#expected-telemetry-flow"},{"type":"extras","title":"Example Event Output - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# Foundation will capture events like:\n[\n  %Foundation.Event{\n    type: :gemini_request_start, \n    data: %{\n      model: \"gemini-2.0-flash\", \n      function: :generate_content,\n      contents_type: :text,\n      url: \"https://generativelanguage.googleapis.com/...\",\n      system_time: 1704067200000\n    }\n  },\n  %Foundation.Event{\n    type: :gemini_request_stop,\n    data: %{\n      duration: 1250,\n      status: 200,\n      model: \"gemini-2.0-flash\"\n    }\n  }\n]\n```","ref":"telemetry_implementation.html#example-event-output"},{"type":"extras","title":"📈 Benefits Achieved - Gemini Telemetry Implementation - Complete","doc":"","ref":"telemetry_implementation.html#benefits-achieved"},{"type":"extras","title":"1. Automatic Observability - Gemini Telemetry Implementation - Complete","doc":"- Zero-configuration telemetry for all API calls\n- Request duration and performance tracking\n- Error tracking and debugging support","ref":"telemetry_implementation.html#1-automatic-observability"},{"type":"extras","title":"2. Streaming Analytics - Gemini Telemetry Implementation - Complete","doc":"- Real-time chunk-level metrics\n- Stream completion tracking  \n- Streaming performance analysis","ref":"telemetry_implementation.html#2-streaming-analytics"},{"type":"extras","title":"3. Content-Type Analytics - Gemini Telemetry Implementation - Complete","doc":"- Automatic classification of text vs multimodal content\n- Usage pattern analysis\n- Content-specific performance metrics","ref":"telemetry_implementation.html#3-content-type-analytics"},{"type":"extras","title":"4. Foundation Integration - Gemini Telemetry Implementation - Complete","doc":"- Seamless integration with Foundation's event system\n- Automatic event storage and processing\n- No additional configuration required","ref":"telemetry_implementation.html#4-foundation-integration"},{"type":"extras","title":"5. Developer Experience - Gemini Telemetry Implementation - Complete","doc":"- Rich debugging information\n- Performance bottleneck identification\n- Production monitoring capabilities","ref":"telemetry_implementation.html#5-developer-experience"},{"type":"extras","title":"🎯 Usage Examples - Gemini Telemetry Implementation - Complete","doc":"","ref":"telemetry_implementation.html#usage-examples"},{"type":"extras","title":"Basic Request Telemetry - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# This call now emits telemetry events automatically\n{:ok, response} = Gemini.generate(\"Explain quantum physics\")\n\n# Events emitted:\n# 1. [:gemini, :request, :start] - with metadata\n# 2. [:gemini, :request, :stop] - with duration and status\n```","ref":"telemetry_implementation.html#basic-request-telemetry"},{"type":"extras","title":"Streaming Telemetry - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# Streaming calls emit additional events\nGemini.stream_generate(\"Write a story\") do |chunk|\n  # Events emitted for each chunk:\n  # [:gemini, :stream, :chunk] - with chunk size\nend\n\n# Final event:\n# [:gemini, :stream, :stop] - with total duration and chunk count\n```","ref":"telemetry_implementation.html#streaming-telemetry"},{"type":"extras","title":"Error Telemetry - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# Failed requests emit exception events\n{:error, reason} = Gemini.generate(\"test\", model: \"invalid-model\")\n\n# Event emitted:\n# [:gemini, :request, :exception] - with error details\n```","ref":"telemetry_implementation.html#error-telemetry"},{"type":"extras","title":"🔧 Configuration Options - Gemini Telemetry Implementation - Complete","doc":"","ref":"telemetry_implementation.html#configuration-options"},{"type":"extras","title":"Enable/Disable Telemetry - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# In config/config.exs\nconfig :gemini, telemetry_enabled: true  # default\n\n# Or at runtime\nApplication.put_env(:gemini, :telemetry_enabled, false)\n```","ref":"telemetry_implementation.html#enable-disable-telemetry"},{"type":"extras","title":"Custom Telemetry Handlers - Gemini Telemetry Implementation - Complete","doc":"```elixir\n# Attach custom handlers for specific events\n:telemetry.attach(\"my-handler\", [:gemini, :request, :stop], fn _event, measurements, metadata, _config ->\n  Logger.info(\"Request completed in #{measurements.duration}ms to #{metadata.model}\")\nend, nil)\n```","ref":"telemetry_implementation.html#custom-telemetry-handlers"},{"type":"extras","title":"✅ Verification Checklist - Gemini Telemetry Implementation - Complete","doc":"- [x] **Dependencies**: Telemetry dependency added to mix.exs\n- [x] **Configuration**: Telemetry enable/disable support\n- [x] **HTTP Client**: Request telemetry instrumentation \n- [x] **Streaming Client**: Streaming telemetry instrumentation\n- [x] **Generate Module**: High-level API telemetry integration\n- [x] **Helper Module**: Comprehensive telemetry utilities\n- [x] **Content Classification**: Automatic text/multimodal detection\n- [x] **Error Handling**: Exception telemetry support  \n- [x] **Metadata**: Rich context in all events\n- [x] **Measurements**: Performance metrics collection\n- [x] **Tests**: Comprehensive test coverage\n- [x] **Documentation**: Complete implementation docs\n- [x] **Compilation**: No errors or warnings\n- [x] **Test Suite**: All existing tests still pass","ref":"telemetry_implementation.html#verification-checklist"},{"type":"extras","title":"🚀 Ready for Foundation Integration - Gemini Telemetry Implementation - Complete","doc":"The Gemini library is now fully instrumented with telemetry and ready for integration with Foundation's `Foundation.Integrations.GeminiAdapter`. The adapter should automatically detect and capture all telemetry events without any additional configuration.","ref":"telemetry_implementation.html#ready-for-foundation-integration"},{"type":"extras","title":"Next Steps for Foundation Integration: - Gemini Telemetry Implementation - Complete","doc":"1. Deploy this version of gemini_ex\n2. Verify Foundation adapter detects telemetry events\n3. Test event storage in Foundation.Events\n4. Monitor telemetry in production\n\n---\n\n**Implementation Complete** ✅  \n**All Requirements Met** ✅  \n**Ready for Production** ✅","ref":"telemetry_implementation.html#next-steps-for-foundation-integration"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.0.1] - 2025-06-09","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"#### Core Features\n- **Dual Authentication System**: Support for both Gemini API keys and Vertex AI OAuth/Service Accounts\n- **Advanced Streaming**: Production-grade Server-Sent Events (SSE) streaming with real-time processing\n- **Comprehensive API Coverage**: Full support for Gemini API endpoints including content generation, model listing, and token counting\n- **Type Safety**: Complete TypeScript-style type definitions with runtime validation\n- **Error Handling**: Detailed error types with recovery suggestions and proper HTTP status code mapping\n- **Built-in Telemetry**: Comprehensive observability with metrics and event tracking\n- **Chat Sessions**: Multi-turn conversation management with state persistence\n- **Multimodal Support**: Text, image, audio, and video content processing\n\n#### Authentication\n- Multi-strategy authentication coordinator with automatic strategy selection\n- Environment variable and application configuration support\n- Per-request authentication override capabilities\n- Secure credential management with validation\n- Support for Google Cloud Service Account JSON files\n- OAuth2 Bearer token generation for Vertex AI\n\n#### Streaming Architecture\n- Unified streaming manager with state management\n- Real-time SSE parsing with event dispatching\n- Configurable buffer management and backpressure handling\n- Stream lifecycle management (start, pause, resume, stop)\n- Event subscription system with filtering capabilities\n- Comprehensive error recovery and retry mechanisms\n\n#### HTTP Client\n- Dual HTTP client system (standard and streaming)\n- Request/response interceptors for middleware support\n- Automatic retry logic with exponential backoff\n- Connection pooling and timeout management\n- Request validation and response parsing\n- Content-Type negotiation and encoding support\n\n#### Type System\n- Comprehensive type definitions for all API structures\n- Runtime type validation with descriptive error messages\n- Request and response schema validation\n- Content type definitions for multimodal inputs\n- Model capability and configuration types\n- Error type hierarchy with actionable information\n\n#### Configuration\n- Hierarchical configuration system (runtime > environment > application)\n- Environment variable detection and parsing\n- Application configuration validation\n- Default value management\n- Configuration hot-reloading support\n\n#### Utilities\n- Content extraction helpers\n- Response transformation utilities\n- Validation helpers\n- Debugging and logging utilities\n- Performance monitoring tools","ref":"changelog.html#added"},{"type":"extras","title":"Technical Implementation - Changelog","doc":"#### Architecture\n- Layered architecture with clear separation of concerns\n- Behavior-driven design for pluggable components\n- GenServer-based application supervision tree\n- Concurrent request processing with actor model\n- Event-driven streaming with backpressure management\n\n#### Dependencies\n- `req` ~> 0.4.0 for HTTP client functionality\n- `jason` ~> 1.4 for JSON encoding/decoding\n- `typed_struct` ~> 0.3.0 for type definitions\n- `joken` ~> 2.6 for JWT handling in Vertex AI authentication\n- `telemetry` ~> 1.2 for observability and metrics\n\n#### Development Tools\n- `ex_doc` for comprehensive documentation generation\n- `credo` for code quality analysis\n- `dialyxir` for static type analysis","ref":"changelog.html#technical-implementation"},{"type":"extras","title":"Documentation - Changelog","doc":"- Complete API reference documentation\n- Architecture documentation with Mermaid diagrams\n- Authentication system technical specification\n- Getting started guide with examples\n- Advanced usage patterns and best practices\n- Error handling and troubleshooting guide","ref":"changelog.html#documentation"},{"type":"extras","title":"Security - Changelog","doc":"- Secure credential storage and transmission\n- Input validation and sanitization\n- Rate limiting and throttling support\n- SSL/TLS enforcement for all communications\n- No sensitive data logging","ref":"changelog.html#security"},{"type":"extras","title":"Performance - Changelog","doc":"- Optimized for high-throughput scenarios\n- Memory-efficient streaming implementation\n- Connection reuse and pooling\n- Minimal latency overhead\n- Concurrent request processing\n\n[0.0.1]: https://github.com/nshkrdotcom/gemini_ex/releases/tag/v0.0.1","ref":"changelog.html#performance"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}